SystemOrganization addCategory: #'Zinc-HTTP'!
SystemOrganization addCategory: 'Zinc-HTTP-Client-Server'!
SystemOrganization addCategory: 'Zinc-HTTP-Core'!
SystemOrganization addCategory: 'Zinc-HTTP-Exceptions'!
SystemOrganization addCategory: 'Zinc-HTTP-Logging'!
SystemOrganization addCategory: 'Zinc-HTTP-Streaming'!
SystemOrganization addCategory: 'Zinc-HTTP-Support'!
SystemOrganization addCategory: 'Zinc-HTTP-Variables'!

DynamicVariable subclass: #ZnConnectionTimeout
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Variables'!

!ZnConnectionTimeout commentStamp: 'SvenVanCaekenberghe 5/9/2012 20:54' prior: 0!
I am ZnConnectionTimeout.
I am a DynamicVariable and a ProcessSpecificVariable.

I can be used to modify the global default socket IO timeout on a per process basis, for example:

ZnConnectionTimeout 
	value: 5 
	during: [ ^ ZnClient new get: 'http://zn.stfx.eu/zn/small.html' ]!

DynamicVariable subclass: #ZnCurrentServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Variables'!

!ZnCurrentServer commentStamp: 'SvenVanCaekenberghe 7/2/2015 22:44' prior: 0!
I am ZnCurrentServer.
I am a DynamicVariable and a ProcessSpecificVariable.

I hold a reference to the current ZnServer instance during HTTP processing, during the dynamic scope of #handleRequest:!

DynamicVariable subclass: #ZnMaximumEntitySize
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Variables'!

!ZnMaximumEntitySize commentStamp: '<historical>' prior: 0!
I am ZnMaximumEntitySize.
I am a DynamicVariable and a ProcessSpecificVariable.

I am used to control the maximum size of entities being read from streams.
!

DynamicVariable subclass: #ZnSignalProgress
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Variables'!

!ZnSignalProgress commentStamp: '<historical>' prior: 0!
I am ZnSignalProgress.
I am a DynamicVariable and a ProcessSpecificVariable.

I can be used to indicate that Zn related code down the call stack has to signal HTTPProgress, for example:

ZnSignalProgress 
	value: true 
	during: [ ^ ZnClient new get: 'http://zn.stfx.eu/zn/small.html' ]!

!ZnSignalProgress class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/12/2012 21:52'!
enabled
	"Answer if HTTPProgress should be signalled.
	The default is false."
	
	^ self value
		ifNil: [ false ]
		ifNotNil: [ :value | value ]! !

!ZnUrl methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 11/6/2013 14:12'!
retrieveContents
	"Download and return the resource that I refer to.
	This will typically return a String or a ByteArray.
	If something goes wrong, an exception will be signalled."
	
	" 'http://zn.stfx.eu/zn/numbers.txt' asZnUrl retrieveContents. "
	
	^ self performOperation: #retrieveContents! !

!ZnUrl methodsFor: '*zinc-http' stamp: 'SvenVanCaekenberghe 6/25/2014 17:15'!
saveContentsToFile: aFilename
	"Download and return a fileReference to the resource that I refer to.

	If something goes wrong, an exception will be signalled."
	
	" 'http://zn.stfx.eu/zn/numbers.txt' asZnUrl saveContentsToFile: Path * 'numbers.txt'. "
	
	^ self performOperation: #saveContentsToFile: with: aFilename! !

Error subclass: #ZnEntityTooLarge
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnEntityTooLarge commentStamp: 'SvenVanCaekenberghe 1/31/2012 12:35' prior: 0!
ZnEntityTooLarge is signalled when too big an entity is read from a stream.

This is a resumable exception!

!ZnEntityTooLarge methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/31/2012 12:35'!
isResumable
	"Determine whether an exception is resumable."

	^ true! !

Error subclass: #ZnHttpUnsuccessful
	instanceVariableNames: 'response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnHttpUnsuccessful commentStamp: 'SvenVanCaekenberghe 12/20/2011 12:52' prior: 0!
ZnHttpUnsuccessful is signalled when an HTTP client receives a non success response.

See ZnResponse>>#isSuccess

I hold the response that was unsuccessful.

Part of Zinc HTTP Components. !

!ZnHttpUnsuccessful class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/20/2011 12:51'!
response: response
	^ self new
		response: response;
		yourself! !

!ZnHttpUnsuccessful methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:53'!
messageText
	"Overwritten to initialiaze the message text to a standard text if it has not yet been set"
	
	^ messageText ifNil: [ messageText := self standardMessageText ]! !

!ZnHttpUnsuccessful methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:50'!
response
	^ response! !

!ZnHttpUnsuccessful methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:50'!
response: anObject
	response := anObject! !

!ZnHttpUnsuccessful methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:56'!
standardMessageText
	"Generate a standard textual description"
	
	^ String streamContents: [ :stream |
		response ifNotNil: [
			response statusLine printCodeAndReasonOn: stream ] ]! !

Error subclass: #ZnMissingHost
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnMissingHost commentStamp: 'SvenVanCaekenberghe 8/11/2011 19:37' prior: 0!
ZnMissingHost is signalled when a host should be specified but was not.

Part of Zinc HTTP Components. !

Error subclass: #ZnParseError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnParseError commentStamp: 'SvenVanCaekenberghe 2/28/2011 14:13' prior: 0!
ZnParseError is the superclass of all Zinc HTTP parsing related errors.

Part of Zinc HTTP Components.!

ZnParseError subclass: #ZnLineTooLong
	instanceVariableNames: 'limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnLineTooLong commentStamp: 'SvenVanCaekenberghe 2/28/2011 14:15' prior: 0!
ZnLineTooLong is signalled when an HTTP CRLF terminated input line exceeds a predefined limit.

Part of Zinc HTTP Components. !

!ZnLineTooLong class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:34'!
limit: limit
	^ self new 
		limit: limit; 
		yourself! !

!ZnLineTooLong methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:32'!
limit
	^ limit! !

!ZnLineTooLong methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:32'!
limit: anObject
	limit := anObject! !

ZnParseError subclass: #ZnUnknownHttpMethod
	instanceVariableNames: 'method'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknownHttpMethod commentStamp: 'SvenVanCaekenberghe 12/11/2011 18:09' prior: 0!
ZnUnknownHttpMethod is signalled when an unknown/unsupported HTTP method is found while parsing a request.

I am resumable to allow non-standard methods.

Part of Zinc HTTP Components. !

!ZnUnknownHttpMethod class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:34'!
method: method
	^ self new
		method: method;
		yourself! !

!ZnUnknownHttpMethod methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/11/2011 18:09'!
isResumable
	^ true! !

!ZnUnknownHttpMethod methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
method
	^ method! !

!ZnUnknownHttpMethod methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
method: anObject
	method := anObject! !

ZnParseError subclass: #ZnUnknownHttpStatusCode
	instanceVariableNames: 'code'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknownHttpStatusCode commentStamp: 'DamienPollet 8/4/2011 14:07' prior: 0!
ZnUnknownHttpStatusCode is signalled when an unknown/unsupported HTTP status code is found.

Part of Zinc HTTP Components. !

!ZnUnknownHttpStatusCode class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:34'!
code: code
	^ self new
		code: code;
		yourself! !

!ZnUnknownHttpStatusCode methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
code
	^ code! !

!ZnUnknownHttpStatusCode methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
code: anObject
	code := anObject! !

ZnParseError subclass: #ZnUnknownHttpVersion
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnknownHttpVersion commentStamp: 'DamienPollet 8/4/2011 14:07' prior: 0!
ZnUnknownHttpVersion is signalled when an unknown/unsupported HTTP version is found.

Part of Zinc HTTP Components. !

!ZnUnknownHttpVersion class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 2/28/2011 14:35'!
version: version
	^ self new
		version: version;
		yourself! !

!ZnUnknownHttpVersion methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
version
	^ version! !

!ZnUnknownHttpVersion methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/28/2011 14:33'!
version: anObject
	version := anObject! !

Error subclass: #ZnTooManyRedirects
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnTooManyRedirects commentStamp: 'SvenVanCaekenberghe 9/19/2011 09:59' prior: 0!
ZnTooManyRedirects is signalled when an HTTP client has been following more redirects than allowed.

Part of Zinc HTTP Components. !

!ZnTooManyRedirects methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/3/2012 15:06'!
isResumable
	"Determine whether an exception is resumable."

	^ true! !

Error subclass: #ZnUnexpectedContentType
	instanceVariableNames: 'expectedContentType actualContentType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnUnexpectedContentType commentStamp: 'SvenVanCaekenberghe 12/20/2011 12:52' prior: 0!
ZnUnexpectedContentType is signalled when a different content type (mime type) was expected.

I hold the expected and actual content types.

Part of Zinc HTTP Components. !

!ZnUnexpectedContentType class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/20/2011 12:48'!
expected: expectedMimeType actual: actualMimeType
	^ self new
		expectedContentType: expectedMimeType;
		actualContentType: actualMimeType;
		yourself! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:39'!
actualContentType
	^ actualContentType! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:39'!
actualContentType: anObject
	actualContentType := anObject! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:39'!
expectedContentType
	^ expectedContentType! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:39'!
expectedContentType: anObject
	expectedContentType := anObject! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:45'!
messageText
	"Overwritten to initialiaze the message text to a standard text if it has not yet been set"
	
	^ messageText ifNil: [ messageText := self standardMessageText ]! !

!ZnUnexpectedContentType methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2011 12:43'!
standardMessageText
	"Generate a standard textual description"
	
	^ String streamContents: [ :stream |
		self expectedContentType ifNotNil: [ :contentType |
			stream << 'expected '; print: contentType ].
		self actualContentType ifNotNil: [ :contentType |
			self expectedContentType ifNotNil: [ stream space ].
			stream << 'actual '; print: contentType ] ]! !

ZnUrlOperation subclass: #ZnHttpRetrieveContents
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnHttpRetrieveContents commentStamp: 'SvenVanCaekenberghe 6/25/2014 17:35' prior: 0!
I am ZnHttpRetrieveContents.
I am a ZnUrlOperation.

I implement the #retrieveContents URL operation for HTTP(S) urls.

  'http://zn.stfx.eu/zn/numbers.txt' asZnUrl retrieveContents.

Part of Zinc HTTP Components.
!

!ZnHttpRetrieveContents class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/6/2013 14:05'!
operation
	^ #retrieveContents! !

!ZnHttpRetrieveContents class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/6/2013 14:06'!
schemes
	^ #( #http #https )! !

!ZnHttpRetrieveContents methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/6/2013 14:08'!
performOperation
	"Download and return the HTTP(S) resource that url refers to.
	This will typically return a String or a ByteArray (see ZnClient>>#get:).
	If something goes wrong, an exception will be signalled."

	^ ZnClient new
		beOneShot;
		enforceHttpSuccess: true;
		get: self url! !

ZnUrlOperation subclass: #ZnHttpSaveContentsToFile
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnHttpSaveContentsToFile commentStamp: 'SvenVanCaekenberghe 6/25/2014 17:36' prior: 0!
I am ZnHttpSaveContentsToFile.
I am a ZnUrlOperation.

I implement the #saveContentsToFile: URL operation for HTTP(S) urls, which downloads the url resource to a file.

  'http://zn.stfx.eu/zn/numbers.txt' asZnUrl saveContentsToFile: Path * 'numbers.txt'.

Part of Zinc HTTP Components.!

!ZnHttpSaveContentsToFile class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/25/2014 17:16'!
operation
	^ #saveContentsToFile:! !

!ZnHttpSaveContentsToFile class methodsFor: 'accessing' stamp: 'HenrikSperreJohansen 6/18/2014 14:35'!
schemes
	^ #( #http #https )! !

!ZnHttpSaveContentsToFile methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 6/25/2014 17:16'!
performOperation
	"Download the HTTP(S) resource specified by url to the file in argument.
	Return a file reference on the file that's just been downloaded.
	If something goes wrong, an exception will be signalled."

	ZnClient new
		enforceHttpSuccess: true;
		url: self url;
		downloadTo: argument.
	^ argument asFileReference! !

Announcement subclass: #ZnLogEvent
	instanceVariableNames: 'timestamp id'
	classVariableNames: 'IdCounter LogEventAnnouncer'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnLogEvent commentStamp: 'SvenVanCaekenberghe 6/11/2014 14:20' prior: 0!
I am ZnLogEvent, the base class of a log events emitted by elements of the Zinc HTTP Components framework.

I add a timestamp and a simple id attribute. The id can wrap around and should only be used to distinguish between events that have the same timestamp.

ZnLogEvents are distributed as Announcement through a singleton Announcer that I maintain.

I have a small convenience API to log to the Transcript or open a simple GUI on the emitted log events.!

ZnLogEvent subclass: #ZnClientLogEvent
	instanceVariableNames: 'clientId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnClientLogEvent commentStamp: 'SvenVanCaekenberghe 6/12/2014 15:25' prior: 0!
I am ZnClientLogEvent, a ZnLogEvent generated by ZnClient.

I hold an optional clientId attribute.
!

ZnClientLogEvent subclass: #ZnClientConnectionClosedEvent
	instanceVariableNames: 'address port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnClientConnectionClosedEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 13:58' prior: 0!
ZnClientConnectionClosedEvent signals that a client side connection to address:port is closed.!

!ZnClientConnectionClosedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:49'!
address
	^ address! !

!ZnClientConnectionClosedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:49'!
address: anObject
	address := anObject! !

!ZnClientConnectionClosedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:57'!
port
	^ port! !

!ZnClientConnectionClosedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:57'!
port: anObject
	port := anObject! !

!ZnClientConnectionClosedEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Connection Closed '.
	address do: [ :each | stream print: each ] separatedBy: [ stream nextPut: $. ].
	stream nextPut: $:; print: port! !

ZnClientLogEvent subclass: #ZnClientFollowingRedirectEvent
	instanceVariableNames: 'target'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnClientFollowingRedirectEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 14:34' prior: 0!
ZnClientFollowingRedirectEvent signals that a redirect to target is being followed. How many redirects to follow, if any is configurable.!

!ZnClientFollowingRedirectEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Following Redirect '; print: target! !

!ZnClientFollowingRedirectEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:46'!
target
	^ target! !

!ZnClientFollowingRedirectEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:46'!
target: anObject
	target := anObject! !

ZnClientLogEvent subclass: #ZnClientIgnoringExceptionOnConnectionReuseEvent
	instanceVariableNames: 'exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnClientIgnoringExceptionOnConnectionReuseEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 14:00' prior: 0!
ZnClientIgnoringExceptionOnConnectionReuseEvent signals that a client connection was reused and that an exception occurred. Normally, there is one retry after that, trying to reconnect.
!

!ZnClientIgnoringExceptionOnConnectionReuseEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:47'!
exception
	^ exception! !

!ZnClientIgnoringExceptionOnConnectionReuseEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:47'!
exception: anObject
	exception := anObject! !

!ZnClientIgnoringExceptionOnConnectionReuseEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Ignoring Exception On Connection Reuse '; print: exception! !

!ZnClientLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:13'!
clientId
	^ clientId! !

!ZnClientLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:13'!
clientId: anObject
	clientId := anObject! !

!ZnClientLogEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 11/6/2014 15:30'!
printContentsOn: stream
	super printContentsOn: stream.
	clientId ifNotNil: [ stream space; << clientId ]! !

ZnClientLogEvent subclass: #ZnClientRetryingEvent
	instanceVariableNames: 'exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnClientRetryingEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 14:01' prior: 0!
ZnClientRetryingEvent signals a client is retrying is main operation. The number of retries is configurable.
!

!ZnClientRetryingEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:47'!
exception
	^ exception! !

!ZnClientRetryingEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:47'!
exception: anObject
	exception := anObject! !

!ZnClientRetryingEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Retrying '; print: exception! !

ZnClientLogEvent subclass: #ZnClientTransactionEvent
	instanceVariableNames: 'request response requestDuration responseDuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnClientTransactionEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 14:09' prior: 0!
ZnClientTransactionEvent is a ZnClientLogEvent that notes the completion of a client side HTTP transaction (request/response).

Instance Variables
	request: a ZnRequest
	requestDuration:	 milliseconds
	response:	a ZnResponse
	responseDuration: milliseconds
	
Note that this can be quite large since it includes the entities transferred. See ZnSimplifiedClientTransactionEvent for a more compact representation.!

!ZnClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2014 14:12'!
duration
	^ requestDuration + responseDuration! !

!ZnClientTransactionEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream space.
	request requestLine printMethodAndUriOn: stream.
	stream space; print: response code.
	response hasEntity
		ifTrue: [ 
			stream space; print: response entity contentLength; nextPut: $B ].
	stream space; print: self duration; nextPutAll: 'ms'! !

!ZnClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
request
	^ request! !

!ZnClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
request: anObject
	request := anObject! !

!ZnClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2014 14:12'!
requestDuration
	^ requestDuration! !

!ZnClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2014 14:12'!
requestDuration: anObject
	requestDuration := anObject! !

!ZnClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
response
	^ response! !

!ZnClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
response: anObject
	response := anObject! !

!ZnClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2014 14:12'!
responseDuration
	^ responseDuration! !

!ZnClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2014 14:12'!
responseDuration: anObject
	responseDuration := anObject! !

ZnClientLogEvent subclass: #ZnConnectionEstablishedEvent
	instanceVariableNames: 'hostname port address duration proxy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnConnectionEstablishedEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 14:35' prior: 0!
ZnConnectionEstablishedEvent signals that a client HTTP connection to hostname:port (address:port) was established in duration milliseconds. This time includes DNS resolving.!

!ZnConnectionEstablishedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:59'!
address
	^ address! !

!ZnConnectionEstablishedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:59'!
address: anObject
	address := anObject! !

!ZnConnectionEstablishedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:59'!
duration
	^ duration! !

!ZnConnectionEstablishedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:59'!
duration: anObject
	duration := anObject! !

!ZnConnectionEstablishedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:59'!
hostname
	^ hostname! !

!ZnConnectionEstablishedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:59'!
hostname: anObject
	hostname := anObject! !

!ZnConnectionEstablishedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:59'!
port
	^ port! !

!ZnConnectionEstablishedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:59'!
port: anObject
	port := anObject! !

!ZnConnectionEstablishedEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 10/23/2015 16:40'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Connection Established '; nextPutAll: hostname; nextPut: $:; print: port; space. 
	proxy ifNotNil: [ stream << 'via proxy '; print: proxy; space ].
	address do: [ :each | stream print: each ] separatedBy: [ stream nextPut: $. ].
	stream space; print: duration; << 'ms '.! !

!ZnConnectionEstablishedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/23/2015 16:34'!
proxy
	^ proxy! !

!ZnConnectionEstablishedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/23/2015 16:34'!
proxy: anObject
	proxy := anObject! !

ZnClientLogEvent subclass: #ZnRequestWrittenEvent
	instanceVariableNames: 'request duration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnRequestWrittenEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 14:05' prior: 0!
ZnRequestWrittenEvent signals that a request was written in duration milliseconds.!

!ZnRequestWrittenEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
duration
	^ duration! !

!ZnRequestWrittenEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
duration: anObject
	duration := anObject! !

!ZnRequestWrittenEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Request Written '; print: request; space; print: duration; << 'ms'! !

!ZnRequestWrittenEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
request
	^ request! !

!ZnRequestWrittenEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
request: anObject
	request := anObject! !

ZnClientLogEvent subclass: #ZnResponseReadEvent
	instanceVariableNames: 'response duration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnResponseReadEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 14:06' prior: 0!
ZnResponseReadEvent signals that a response was read in duration milliseconds. This time includes all waiting, server processing and networking.!

!ZnResponseReadEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
duration
	^ duration! !

!ZnResponseReadEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
duration: anObject
	duration := anObject! !

!ZnResponseReadEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Response Read '; print: response; space; print: duration; << 'ms'! !

!ZnResponseReadEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
response
	^ response! !

!ZnResponseReadEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 16:23'!
response: anObject
	response := anObject! !

ZnClientLogEvent subclass: #ZnSimplifiedClientTransactionEvent
	instanceVariableNames: 'method url response size duration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnSimplifiedClientTransactionEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:18' prior: 0!
ZnSimplifiedClientTransactionEvent is a ZnClientLogEvent that notes the completion of a client side HTTP transaction (request/response). Only a limited number of simpler data is held. ZnClientTransactionEvent contains much more detailed data.

Instance Variables
	method: HTTP verb
	url: ZnUrl requested
	response:	numeric response code
	size: number of bytes in response
	duration: milliseconds

ZnSimplifiedClientTransactionEvent is ideal for classic HTTP logging with little overhead through its default string representation.!

!ZnSimplifiedClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2014 19:38'!
duration
	^ duration! !

!ZnSimplifiedClientTransactionEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/12/2014 19:37'!
initialize
	super initialize.
	duration := 0! !

!ZnSimplifiedClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2014 19:38'!
method
	^ method! !

!ZnSimplifiedClientTransactionEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream space; nextPutAll: method.
	stream space.
	url printPathQueryFragmentOn: stream.
	stream space; print: response.
	size ifNotNil: [ 
		stream space; print: size; nextPut: $B ].
	stream space; print: duration; nextPutAll: 'ms'! !

!ZnSimplifiedClientTransactionEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/12/2014 19:40'!
request: aZnRequest
	method := aZnRequest method.
	url := aZnRequest url! !

!ZnSimplifiedClientTransactionEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/12/2014 19:41'!
requestDuration: requestDuration
	duration := duration + requestDuration ! !

!ZnSimplifiedClientTransactionEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/12/2014 19:44'!
response: aZnResponse
	response := aZnResponse code.
	aZnResponse hasEntity
		ifTrue: [ size := aZnResponse entity contentLength ]! !

!ZnSimplifiedClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2014 19:39'!
responseCode
	^ response! !

!ZnSimplifiedClientTransactionEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/12/2014 19:41'!
responseDuration: responseDuration
	duration := duration + responseDuration ! !

!ZnSimplifiedClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:06'!
size
	^ size! !

!ZnSimplifiedClientTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/12/2014 19:38'!
url
	^ url! !

!ZnLogEvent class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 13:58'!
announcer
	^ LogEventAnnouncer ifNil: [ LogEventAnnouncer := Announcer new ]! !

!ZnLogEvent class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 6/11/2014 13:57'!
initialize
	IdCounter := 0.
	Smalltalk addToStartUpList: self! !

!ZnLogEvent class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 2/11/2015 15:23'!
logToTranscript
	self stopLoggingToTranscript.
	^ self announcer when: ZnLogEvent do: [ :event | Transcript crShow: event ].! !

!ZnLogEvent class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 13:57'!
nextId
	"This should be thread safe because SmallInteger 
	arithmetic primitives cannot be interrupted"
	
	^ IdCounter := IdCounter + 1! !

!ZnLogEvent class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 6/11/2014 13:57'!
open
	^ self announcer open! !

!ZnLogEvent class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 6/11/2014 13:57'!
startUp
	"Reset the id counter"
	
	IdCounter := 0! !

!ZnLogEvent class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 3/18/2015 11:45'!
stopLoggingToTranscript
	self announcer unsubscribe: self! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 13:58'!
announcer
	^ self class announcer! !

!ZnLogEvent methodsFor: 'actions' stamp: 'SvenVanCaekenberghe 6/11/2014 13:58'!
emit
	self announcer announce: self! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 13:58'!
id
	^ id! !

!ZnLogEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 7/1/2015 15:14'!
initialize
	super initialize.
	timestamp := DateAndTime now.
	id := self nextId! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 14:04'!
nextId
	^ self class nextId! !

!ZnLogEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 16:52'!
printContentsOn: stream
	"Subclasses should implement this to add output"! !

!ZnLogEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 16:52'!
printOn: stream
	timestamp printYMDOn: stream.
	stream space.
	timestamp printHMSOn: stream. 
	stream space.
	id \\ 1000 printOn: stream base: 10 length: 3 padded: true.
	self printContentsOn: stream! !

!ZnLogEvent methodsFor: 'beacon' stamp: 'SvenVanCaekenberghe 6/30/2014 17:13'!
printOneLineContentsOn: stream
	self printContentsOn: stream! !

!ZnLogEvent methodsFor: 'beacon' stamp: 'SvenVanCaekenberghe 6/30/2014 17:13'!
printOneLineOn: stream
	self printOn: stream! !

!ZnLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 13:59'!
timestamp
	^ timestamp! !

ZnLogEvent subclass: #ZnServerLogEvent
	instanceVariableNames: 'serverId processId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerLogEvent commentStamp: 'SvenVanCaekenberghe 6/12/2014 15:27' prior: 0!
I am ZnServerLogEvent, a ZnLogEvent generated by ZnServer and its subclasses.

I hold a processId attribute.
I hold an optional serverId attribute.!

ZnServerLogEvent subclass: #ZnConnectionAcceptedEvent
	instanceVariableNames: 'address'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnConnectionAcceptedEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:04' prior: 0!
ZnConnectionAcceptedEvent is signalled when an HTTP server accepts an incoming connection from a client at address.!

!ZnConnectionAcceptedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 11:45'!
address
	^ address! !

!ZnConnectionAcceptedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 11:45'!
address: anObject
	address := anObject! !

!ZnConnectionAcceptedEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Connection Accepted '.
	address do: [ :each | stream print: each ] separatedBy: [ stream nextPut: $. ]! !

ZnServerLogEvent subclass: #ZnRequestReadEvent
	instanceVariableNames: 'request duration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnRequestReadEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:07' prior: 0!
ZnRequestReadEvent is signalled when an HTTP server reads a request in duration milliseconds.

Note that the duration can be misleading: when servicing multiple requests over a kept alive connection (the default for HTTP 1.1), the read time includes any wait time (as long as it is below the timeout). The first request read should not contain wait time.!

!ZnRequestReadEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
duration
	^ duration! !

!ZnRequestReadEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
duration: anObject
	duration := anObject! !

!ZnRequestReadEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Request Read '; print: request; space; print: duration; << 'ms'! !

!ZnRequestReadEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
request
	^ request! !

!ZnRequestReadEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
request: anObject
	request := anObject! !

ZnServerLogEvent subclass: #ZnRequestResponseHandledEvent
	instanceVariableNames: 'request response duration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnRequestResponseHandledEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:09' prior: 0!
ZnRequestResponseHandledEvent signals that the handler (delegate) processed a request and produced a response in duration in milliseconds. This duration is thus the time taken by the application itself.!

!ZnRequestResponseHandledEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
duration
	^ duration! !

!ZnRequestResponseHandledEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
duration: anObject
	duration := anObject! !

!ZnRequestResponseHandledEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Request Handled '; print: request; space; print: duration; << 'ms'! !

!ZnRequestResponseHandledEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
request
	^ request! !

!ZnRequestResponseHandledEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
request: anObject
	request := anObject! !

!ZnRequestResponseHandledEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
response
	^ response! !

!ZnRequestResponseHandledEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
response: anObject
	response := anObject! !

ZnServerLogEvent subclass: #ZnResponseWrittenEvent
	instanceVariableNames: 'response duration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnResponseWrittenEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:09' prior: 0!
ZnResponseWrittenEvent signals that response was written in duration milliseconds.!

!ZnResponseWrittenEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
duration
	^ duration! !

!ZnResponseWrittenEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
duration: anObject
	duration := anObject! !

!ZnResponseWrittenEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Response Written '; print: response; space; print: duration; << 'ms'! !

!ZnResponseWrittenEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
response
	^ response! !

!ZnResponseWrittenEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:55'!
response: anObject
	response := anObject! !

ZnServerLogEvent subclass: #ZnServerConnectionClosedEvent
	instanceVariableNames: 'address'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerConnectionClosedEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:10' prior: 0!
ZnServerConnectionClosedEvent signals that the server closed a client connection to address. !

!ZnServerConnectionClosedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 13:28'!
address
	^ address! !

!ZnServerConnectionClosedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 13:28'!
address: anObject
	address := anObject! !

!ZnServerConnectionClosedEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Server Connection Closed '.
	address do: [ :each | stream print: each ] separatedBy: [ stream nextPut: $. ]! !

ZnServerLogEvent subclass: #ZnServerErrorEvent
	instanceVariableNames: 'exception'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerErrorEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:10' prior: 0!
ZnServerErrorEvent is a ZnServerLogEvent related to an exception that occurred.!

!ZnServerErrorEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:22'!
exception
	^ exception! !

!ZnServerErrorEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 14:22'!
exception: anObject
	exception := anObject! !

ZnServerErrorEvent subclass: #ZnServerHandlerErrorEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerHandlerErrorEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:12' prior: 0!
ZnServerHandlerErrorEvent signals an error that occurred during handling by the delegate. This is considered a real error.!

!ZnServerHandlerErrorEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Server Handler Error ';  print: exception! !

ZnServerErrorEvent subclass: #ZnServerReadErrorEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerReadErrorEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:13' prior: 0!
ZnServerReadErrorEvent signals an error during the reading of a request. These errors are considered mostly benign and the connection gets closed.
!

!ZnServerReadErrorEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Server Read Error ';  print: exception! !

ZnServerErrorEvent subclass: #ZnServerWriteErrorEvent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerWriteErrorEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:13' prior: 0!
ZnServerWriteErrorEvent signals an error during the writing of a response. These errors are considered mostly benign and the connection gets closed.
!

!ZnServerWriteErrorEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Server Write Error ';  print: exception! !

ZnServerLogEvent subclass: #ZnServerGenericLogEvent
	instanceVariableNames: 'subject'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerGenericLogEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:03' prior: 0!
I am ZnServerGenericLogEvent, a ZnServerLogEvent and ZnLogEvent that holds a generic subject, possibly not limited to a String.

Here are two examples:

(ZnServerGenericLogEvent subject: 'You can''t do that, Dave') emit.

(ZnServerGenericLogEvent subject: { #id->123. #reason->#timeout } asDictionary) emit.!

!ZnServerGenericLogEvent class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/16/2014 12:33'!
subject: object
	"self subject: #(i 10 j -1)."
	"self subject: 'How are you?'."
	
	^ self new
		subject: object;
		yourself! !

!ZnServerGenericLogEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 11/6/2014 15:10'!
printContentsOn: stream
	super printContentsOn: stream.
	stream space; nextPutAll: subject asString! !

!ZnServerGenericLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2014 11:39'!
subject
	^ subject! !

!ZnServerGenericLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/16/2014 11:39'!
subject: anObject
	subject := anObject! !

!ZnServerLogEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/11/2014 15:17'!
initialize
	super initialize.
	processId := ZnUtils currentProcessID! !

!ZnServerLogEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 11/6/2014 15:30'!
printContentsOn: stream
	super printContentsOn: stream.
	serverId ifNotNil: [ stream space; << serverId ].
	stream space.
	processId printOn: stream base: 10 length: 6 padded: true! !

!ZnServerLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:15'!
processId
	^ processId! !

!ZnServerLogEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/11/2014 15:15'!
processId: anObject
	processId := anObject! !

!ZnServerLogEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2014 15:15'!
serverId
	^ serverId! !

!ZnServerLogEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/11/2014 15:15'!
serverId: anObject
	serverId := anObject! !

ZnServerLogEvent subclass: #ZnServerSocketBoundEvent
	instanceVariableNames: 'address port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerSocketBoundEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:14' prior: 0!
ZnServerSocketBoundEvent signals a new server socket on address:port is bound (opened).!

!ZnServerSocketBoundEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 11:10'!
address
	^ address! !

!ZnServerSocketBoundEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 11:10'!
address: anObject
	address := anObject! !

!ZnServerSocketBoundEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 11:10'!
port
	^ port! !

!ZnServerSocketBoundEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 11:10'!
port: anObject
	port := anObject! !

!ZnServerSocketBoundEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Server Socket Bound '.
	address do: [ :each | stream print: each ] separatedBy: [ stream nextPut: $. ].
	stream nextPut: $:; print: port
	! !

ZnServerLogEvent subclass: #ZnServerSocketReleasedEvent
	instanceVariableNames: 'address port'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerSocketReleasedEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:14' prior: 0!
ZnServerSocketReleasedEvent signals a new server socket on address:port is released (closed).!

!ZnServerSocketReleasedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 13:32'!
address
	^ address! !

!ZnServerSocketReleasedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 13:32'!
address: anObject
	address := anObject! !

!ZnServerSocketReleasedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 13:32'!
port
	^ port! !

!ZnServerSocketReleasedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 13:32'!
port: anObject
	port := anObject! !

!ZnServerSocketReleasedEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Server Socket Released '.
	address do: [ :each | stream print: each ] separatedBy: [ stream nextPut: $. ].
	stream nextPut: $:; print: port! !

ZnServerLogEvent subclass: #ZnServerStartedEvent
	instanceVariableNames: 'description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerStartedEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:14' prior: 0!
ZnServerStartedEvent notes the startup of an HTTP server is complete.
!

!ZnServerStartedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 10:54'!
description
	^ description! !

!ZnServerStartedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 10:54'!
description: anObject
	description := anObject! !

!ZnServerStartedEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Started '; << description! !

ZnServerLogEvent subclass: #ZnServerStoppedEvent
	instanceVariableNames: 'description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerStoppedEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:15' prior: 0!
ZnServerStoppedEvent notes the shutdown of an HTTP server is complete.!

!ZnServerStoppedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 10:55'!
description
	^ description! !

!ZnServerStoppedEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 10:55'!
description: anObject
	description := anObject! !

!ZnServerStoppedEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream << ' Stopped '; << description! !

ZnServerLogEvent subclass: #ZnServerTransactionEvent
	instanceVariableNames: 'request response timing'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerTransactionEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:18' prior: 0!
ZnServerTransactionEvent is a ZnServerLogEvent that notes the completion of a server side HTTP transaction (request/response).

Instance Variables
	request: a ZnRequest
	response:	a ZnResponse
	timing: a ZnServerTransactionTiming

Note that this can be quite large since it includes the entities transferred. See ZnSimplifiedServerTransactionEvent for a more compact representation.!

!ZnServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:03'!
duration
	^ timing totalDuration! !

!ZnServerTransactionEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream space.
	request requestLine printMethodAndUriOn: stream.
	stream space; print: response code.
	response hasEntity
		ifTrue: [ 
			stream space; print: response entity contentLength; nextPut: $B ].
	stream space; print: self duration; nextPutAll: 'ms'! !

!ZnServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:02'!
request
	^ request! !

!ZnServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:02'!
request: anObject
	request := anObject! !

!ZnServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:02'!
response
	^ response! !

!ZnServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:02'!
response: anObject
	response := anObject! !

!ZnServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:02'!
timing
	^ timing! !

!ZnServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:02'!
timing: anObject
	timing := anObject! !

ZnServerLogEvent subclass: #ZnSimplifiedServerTransactionEvent
	instanceVariableNames: 'method url response size duration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnSimplifiedServerTransactionEvent commentStamp: 'SvenVanCaekenberghe 6/16/2014 15:20' prior: 0!
ZnSimplifiedServerTransactionEvent is a ZnServerLogEvent that notes the completion of a served side HTTP transaction (request/response). Only a limited number of simpler data is held. ZnServerTransactionEvent contains much more detailed data.

Instance Variables
	method: HTTP verb
	url: ZnUrl requested
	response:	numeric response code
	size: number of bytes in response
	duration: milliseconds

ZnSimplifiedServerTransactionEvent is ideal for classic HTTP logging with little overhead through its default string representation.
!

!ZnSimplifiedServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:58'!
duration
	^ duration! !

!ZnSimplifiedServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:58'!
method
	^ method! !

!ZnSimplifiedServerTransactionEvent methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/30/2014 17:06'!
printContentsOn: stream
	super printContentsOn: stream.
	stream space; nextPutAll: method.
	stream space.
	url printPathQueryFragmentOn: stream.
	stream space; print: response.
	size ifNotNil: [ 
		stream space; print: size; nextPut: $B ].
	stream space; print: duration; nextPutAll: 'ms'! !

!ZnSimplifiedServerTransactionEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/13/2014 16:57'!
request: aZnRequest
	method := aZnRequest method.
	url := aZnRequest url! !

!ZnSimplifiedServerTransactionEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/13/2014 16:57'!
response: aZnResponse
	response := aZnResponse code.
	aZnResponse hasEntity
		ifTrue: [ size := aZnResponse entity contentLength ]! !

!ZnSimplifiedServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:58'!
responseCode
	^ response! !

!ZnSimplifiedServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:58'!
size
	^ size! !

!ZnSimplifiedServerTransactionEvent methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/13/2014 16:56'!
timing: timing
	duration := timing totalDuration ! !

!ZnSimplifiedServerTransactionEvent methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:58'!
url
	^ url! !

Notification subclass: #ZnRespond
	instanceVariableNames: 'response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Exceptions'!

!ZnRespond commentStamp: 'SvenVanCaekenberghe 7/2/2015 23:18' prior: 0!
I am ZnRespond, a Notification to signal the end of #handleRequest: processing with a specific ZnResponse, earlier than normal stack unwinding.

  ZnRespond signalWith: ZnResponse unauthorized

Normal #handleRequest: processing in a ZnServer delegate takes a ZnRequest object as input argument and needs to produce a ZnResponse as output, returning it as a regular result. Sometimes you do not want to keep managing this single result with many levels of condition on a complex or deep stack, but instead directly want to return a response: that is when you can use the ZnRespond notification.!

!ZnRespond class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:50'!
signalWith: response
	^ self new
		response: response;
		signal! !

!ZnRespond methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/2/2015 22:47'!
response
	^ response! !

!ZnRespond methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/2/2015 22:47'!
response: anObject
	response := anObject! !

Stream subclass: #ZnBivalentWriteStream
	instanceVariableNames: 'stream binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnBivalentWriteStream commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:50' prior: 0!
I am ZnBivalentWriteStream, wrapping another write stream.

I allow both Characters and Strings as well as Integers (0 to: 255) and ByteArrays to be writtten to me.

The write stream that I wrap can be both binary as well as character based.

Part of Zinc HTTP Components.!

!ZnBivalentWriteStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:44'!
on: writeStream
	"Wrap writeStream as a ZnBivalentWriteStream, unless it is already one"
	
	(writeStream isKindOf: self) ifTrue: [ ^ writeStream ].
	
	^ self basicNew
		on: writeStream;
		yourself! !

!ZnBivalentWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/4/2010 20:07'!
close
	stream close! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/4/2012 16:06'!
flush
	stream flush! !

!ZnBivalentWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/4/2011 20:54'!
isBinary
	binary isNil
		ifTrue: [ [ binary := stream isBinary ] on: MessageNotUnderstood do: [ :err | binary := false ] ].
	^ binary! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:01'!
next: count putAll: collection startingAt: offset
	| buffer |
	self isBinary
		ifTrue: [ 
			(collection isKindOf: ByteArray)
				ifTrue: [ stream next: count putAll: collection startingAt: offset ]
				ifFalse: [ 
					buffer := ByteArray new: count.
					buffer replaceFrom: 1 to: count with: collection startingAt: offset.
					stream nextPutAll: buffer ] ]
		ifFalse: [ 
			(collection isKindOf: ByteArray)
				ifTrue: [ 
					buffer := String new: count.
					buffer replaceFrom: 1 to: count with: collection startingAt: offset.
					stream nextPutAll: buffer ]
				ifFalse: [ stream next: count putAll: collection startingAt: offset ] ]! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:54'!
nextPut: object
	self isBinary
		ifTrue: [ stream nextPut: object asInteger ]
		ifFalse: [ stream nextPut: object asCharacter ]! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:54'!
nextPutAll: collection
	self isBinary
		ifTrue: [ stream nextPutAll: collection asByteArray ]
		ifFalse: [ stream nextPutAll: collection asString ]! !

!ZnBivalentWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/4/2010 20:07'!
on: writeStream
	stream := writeStream! !

!ZnBivalentWriteStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 10/4/2010 20:08'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnBivalentWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 20:12'!
space
	self nextPut: Character space! !

Stream subclass: #ZnChunkedReadStream
	instanceVariableNames: 'stream chunk position limit atEnd totalSize chunkCount extraHeaders'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnChunkedReadStream commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:01' prior: 0!
I am ZnChunkedReadStream, implementing HTTP 1.1 chunked transfer encoding on a wrapped streams.

Clients should read me until I am atEnd.

After I am completely read, I can tell you my totalSize and optional extraHeaders.

Part of Zinc HTTP Components.!

!ZnChunkedReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/26/2010 14:10'!
new
	^ super new initialize! !

!ZnChunkedReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/26/2010 19:43'!
on: stream
	^ self basicNew
		initialize;
		on: stream;
		yourself! !

!ZnChunkedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:57'!
atEnd
	self ensureChunkOrAtEnd.
	^ atEnd! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 08:11'!
binary
	stream binary! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 00:36'!
chunkCount
	^ chunkCount! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:45'!
collectionSpecies
	^ self isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]
	! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2012 16:02'!
contents
	"This is technically not correct, but it is better than nothing"
	
	^ self upToEnd! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/19/2011 12:39'!
ensureChunkBufferOfSize: size
	(chunk notNil and: [ size <= chunk size ]) ifTrue: [ ^ self ].  
	chunk := self collectionSpecies new: size.
! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/19/2011 12:45'!
ensureChunkOrAtEnd
	atEnd 
		ifTrue: [ ^ self ].
	(chunk isNil or: [ position >= limit ]) 
		ifTrue: [ self getNextChunk ]
! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:21'!
extraHeaders
	^ extraHeaders! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/26/2010 17:08'!
getExtraHeaders
	extraHeaders := ZnHeaders readFrom: stream! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/18/2013 00:36'!
getNextChunk
	| size |
	(size := self getNextChunkSize) = 0
		ifTrue: [
			atEnd := true.
			chunk := position := limit := nil.
			self getExtraHeaders ]
		ifFalse: [
			totalSize := totalSize + size.
			chunkCount := chunkCount + 1.
			self ensureChunkBufferOfSize: size.  
			stream next: size into: chunk.
			stream next: 2. "consume CRLF"
			limit := size.
			position := 0 ]! !

!ZnChunkedReadStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 19:26'!
getNextChunkSize
	| line |
	(line := (ZnLineReader on: stream) nextLine) isEmpty ifTrue: [ ^ 0 ].
	^ Integer readFrom: (line copyUpTo: $;) base: 16! !

!ZnChunkedReadStream methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 5/18/2013 00:36'!
initialize
	atEnd := false.
	chunkCount := 0.
	totalSize := 0! !

!ZnChunkedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:45'!
isBinary
	^ stream isBinary
! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2012 16:06'!
match: subCollection
	"Try to read and match the elements of subCollection. 
	If successful return true and leave the me positioned after the match.
	If unsuccesful return false and leave me at end."
	
	| buffer pattern bufferPosition bufferLimit |
	pattern := subCollection readStream.
	"we have to use an internal buffer because we are not positionable"
	buffer := subCollection species new: subCollection size.
	bufferPosition := bufferLimit := 0.
	[ pattern atEnd ] whileFalse: [ | nextElement |
		self atEnd ifTrue: [ ^ false ].
		"get the next char to match either from the buffer or from the stream while buffering it"
		(bufferPosition < bufferLimit) 
			ifTrue: [ 
				nextElement := buffer at: bufferPosition + 1 ]
			ifFalse: [
				nextElement := buffer at: bufferPosition + 1 put: self next.
				bufferLimit := bufferLimit + 1 ].
		bufferPosition := bufferPosition + 1.
		pattern next = nextElement
			ifFalse: [
				"shift the buffer down one element and restart"
				2 to: bufferLimit do: [ :each |
					buffer at: each - 1 put: (buffer at: each) ].
				bufferLimit := bufferLimit - 1.
				bufferPosition := 0.
				pattern position: 0 ] ].
	^ true! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 19:57'!
next
	self ensureChunkOrAtEnd.
	self atEnd ifTrue: [ ^ nil ].
	^ chunk at: (position := position + 1)! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 21:11'!
next: requestedCount 
	"Read requestedCount elements into new collection and return it,
	 it could be that less elements were available"

	^ self 
		next: requestedCount 
		into: (self collectionSpecies new: requestedCount)! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 21:11'!
next: requestedCount into: collection
	"Read requestedCount elements into collection,
	returning a copy if less elements are available"
	
	^ self
		next: requestedCount
		into: collection
		startingAt: 1! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 21:11'!
next: requestedCount into: collection startingAt: offset
	"Read requestedCount elements into collection starting at offset,
	returning a copy if less elements are available"
	
	| readCount |
	readCount := self 
		readInto: collection 
		startingAt: offset 
		count: requestedCount.
	^ requestedCount = readCount
		ifTrue: [ collection ]
		ifFalse: [ collection copyFrom: 1 to: offset + readCount - 1 ]! !

!ZnChunkedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/26/2010 14:13'!
on: readStream
	stream := readStream! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/19/2011 12:36'!
peek
	self ensureChunkOrAtEnd.
	self atEnd ifTrue: [ ^ nil ].
	^ chunk at: (position + 1)! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 15:30'!
position
	^ self atEnd 
		ifTrue: [ totalSize ]
		ifFalse: [ totalSize - chunk size + position ]! !

!ZnChunkedReadStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:12'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2013 21:06'!
readInto: collection startingAt: offset count: requestedCount
	"Read requestedCount elements into collection starting at offset.
	Return the actual number of elements read."

	| read toRead |
	read := 0.
	[ read < requestedCount and: [ self atEnd not] ] whileTrue: [
		toRead := (limit - position) min: (requestedCount - read).
		collection 
			replaceFrom: offset + read 
			to: offset + read + toRead - 1 
			with: chunk 
			startingAt: position + 1.
		read := read + toRead.
		position := position + toRead ].
	^ read! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:12'!
reset
	"We don't allow it"! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 17:10'!
skip: count
	count timesRepeat: [ self next ]! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 14:20'!
totalSize
	^ totalSize! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 13:15'!
upTo: anObject 
	"We use our own collectionSpecies."
	
	^ self collectionSpecies streamContents: [ :out | | element |
		[ self atEnd or: [ (element := self next) = anObject ] ]
			whileFalse: [ out nextPut: element ] ]! !

!ZnChunkedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/17/2013 23:49'!
upToEnd
	"We use our own collectionSpecies and read by chunk"

	^ self collectionSpecies streamContents: [ :out |
		[ self atEnd ] whileFalse: [ 
			out next: limit putAll: chunk.
			position := limit ] ]! !

Stream subclass: #ZnLimitedReadStream
	instanceVariableNames: 'stream position limit binary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnLimitedReadStream commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:02' prior: 0!
I am ZnLimitedReadStream, wrapping another read stream delegating to it 
but limiting reading to a fixed number of elements.

I can be atEnd while my wrapped stream is not.

My contentSpecies can be forced to be ByteArray (binary) or ByteString (ascii).

I do byte/char conversions on the fly (support bivalent access).

This implementation is really a kludge.

Part of Zinc HTTP Components.!

!ZnLimitedReadStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/15/2010 11:15'!
on: stream limit: limit
	^ self basicNew
		on: stream limit: limit;
		yourself! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:38'!
ascii
	binary := false! !

!ZnLimitedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/31/2012 20:03'!
atEnd
	^ position >= limit or: [ stream atEnd ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 13:36'!
binary
	binary := true! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 19:34'!
collectionSpecies
	"I can be set to binary or ascii explicitely.
	If not set, delegate to our wrapped stream."
	
	^ binary notNil
		ifTrue: [ 
			binary ifTrue: [ ByteArray ] ifFalse: [ ByteString ] ]
		ifFalse: [
			(stream respondsTo: #collectionSpecies)
				ifTrue: [ stream collectionSpecies ]  
				ifFalse: [ stream isBinary ifTrue: [ ByteArray ] ifFalse: [ ByteString ] ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2012 16:02'!
contents
	"This is technically not correct, but it is better than nothing"
	
	^ self upToEnd! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 20:17'!
contentsOfEntireFile
	^ self upToEnd! !

!ZnLimitedReadStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/17/2010 20:06'!
isBinary
	^ binary isNil
		ifTrue: [ stream isBinary ]
		ifFalse: [ binary ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2012 15:55'!
match: subCollection
	"Try to read and match the elements of subCollection. 
	If successful return true and leave the me positioned after the match.
	If unsuccesful return false and leave me at end."
	
	| buffer pattern bufferPosition bufferLimit |
	pattern := subCollection readStream.
	"we have to use an internal buffer because we are not positionable"
	buffer := subCollection species new: subCollection size.
	bufferPosition := bufferLimit := 0.
	[ pattern atEnd ] whileFalse: [ | nextElement |
		self atEnd ifTrue: [ ^ false ].
		"get the next char to match either from the buffer or from the stream while buffering it"
		(bufferPosition < bufferLimit) 
			ifTrue: [ 
				nextElement := buffer at: bufferPosition + 1 ]
			ifFalse: [
				nextElement := buffer at: bufferPosition + 1 put: self next.
				bufferLimit := bufferLimit + 1 ].
		bufferPosition := bufferPosition + 1.
		pattern next = nextElement
			ifFalse: [
				"shift the buffer down one element and restart"
				2 to: bufferLimit do: [ :each |
					buffer at: each - 1 put: (buffer at: each) ].
				bufferLimit := bufferLimit - 1.
				bufferPosition := 0.
				pattern position: 0 ] ].
	^ true! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/21/2011 23:19'!
next
	"We do byte/char conversions on the fly."
	
	^ self atEnd 
		ifTrue: [ nil ] 
		ifFalse: [ 
			position := position + 1. 
			binary isNil
				ifTrue: [ stream next ] 
				ifFalse: [ 
					binary
					 	ifTrue: [ stream next asInteger ]
						ifFalse: [ stream next asCharacter ] ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 20:14'!
next: requestedCount 
	"Read requestedCount elements into new collection and return it,
	 it could be that less elements were available"

	^ self 
		next: requestedCount 
		into: (self collectionSpecies new: requestedCount)! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 20:11'!
next: requestedCount into: collection
	"Read requestedCount elements into collection,
	returning a copy if less elements are available"
	
	^ self
		next: requestedCount
		into: collection
		startingAt: 1! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 20:14'!
next: requestedCount into: collection startingAt: offset
	"Read requestedCount elements into collection starting at offset,
	returning a copy if less elements are available"
	
	| readCount |
	readCount := self 
		readInto: collection 
		startingAt: offset 
		count: requestedCount.
	^ requestedCount = readCount
		ifTrue: [ collection ]
		ifFalse: [ collection copyFrom: 1 to: offset + readCount - 1 ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/1/2012 13:35'!
nextInto: collection
	"Read the next elements of the receiver into collection,
	returning a copy if less elements are available"
	
	^ self
		next: collection size
		into: collection! !

!ZnLimitedReadStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 13:06'!
on: readStream limit: integer
	stream := readStream.
	limit := integer.
	position := 0! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/4/2011 18:43'!
peek
	"We do byte/char conversions on the fly."
	
	^ self atEnd 
		ifTrue: [ nil ] 
		ifFalse: [ 
			binary isNil
				ifTrue: [ stream peek ] 
				ifFalse: [ 
					binary
					 	ifTrue: [ stream peek asInteger ]
						ifFalse: [ stream peek asCharacter ] ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:09'!
position
	^ position! !

!ZnLimitedReadStream methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/15/2010 12:50'!
printOn: aStream
	aStream 
		nextPutAll: 'a '; 
		nextPutAll: self class name! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 21:06'!
readInto: collection startingAt: offset count: count 
	"Read count elements and place them in collection starting at offset.
	Return the number of elements actually read."
	
	| target actual |
	target := count min: (limit - position).
	actual := stream 
		readInto: collection 
		startingAt: offset 
		count: target.
	position := position + actual.
	^ actual! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/26/2010 17:20'!
reset
	"We don't allow it"! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 20:03'!
skip: count
	count timesRepeat: [ self next ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 20:02'!
upTo: anObject 
	"We use our own collectionSpecies."
	
	^ self collectionSpecies streamContents: [ :out | | element |
		[ self atEnd or: [ (element := self next) = anObject ] ] whileFalse: [ 
			out nextPut: element ] ]! !

!ZnLimitedReadStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 21:07'!
upToEnd
	^ ZnUtils readUpToEnd: self limit: nil! !

ProcessLocalVariable subclass: #ZnCurrentServerSession
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Variables'!

!ZnCurrentServerSession commentStamp: 'SvenVanCaekenberghe 7/2/2015 22:43' prior: 0!
I am ZnCurrentServerSession.
I am a ProcessLocalVariable and a ProcessSpecificVariable.

I hold a reference to the current ZnServerSession instance during HTTP processing, during the dynamic scope of #handleRequest:

ZnServerSessions are created lazily.
!

Object subclass: #ZnBasicAuthenticator
	instanceVariableNames: 'credentials realm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnBasicAuthenticator commentStamp: 'SvenVanCaekenberghe 9/30/2010 15:37' prior: 0!
I am ZnBasicAuthenticator.
I help servers handle HTTP Basic Authentication.
I implement the required #authenticateRequest:do:

I have a credentials dictionary with username->password associations.

Part of Zinc HTTP Components.!

!ZnBasicAuthenticator class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
defaultRealm
	^ 'ZincHTTPComponents'! !

!ZnBasicAuthenticator class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 15:36'!
new
	^ super new 
		initialize;
		yourself! !

!ZnBasicAuthenticator class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 15:36'!
username: username password: password
	^ self new
		atUsername: username putPassword: password;
		yourself! !

!ZnBasicAuthenticator class methodsFor: 'instance creation' stamp: 'MattKennedy 9/30/2010 15:55'!
username: username password: password realm: realm
	^ self new
		atUsername: username putPassword: password;
		realm: realm;
		yourself! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 17:19'!
atUsername: username putPassword: password
	credentials at: username put: password! !

!ZnBasicAuthenticator methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
authHeader
	^ 'Basic realm=', self realm! !

!ZnBasicAuthenticator methodsFor: 'public' stamp: 'MattKennedy 9/30/2010 15:56'!
authenticateRequest: request do: block
	^ (self isRequestAuthenticated: request)
		ifTrue: [ block value ]
		ifFalse: [ ZnResponse unauthorized: self authHeader ]! !

!ZnBasicAuthenticator methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 9/30/2010 15:37'!
initialize
	credentials := Dictionary new! !

!ZnBasicAuthenticator methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 2/23/2012 22:13'!
isRequestAuthenticated: request
	| authorization username password |
	authorization := [ request basicAuthentication ] on: Error do: [ ^ false ].
	username := authorization first.
	password := authorization second.
	^ (credentials at: username ifAbsent: [ nil ]) = password
	! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:02'!
realm
	^ realm ifNil: [ realm := self class defaultRealm ]! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'MattKennedy 9/30/2010 15:51'!
realm: anObject
	realm := anObject! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 15:41'!
removeAll
	credentials removeAll! !

!ZnBasicAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 15:41'!
removeUsername: username
	credentials removeKey: username! !

ZnBasicAuthenticator subclass: #ZnDigestAuthenticator
	instanceVariableNames: 'nonces'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnDigestAuthenticator commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:47' prior: 0!
I am ZnDigestAuthenticator.
I help servers handle HTTP Digest Authentication.

I have a nonces dictionary with nonce->opaque associations. 
I generate a new nonce value for every 401 response I trigger.

Part of Zinc HTTP Components.!

!ZnDigestAuthenticator class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
hasMD5Support
	^ (self md5Hash: 'test') isNil not! !

!ZnDigestAuthenticator class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/12/2014 12:52'!
md5Hash: aString
	| hash |
	"Answer hash of aString as lowercase 32 digit hex String."
	Smalltalk at: #MD5 ifPresent: [ :cls |
		^ ((cls new initialize) hashStream: (aString asByteArray readStream)) hex ].
	hash := Smalltalk at: #CMD5Hasher ifPresent: [ :cls |
		cls hashMessage: aString ].
	hash ifNil: [
		hash := Smalltalk at: #TCryptoRandom ifPresent: [ :cls |
			(cls basicNew perform: #md5HashMessage: with: aString) asInteger ] ].
	hash ifNotNil: [
		hash := hash hex asLowercase.
		(hash beginsWith: '16r') ifTrue: [ hash := hash allButFirst: 3 ].
		hash := hash padLeftTo: 32 with: $0 ].
	^ hash
! !

!ZnDigestAuthenticator class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/28/2015 18:15'!
parseAuthRequest: headerValue
	| data dict |
	dict := Dictionary new.
	"Chops off the ' Digest ' scheme name, a bit too brittle."
	data := headerValue copyFrom: 8 to: headerValue size.
	(data substrings: ',') do: [ :fragment | | tokens i key value |
		tokens := fragment trimBoth.
		i := tokens indexOf: $=.
		(i > 0) ifTrue: [
			key := tokens copyFrom: 1 to: i - 1.
			value := tokens copyFrom: i + 1 to: tokens size.  
			dict at: key put: (value trimBoth: [ :char | char = $" ]) ] ].
	^ dict! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:03'!
a1for: username
	| password |
	password := credentials at: username ifAbsent: [ ^ nil ].
	^ self class md5Hash: username, ':', self realm, ':', password
	! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
a2forUrl: uri method: method
	^ self class md5Hash: method, ':', uri
	! !

!ZnDigestAuthenticator methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
authHeader
	| nonce opaque |
	nonce := self createNonce.
	opaque := self createOpaque.
	self nonces at: nonce put: opaque.
	^ 'Digest realm="', self realm, '", nonce="', nonce, '", opaque="', opaque, '"'! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/10/2013 11:32'!
createNonce
	| pt |
	pt := DateAndTime now asString, ':', Random new next asString.
	^ self class md5Hash: pt! !

!ZnDigestAuthenticator methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/10/2013 11:32'!
createOpaque
	| pt |
	pt := Random new next asString, ':', DateAndTime now asString.
	^ self class md5Hash: pt! !

!ZnDigestAuthenticator methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
isRequestAuthenticated: request
	| authorization response cresponse a1 a2 nonce opaque |
	authorization := self class parseAuthRequest: (request headers at: 'Authorization' ifAbsent: [ ^ false ]).
	nonce := authorization at: 'nonce' ifAbsent: [ ^ false ].
	opaque := self nonces at: nonce ifAbsent: [ ^ false ].
	a1 := self a1for: (authorization at: 'username' ifAbsent: [ ^ false ]).
	a1 ifNil: [ ^ false ].
	a2 := self a2forUrl: (authorization at: 'uri' ifAbsent: [ '/' ]) method: request method.
	response := self class md5Hash: (a1, ':', nonce, ':', a2).
	"(opaque = (authorization at: 'opaque' ifAbsent: [^false])) ifFalse: [^false]."
	cresponse := authorization at: 'response' ifAbsent: [ ^ false ].
	^ (response = cresponse)! !

!ZnDigestAuthenticator methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
nonces 
	^ nonces ifNil: [ nonces := Dictionary new ]! !

Object subclass: #ZnChunkedWriteStream
	instanceVariableNames: 'stream extraHeaders totalSize chunkCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnChunkedWriteStream commentStamp: '<historical>' prior: 0!
I am ZnChunkedWriteStream, I wrap another write stream and add HTTP 1.1 chunked transfer encoding.

I write a chunk for each #nextPutAll: or #next:putAll:startingAt:
I should be wrapped in a ZnBufferedWriteStream for #next: to work.
When done, send #close or #finish to me.

Part of Zinc HTTP Components.
!

!ZnChunkedWriteStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 5/17/2013 15:08'!
on: writeStream
	^ self new
		on: writeStream;
		yourself! !

!ZnChunkedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/17/2013 15:10'!
binary
	stream binary! !

!ZnChunkedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 00:37'!
chunkCount
	^ chunkCount! !

!ZnChunkedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/17/2013 15:09'!
close
	self finish.
	stream close! !

!ZnChunkedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/17/2013 16:22'!
crlf
	stream nextPutAll: (self isBinary ifTrue: [ #[ 13 10 ] ] ifFalse: [ String crlf ])! !

!ZnChunkedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/2/2014 22:21'!
extraHeaders
	^ extraHeaders ifNil: [ extraHeaders := ZnHeaders new ]! !

!ZnChunkedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/17/2013 15:31'!
finish
	self writeChunkSize: 0.
	self crlf.
	extraHeaders ifNotNil: [ extraHeaders writeOn: stream ]! !

!ZnChunkedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/17/2013 15:09'!
flush
	stream flush! !

!ZnChunkedWriteStream methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 5/18/2013 00:35'!
initialize
	super initialize.
	totalSize := 0.
	chunkCount := 0! !

!ZnChunkedWriteStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/17/2013 15:25'!
isBinary
	^ stream isBinary
! !

!ZnChunkedWriteStream methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/18/2013 00:12'!
isStream
	^ true! !

!ZnChunkedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 00:23'!
next: count putAll: collection
	self 
		next: count 
		putAll: collection 
		startingAt: 1! !

!ZnChunkedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 00:35'!
next: count putAll: collection startingAt: offset
	self writeChunkSize: count.
	stream next: count putAll: collection startingAt: offset.
	self crlf.
	chunkCount := chunkCount + 1.
	totalSize := totalSize + count
! !

!ZnChunkedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 21:02'!
nextPut: element
	self error: 'Wrap me into a buffered stream, because each #nexPut: will create a new chunk'! !

!ZnChunkedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 00:26'!
nextPutAll: collection
	self 
		next: collection size
		putAll: collection
		startingAt: 1! !

!ZnChunkedWriteStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/17/2013 15:08'!
on: writeStream
	stream := writeStream! !

!ZnChunkedWriteStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 00:28'!
position
	^ totalSize! !

!ZnChunkedWriteStream methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/17/2013 15:28'!
writeChunkSize: size
	stream nextPutAll: (self isBinary ifTrue: [ size printStringHex asByteArray ] ifFalse: [ size printStringHex ]).
	self crlf! !

Object subclass: #ZnClient
	instanceVariableNames: 'request response connection lastUsed options session logLevel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnClient commentStamp: 'SvenVanCaekenberghe 11/8/2011 22:31' prior: 0!
I am ZnClient, an object to build, execute and process HTTP client requests.

I have a rich protocol to construct requests and to access responses.
I have various error handling options.
I can reuse an existing connection to a specific host:port.
I handle sessions, cookies, redirects and authentication.
I have many options (settings) with sensible defaults.

Simplest possible invocation:

	ZnClient new
		get: 'http://zn.stfx.eu/zn/numbers.txt'.
	
Using some features to make a better HTTP request:

	ZnClient new
		systemPolicy;
		accept: ZnMimeType textPlain;
		http;
		host: 'zn.stfx.eu';
		path: 'zn/numbers.txt';
		contentReader: [ :entity | entity contents lines collect: [ :each | each asNumber ] ];
		ifFail: [ :exception | self inform: 'I am sorry: ', exception printString ];
		get.

Part of Zinc HTTP Components.!

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 3/2/2014 18:40'!
accept: mimeTypeObject
	"Set the Accept HTTP header to mimeTypeObject for the current request."

	| mimeType |
	mimeType := mimeTypeObject asZnMimeType.
	self request setAccept: mimeType! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:18'!
addPart: mimePart
	"Add mimePart to the multipart/form-data entity of the current request"

	self multiPartFormDataEntity 
		addPart: mimePart ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:03'!
addPath: object
	"Extend the path of the current request with object.
	Either a string or a collection of strings is allowed as argument."
	
	object isString
		ifTrue: [ self addPathSegment: object ] 
		ifFalse: [ object do: [ :each | self addPathSegment: each ] ]! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:03'!
addPathSegment: pathSegment
	"Extend the path of the current request with a single pathSegment."
	
	request url 
		ifNil: [ request url: (ZnUrl new addPathSegment: pathSegment; yourself) ]
		ifNotNil: [ request url addPathSegment: pathSegment ]  ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:30'!
applicationFormUrlEncodedEntity
	"Initialize the current request to have an application/x-www-form-urlencoded entity.
	Return the existing entity if there is one. 
	Fail if another type was already set."
	
	request hasEntity
		ifTrue: [
			request entity contentType = ZnMimeType applicationFormUrlEncoded 
				ifFalse: [ self error: 'An incompatible entity was already set for the request' ] ]
		ifFalse: [
			request entity: ZnApplicationFormUrlEncodedEntity new ].
	^ request entity! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 3/2/2014 18:50'!
autoResetEntityMethods
	"Return a collection of HTTP methods for which to auto #resetEntity.
	The default is to do this only for HEAD and DELETE, although most people 
	would want it for GET too, but this can be used for forms"
	
	^ self optionAt: #autoResetEntityMethods ifAbsent: [ #(HEAD DELETE) ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 3/2/2014 18:51'!
autoResetEntityMethods: methods
	"Set the collection of HTTP methods for which to auto #resetEntity.
	The default is to do this only for HEAD and DELETE, although most people 
	would want it for GET too, but this can be used for forms"
	
	^ self optionAt: #autoResetEntityMethods put: methods! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/27/2012 14:29'!
beOneShot
	"Switch me to use connections only for a single request/response cycle.
	By default, I will keep HTTP connections open according to the HTTP 1.1 spec.
	When oneShot is true, I will add a Connection: close header to requests and close after one cycle.
	I cannot be one shot when #streaming is true because the stream would be closed too soon."
	
	self streaming
		ifTrue: [ self error: 'I cannot be one shot when #streaming is true.' ].
	self optionAt: #oneShot put: true   ! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2013 11:24'!
canReuseConnection
	self isConnected ifFalse: [ ^ false ].
	lastUsed ifNil: [ ^ false ].
	^ Time totalSeconds - lastUsed < self connectionReuseTimeout! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 10/17/2014 13:18'!
certificate
	"Return the TLS/SSL certificate to be used, or nil for none."
	
	^ self optionAt: #certificate ifAbsent: [ nil ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 10/17/2014 13:20'!
certificate: anObject
	"Set the TLS/SSL certificate to be used to anObject, or nil for none.
	See ZdcAbstractSSLSession>>#certificateName: for more information."
	
	^ self optionAt: #certificate put: anObject! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:10'!
cleanupConnection
	(self isOneShot or: [ request notNil and: [ request wantsConnectionClose ] ])
		ifTrue: [
			self close ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 6/11/2014 15:34'!
clientId
	"Return the id for this HTTP client. Defaults to nil.
	Can be used identify different clients' log events."
	
	^ self optionAt: #clientId ifAbsent: [ nil ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 6/11/2014 15:35'!
clientId: anObject
	"Set the id for this HTTP client to anObject. Defaults to nil.
	Can be used identify different clients' log events."
	
	^ self optionAt: #clientId put: anObject! !

!ZnClient methodsFor: 'cleanup' stamp: 'SvenVanCaekenberghe 2/11/2015 15:26'!
close
	"Close any HTTP network connection that I might have kept open."

	connection
		ifNotNil: [ 
			| address port |
			address := [ connection socket remoteAddress ] on: Error do: [ nil ].
			port := [ connection socket remotePort ] on: Error do: [ nil ].
			connection close.
			connection := nil.
			self logConnectionClosedAddress: address port: port ].
	lastUsed := nil! !

!ZnClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/19/2012 15:43'!
connection
	"Return the socket stream connection that I am currently using"
	
	^ connection! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 19:36'!
connectionReuseTimeout
	"Return the timeout in seconds currently used to determine
	if I should try to reuse an open connection. Defaults to 10 seconds."

	^ self optionAt: #connectionReuseTimeout ifAbsent: [ 10 ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:30'!
connectionReuseTimeout: seconds
	"Set the timeout used to determine if I should try 
	to reuse an open connection to seconds."

	self optionAt: #connectionReuseTimeout put: seconds! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 19:41'!
contentReader
	"Return the current content reader, an object conforming 
	to the #value: protocol to convert a response entity
	as used by #contents. Defaults to nil.
	Can be used to parse incoming data."
	
	^ self optionAt: #contentReader ifAbsent: [ nil ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:30'!
contentReader: object
	"Set the content reader, an object conforming 
	to the #value: protocol to convert a response entity
	as used by #contents to object.
	Can be used to parse incoming data."

	self optionAt: #contentReader put: object! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 5/9/2012 09:21'!
contentType: mimeType
	"Set the content-type of the request entity, provided we have one"
	
	request hasEntity
		ifTrue: [ 
			request entity contentType: mimeType asZnMimeType ]
		ifFalse: [ 
			self error: 'There is no request entity yet, cannot set its content-type' ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 19:42'!
contentWriter
	"Return the current content writer, an object conforming 
	to the #value: protocol to convert an object to a request content entity
	as used by #contents:. Defaults to nil.
	Can be used to format data to be uploaded."

	^ self optionAt: #contentWriter ifAbsent: [ nil ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:31'!
contentWriter: object
	"Set current content writer, an object conforming 
	to the #value: protocol to convert an object to a request content entity
	as used by #contents:. Defaults to nil.
	Can be used to format data to be uploaded."

	self optionAt: #contentWriter put: object! !

!ZnClient methodsFor: 'accessing response' stamp: 'SvenVanCaekenberghe 11/8/2011 20:56'!
contents 
	"Answer the contents or HTTP payload of the last response, 
	assuming the current request was executed.
	If I have a contentReader, it will be used to compute the contents.
	Otherwise I delegate to ZnResponse>>#contents, unless I am #streaming,
	in which case I return the stream of the streaming entity."

	^ response ifNotNil: [ 
		self contentReader 
			ifNil: [ 
				self streaming
					ifTrue: [  response entity stream ]
					ifFalse: [ response contents ] ]
			ifNotNil: [ :reader | reader value: response entity ] ]  ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:13'!
contents: object
	"Set the entity contents of the current request to object.
	I will pass this through my contentWriter if I have one.
	Otherwise I delegate to ZnEntity to convert object to a generic entity."
	
	self contentWriter 
		ifNil: [ self entity: (ZnEntity with: object) ]
		ifNotNil: [ :writer | self entity: (writer value: object) ]! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/7/2013 16:04'!
curl
	"Create and return a curl command line invocation equivalent to the current request."
	"ZnClient new url: 'http://zn.stfx.eu/zn/small.html'; method: #GET; curl."
	"ZnClient new url: 'http://zn.stfx.eu/echo'; contents: #[2 4 6 8]; method: #POST; curl."
	
	self prepareRequest.
	^ String streamContents: [ :stream |
		request hasEntity ifTrue: [ 
			 stream << 'echo ' << request entity asByteArray hex << ' | xxd -r -p | ' ].
		stream << 'curl'.
		request method = #GET ifFalse: [ stream << ' -X ' << request method ].
		stream space; print: request uri.
		request headersDo: [ :key :value |
			stream << ' -H"' << key << ':' << value << '"' ].
		request hasEntity ifTrue: [ stream << ' --data-binary @-' ] ]! !

!ZnClient methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 8/18/2011 15:07'!
debugPolicy
	self 
		beOneShot! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:13'!
delete
	"Execute a HTTP DELETE ont the request set up and return the response #contents."
	
	^ self 
		method: #DELETE; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/8/2011 19:04'!
delete: url
	"Execute an HTTP DELETE request on url and return the response #contents."
	
	^ self 
		url: url; 
		delete! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/23/2011 16:59'!
dontFollowRedirects
	"Disable the default automatic following of HTTP redirects.
	Redirects will be returned as normal responses."

	^ self followRedirects: false! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/12/2012 21:46'!
downloadEntityTo: path
	"If I hold an entity, download it directly to a file.
	If path describes a file, use that. 
	If path describes a directory, use the last path segment of the URI to construct a file in that directory.."

	| target |
	self entity ifNil: [ ^ self ].
	target := ZnFileSystemUtils 
		downloadPathTo: path 
		for: self request uri lastPathSegment.
	ZnFileSystemUtils 
		newFileNamed: target 
		do:  [ :fileStream |
			fileStream binary.
			self withProgressDo: [
				self entity writeOn: fileStream ] ]! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 5/10/2012 22:21'!
downloadTo: path
	"Setup streaming, do a GET request and download the entity directly to a file.
	If path describes a file, use that. 
	If path describes a directory, use the last path segment of the URI to construct a file in that directory.
	Return true when successful, false otherwise."

	self 
		streaming: true; 
		get.
	^ self isSuccess
		ifTrue: [
			self downloadEntityTo: path.
			true ]
		ifFalse: [ 
			false ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:27'!
enforceAcceptContentType
	"Return whether or not I enforce that the content type of the
	response matches the content type specified in the accept request header."
	
	^ self optionAt: #enforceAcceptContentType ifAbsent: [ false ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:27'!
enforceAcceptContentType: boolean
	"Set whether or not I enforce that the content type of the
	response matches the content type specified in the accept request
	header to boolean."

	self optionAt: #enforceAcceptContentType put: boolean! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:27'!
enforceHttpSuccess
	"Return whether or not I enforce that the response 
	to the request is an HTTP success code."
	
	^ self optionAt: #enforceHttpSuccess ifAbsent: [ false ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:28'!
enforceHttpSuccess: boolean
	"Set whether or not I enforce that the response 
	to the request is an HTTP success code to boolean."

	self optionAt: #enforceHttpSuccess put: boolean! !

!ZnClient methodsFor: 'accessing response' stamp: 'SvenVanCaekenberghe 11/8/2011 18:58'!
entity
	"Answer the entity or HTTP payload of the last response, 
	assuming the current request was executed."
	
	^ response ifNotNil: [ response entity ]  ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:14'!
entity: entity
	"Set the entity of the current request to enity.
	A ZnEntity subclass or compatible object is expected."
	
	request entity: entity! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 6/12/2014 16:01'!
execute
	"Execute the currently set up request to generate a response.
	Return the #contents of the response, if any."

	| result |
	self prepareRequest.
	self withProgressDo: [ result := self executeWithTimeout ].
	self prepareNextRequest.
	^ result! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 6/12/2014 20:04'!
executeRequestResponse
	| initialMilliseconds requestDuration responseDuration |
	initialMilliseconds := Time millisecondClockValue.
	self writeRequest.
	requestDuration := self logRequestWrittenStarted: initialMilliseconds.
	initialMilliseconds := Time millisecondClockValue.
	self readResponse.
	responseDuration := self logResponseReadStarted: initialMilliseconds.
	self logClientTransactionRequestDuration: requestDuration responseDuration: responseDuration! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 11/23/2011 17:01'!
executeWithRedirectsRemaining: redirectCount
	self getConnectionAndExecute.
	response isRedirect 
		ifTrue: [
			(redirectCount > 0 and: [ self followRedirects ])
				ifTrue: [
					self 
						prepareRedirect; 
						executeWithRedirectsRemaining: redirectCount - 1 ]
				ifFalse: [
					self followRedirects 
						ifTrue: [ ZnTooManyRedirects signal ] ] ].
	^ self handleResponse
	! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 6/12/2014 20:09'!
executeWithRetriesRemaining: retryCount
	^ [ self executeWithRedirectsRemaining: self maxNumberOfRedirects ]
		on: self retryExceptionSet
		do: [ :exception |
			retryCount > 0
				ifTrue: [
					self 
						handleRetry: exception;
						executeWithRetriesRemaining: retryCount - 1 ]
				ifFalse: [
					exception pass ] ]! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 9/17/2012 10:03'!
executeWithTimeout
	^ self withTimeoutDo: [ 
		[ self executeWithRetriesRemaining: self numberOfRetries ]
			on: Error
			do: self ifFailBlock ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/23/2011 16:58'!
followRedirects
	"Return whether I will automatically following of HTTP redirects.
	This is still subject to #maxNumberOfRedirects. Defaults to true.
	When set to false, redirects will be returned as normal responses."

	^ self optionAt: #followRedirects ifAbsent: [ true ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:31'!
followRedirects: boolean
	"Set whether or not I will automatically following of HTTP redirects.
	This is still subject to #maxNumberOfRedirects. Defaults to true.
	When set to false, redirects will be returned as normal responses."

	self optionAt: #followRedirects put: boolean ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:19'!
formAdd: association
	"Add the key->value association to the application/x-www-form-urlencoded 
	entity of the current request."
	
	self applicationFormUrlEncodedEntity 
		add: association ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:20'!
formAddAll: keyedCollection
	"Add all key->value associations of keyedCollection to 
	the application/x-www-form-urlencoded entity of the current request."

	self applicationFormUrlEncodedEntity 
		addAll: keyedCollection ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:26'!
formAt: key add: value
	"Add key equals value to the application/x-www-form-urlencoded 
	entity of the current request. This is for multi-values form fields."

	self applicationFormUrlEncodedEntity 
		at: key add: value! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:21'!
formAt: key put: value
	"Set key equal to value in the application/x-www-form-urlencoded 
	entity of the current request."

	self applicationFormUrlEncodedEntity 
		at: key put: value! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:13'!
get
	"Execute an HTTP GET ont the request set up and return the response #contents."
	
	^ self 
		method: #GET; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/8/2011 19:04'!
get: url
	"Execute an HTTP GET request on url and return the response #contents."

	^ self 
		url: url; 
		get! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 6/12/2014 20:06'!
getConnectionAndExecute
	self canReuseConnection 
		ifTrue: [ 
			[ ^ self executeRequestResponse; cleanupConnection ] 
				on: Exception 
				do: [ :exception | self logIgnoringExceptionOnReusedConnection: exception; close ] ].
	self newConnectionTo: request url.
	[ self executeRequestResponse ]
		ensure: [ self cleanupConnection ]! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 6/12/2014 16:18'!
handleResponse
	self receiveCookies.
	(self enforceHttpSuccess and: [ response isSuccess not ])
		ifTrue: [ 
			(self followRedirects not and: [ response isRedirect ])
				ifFalse: [ (ZnHttpUnsuccessful response: response) signal ] ].
	(self enforceAcceptContentType and: [ self isContentTypeAcceptable not ])
		ifTrue: [ 
			(ZnUnexpectedContentType 
				expected: request accept asZnMimeType 
				actual: response contentType) signal ].
	^ self contents! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 6/12/2014 14:37'!
handleRetry: exception
	self logRetrying: exception.
	self close.
	(Delay forSeconds: self retryDelay) wait.
! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:13'!
head
	"Execute a HTTP HEAD ont the request set up and return nil."

	^ self 
		method: #HEAD; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/8/2011 19:05'!
head: url
	"Execute an HTTP HEAD request on url and return nil."

	^ self 
		url: url; 
		head! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 3/2/2014 18:42'!
headerAddAll: keyedCollection
	"Add all key/value pairs of keyedCollection as headers.
	Note that #addAllMulti: is used"
	
	request headers 
		addAll: keyedCollection ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 12/12/2012 10:19'!
headerAt: key add: value
	"Add value to key in the HTTP header of the current request.
	This is for multi-values headers."
	
	request headers 
		at: key add: value! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 12/12/2012 10:19'!
headerAt: key put: value
	"Set key equals value in the HTTP header of the current request"
	
	request headers 
		at: key put: value   ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:26'!
host: hostName
	"Set the host of the current request to hostName."
	
	self url: (ZnUrl new host: hostName; yourself)! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:27'!
http
	"Set the scheme of the current request to HTTP."
	
	self url: (ZnUrl new scheme: #http; yourself)! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:27'!
https
	"Set the scheme of the current request to HTTPS."
	
	self url: (ZnUrl new scheme: #https; yourself)! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:29'!
ifFail: block
	"Set the block that I will be execute when the request fails.
	The exception of the failure will be passed as argument."
	
	self optionAt: #ifFailBlock put: block! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/17/2012 10:02'!
ifFailBlock
	"Return the block that I should execute if the request fails.
	Defaults to #pass-ing the exception up. 
	Accepts an exception as argument."
	
	^ self optionAt: #ifFailBlock ifAbsent: [ [ :exception | exception pass ] ]! !

!ZnClient methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 6/12/2014 19:48'!
initialize
	request := ZnRequest empty.
	self loggingOn ! !

!ZnClient methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 8/19/2011 16:57'!
interactivePolicy
	self 
		beOneShot;
		ifFail: [ :exception | 
			self inform: 'HTTP call failed with ', exception printString.
			nil ]! !

!ZnClient methodsFor: 'testing' stamp: 'ChristopheDemarey 12/11/2014 10:24'!
isBadRequest
	"Return true if the last executed HTTP request returned an HTTP bad request response code."
	
	^ response 
		ifNil: [ self error: 'Request not yet executed' ]
		ifNotNil: [ response isBadRequest ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/8/2011 19:14'!
isConnected
	"Return true if I hold an open HTTP connection."

	^ connection notNil and: [ connection isConnected ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/8/2011 19:17'!
isContentTypeAcceptable
	"Return true if the contentType of the response of the last executed 
	HTTP request matches the accept header of the request."
	
	^ response 
		ifNil: [ self error: 'Request not yet executed' ]
		ifNotNil: [ 
			response hasEntity not 
				or: [ response contentType matches: request accept asZnMimeType ] ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/12/2012 10:26'!
isCreated
	"Return true it the last executed HTTP request returned an HTTP created response code."
	
	^ response 
		ifNil: [ self error: 'Request not yet executed' ]
		ifNotNil: [ response isCreated ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/12/2012 10:25'!
isNotFound
	"Return true it the last executed HTTP request returned an HTTP not found response code."
	
	^ response 
		ifNil: [ self error: 'Request not yet executed' ]
		ifNotNil: [ response isNotFound ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/30/2012 20:59'!
isNotModified
	"Return true it the last executed HTTP request returned an HTTP not modified response code."
	
	^ response 
		ifNil: [ self error: 'Request not yet executed' ]
		ifNotNil: [ response isNotModified ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:30'!
isOneShot
	"Return if I use connections only for a single request/response cycle.
	By default, I will keep HTTP connections open according to the HTTP 1.1 spec.
	When oneShot is true, I will add a Connection: close header to requests."

	^ self optionAt: #oneShot ifAbsent: [ false ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/23/2011 17:17'!
isRedirect
	"Return true it the last executed HTTP request returned an HTTP redirect response code."
	
	^ response 
		ifNil: [ self error: 'Request not yet executed' ]
		ifNotNil: [ response isRedirect ]! !

!ZnClient methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/8/2011 19:17'!
isSuccess 
	"Return true it the last executed HTTP request returned an HTTP success response code."
	
	^ response 
		ifNil: [ self error: 'Request not yet executed' ]
		ifNotNil: [ response isSuccess ]! !

!ZnClient methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 13:24'!
logClientTransactionRequestDuration: requestDuration responseDuration: responseDuration
	| logEventClass |
	logLevel < 1 ifTrue: [ ^ nil ].
	logEventClass := logLevel = 1
		ifTrue: [ ZnSimplifiedClientTransactionEvent ]
		ifFalse: [ ZnClientTransactionEvent ].
	^ (self newLogEvent: logEventClass)
		request: request;
		requestDuration: requestDuration;
		response: response; 
		responseDuration: responseDuration;
		emit;
		yourself! !

!ZnClient methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/12/2014 19:51'!
logConnectionClosedAddress: address port: port
	logLevel < 3 ifTrue: [ ^ nil ].
	^ (self newLogEvent: ZnClientConnectionClosedEvent)
		address: address;
		port: port;
		emit! !

!ZnClient methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 10/23/2015 16:38'!
logConnectionEstablishedTo: url started: initialMilliseconds
	| event |
	logLevel < 3 ifTrue: [ ^ nil ].
	(event := self newLogEvent: ZnConnectionEstablishedEvent)
		hostname: url host;
		port: url portOrDefault;
		address: ([ connection socket remoteAddress ] on: Error do: [ nil ]);
		duration: (Time millisecondsSince: initialMilliseconds).
	(ZnNetworkingUtils shouldProxyUrl: url)
		ifTrue: [ event proxy: ZnNetworkingUtils proxyUrl ].
	event emit! !

!ZnClient methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/12/2014 19:51'!
logIgnoringExceptionOnReusedConnection: exception
	logLevel < 3 ifTrue: [ ^ nil ].
	^ (self newLogEvent: ZnClientIgnoringExceptionOnConnectionReuseEvent)
		exception: exception;
		emit! !

!ZnClient methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 6/12/2014 20:12'!
logLevel: integer
	"Set the log level to integer.
	0 - no logging
	1 - simplified transaction logging
	2 - detailed transaction logging
	3 - log everything"
	
	logLevel := integer! !

!ZnClient methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/12/2014 14:38'!
logRedirect: url
	^ (self newLogEvent: ZnClientFollowingRedirectEvent)
		target: url;
		emit! !

!ZnClient methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/12/2014 19:53'!
logRequestWrittenStarted: initialMilliseconds
	^ logLevel < 3
		ifTrue: [ Time millisecondsSince: initialMilliseconds ]
		ifFalse: [ 
			(self newLogEvent: ZnRequestWrittenEvent)
				request: request;
				duration: (Time millisecondsSince: initialMilliseconds);
				emit;
				duration ]! !

!ZnClient methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/12/2014 19:53'!
logResponseReadStarted: initialMilliseconds
	^ logLevel < 3
		ifTrue: [ Time millisecondsSince: initialMilliseconds ]
		ifFalse: [ 
			(self newLogEvent: ZnResponseReadEvent)
				response: response;
				duration: (Time millisecondsSince: initialMilliseconds);
				emit;
				duration ]! !

!ZnClient methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/12/2014 19:51'!
logRetrying: exception
	logLevel < 3 ifTrue: [ ^ nil ].
	^ (self newLogEvent: ZnClientRetryingEvent)
		exception: exception;
		emit! !

!ZnClient methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 6/16/2014 14:59'!
logToTranscript
	"Setup logging to the Transcript.
	If logging was completely off, turn it on"

	logLevel isZero ifTrue: [ self loggingOn ].
	ZnLogEvent logToTranscript ! !

!ZnClient methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 6/12/2014 20:13'!
loggingOff
	"Turn logging completely off - do not generate log events."
	
	self logLevel: 0! !

!ZnClient methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 6/12/2014 20:13'!
loggingOn
	"Turn full logging on - generate all log events"
	
	self logLevel: 3! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/23/2011 17:23'!
maxNumberOfRedirects
	"Return the maximum number of HTTP redirect that will be followed.
	Note that when the #followsRedirect option is false, 
	no redirects will ever be followed regardless of the value of this option.
	Defaults to 3. A ZnTooManyRedirects will be signalled when the count reaches zero."

	^ self optionAt: #maxNumberOfRedirects ifAbsent: [ 3 ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/23/2011 17:23'!
maxNumberOfRedirects: count
	"Set the maximum number of HTTP redirect that will be followed to count.
	Note that when the #followsRedirect option is false, 
	no redirects will ever be followed regardless of the value of this option.
	A ZnTooManyRedirects will be signalled when the count reaches zero."

	self optionAt: #maxNumberOfRedirects put: count! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 3/2/2014 18:54'!
method: httpMethod
	"Set the HTTP method of the current request to httpMethod.
	An uppercase symbol such as #GET is expected.
	Note that for all methods in #autoResetEntityMethods #resetEntity is called automatically,
	since you don't want the entity of a previous request to be sent again"

	(self autoResetEntityMethods includes: httpMethod)
		ifTrue: [ self resetEntity ].
	request method: httpMethod! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 5/21/2013 22:31'!
multiPartFormDataEntity
	"Initialize the current request to have an multipart/form-data entity.
	Return the existing entity if there is one. 
	Fail if another type was already set."

	request hasEntity
		ifTrue: [
			(request entity contentType matches: ZnMimeType multiPartFormData)
				ifFalse: [ self error: 'An incompatible entity was already set for the request' ] ]
		ifFalse: [
			request entity: ZnMultiPartFormDataEntity new ].
	^ request entity! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/6/2015 16:26'!
newConnectionTo: url
	| initialMilliseconds |
	self signalHTTPProgress: 'Connecting to ',  url authority.
	initialMilliseconds := Time millisecondClockValue.
	(connection notNil and: [ connection isConnected ]) 
		ifTrue: [ connection close ].
	connection := ZnNetworkingUtils socketStreamToUrl: url.
	url hasSecureScheme
		ifTrue: [ self setupTLSTo: url ].
	self logConnectionEstablishedTo: url started: initialMilliseconds! !

!ZnClient methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/11/2014 15:35'!
newLogEvent: logEventClass
	^ logEventClass new
		clientId: self clientId;
		yourself! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:31'!
numberOfRetries
	"Return how many times I will retry a failed request.
	Defaults to 0."
	
	^ self optionAt: #numberOfRetries ifAbsent: [ 0 ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:31'!
numberOfRetries: count
	"Set the number of times I will retry a failed request to count."
	
	self optionAt: #numberOfRetries put: count! !

!ZnClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/8/2011 16:53'!
optionAt: key ifAbsent: block
	"Return my option/settings stored under key.
	Execute block if I have no such option/setting.
	This is a generic interface, see my options protocol for specific usages."

	options ifNil: [ ^ block value ].
	^ options at: key ifAbsent: block! !

!ZnClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/30/2012 21:30'!
optionAt: key put: value
	"Set my option/setting identified by key to be value.
	This is a generic interface, see my options protocol for specific usages."
	
	options ifNil: [ options := Dictionary new ].
	options at: key put: value! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/3/2015 14:39'!
options
	"Execute an HTTP OPTIONS ont the request set up and return the response #contents."
	
	^ self 
		method: #OPTIONS; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/3/2015 14:44'!
options: url
	"Execute an HTTP OPTIONS request on url and return the response #contents."

	^ self 
		url: url; 
		options! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/3/2015 14:38'!
patch
	"Execute a HTTP PATCH ont the request set up and return the response #contents."

	^ self 
		method: #PATCH; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 9/3/2015 14:39'!
patch: url contents: object
	"Execute an HTTP PATCH request on url with object as request #contents:
	and return the response #contents."

	^ self 
		url: url; 
		contents: object; 
		patch! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:43'!
path: pathString
	"Set the path of the current request to pathString.
	The argument will be merged with of the current/previous request."
	
	self url: pathString! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:32'!
port: portNumber
	"Set the port of the current request to portNumber."
	
	self url: (ZnUrl new port: portNumber; yourself)! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:13'!
post
	"Execute a HTTP POST ont the request set up and return the response #contents."

	^ self 
		method: #POST; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/8/2011 19:12'!
post: url contents: object
	"Execute an HTTP POST request on url with object as request #contents:
	and return the response #contents."

	^ self 
		url: url; 
		contents: object; 
		post! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 6/12/2014 16:05'!
prepareNextRequest
	"The request could be modified destructively in the next run,
	but the request can also be part of a log event, hence copy it"
	request := request copy	
! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 4/29/2015 13:15'!
prepareRedirect
	"We implement the common behavior that POST/PUT redirects turn into GET request
	and that the body data is dropped, except for a 307 or 308 where this is explicitly forbidden.
	Note the cookie handling too."

	| targetUrl |
	targetUrl := self redirectUrl.
	self logRedirect: targetUrl.
	self receiveCookies.
	self prepareNextRequest.
	(#(307 308) includes: response code)
		ifFalse: [  
			(#(POST PUT) includes: request method)
				ifTrue: [ self method: #GET ].
			request hasEntity
				ifTrue: [ request resetEntity: nil ] ].
	self url: targetUrl.
	self sendCookies! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 2/21/2013 15:59'!
prepareRequest
	self isOneShot 
		ifTrue: [ request setConnectionClose ].
	request hasEntity
		ifTrue: [ 
			"This is necessary because entities are often modified after first being added empty,
			and a wrong content length is cached at the request level"
			request resetEntity: request entity ].
	self sendCookies.
	self prepareRequestHook 
		ifNotNil: [ :hook | hook value: request ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 2/21/2013 15:57'!
prepareRequest: hook
	"Set the current request preparation hook, an object conforming 
	to the #value: protocol that gets the final chance to change a request 
	right before it gets executed. Defaults to nil.
	Typically used to sign requests. See also #prepareRequestHook "

	^ self optionAt: #prepareRequestHook put: hook! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 2/21/2013 15:57'!
prepareRequestHook
	"Return the current request preparation hook, an object conforming 
	to the #value: protocol that gets the final chance to change a request 
	right before it gets executed. Defaults to nil.
	Typically used to sign requests. See also #prepareRequest: "

	^ self optionAt: #prepareRequestHook ifAbsent: [ nil ]! !

!ZnClient methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 11/8/2011 19:14'!
put
	"Execute a HTTP PUT ont the request set up and return the response #contents."

	^ self 
		method: #PUT; 
		execute! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/8/2011 19:12'!
put: url contents: object
	"Execute an HTTP PUT request on url with object as request #contents:
	and return the response #contents."

	^ self 
		url: url; 
		contents: object; 
		put! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:36'!
queryAddAll: keyedCollection
	"Add all key -> value elements of keyedCollection 
	as an HTTP query parameters to the current request."

	keyedCollection keysAndValuesDo: [ :key :value |
		self queryAt: key put: value ]! !

!ZnClient methodsFor: 'accessing request' stamp: 'ChristopheDemarey 9/12/2014 17:28'!
queryAt: key add: value 
	"Modify the receiver by adding a query variable key=value.
	If key already exists, value is added to it."

	request url 
		ifNil: [ request url: (ZnUrl new queryAt: key add: value; yourself) ]
		ifNotNil: [ request url queryAt: key add: value ]  ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 12/12/2012 10:19'!
queryAt: key put: value
	"Add key equals value as an HTTP query parameter to the current request."
	
	request url 
		ifNil: [ request url: (ZnUrl new queryAt: key put: value; yourself) ]
		ifNotNil: [ request url queryAt: key put: value ]  ! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 10/6/2015 16:45'!
readResponse
	self signalHTTPProgress: 'Reading response'.
	response := (#(HEAD CONNECT) includes: request method)
		ifTrue: [ ZnResponse readHeaderFrom: connection ]
		ifFalse: [ 
			self streaming
				ifTrue: [ ZnResponse readStreamingFrom: connection ]
				ifFalse: [ ZnResponse readFrom: connection ] ].
	lastUsed := Time totalSeconds! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/11/2014 15:40'!
receiveCookies
	response cookies do: [ :each |
		each defaultDomain: request url host.
		self session cookieJar add: each ]! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 1/7/2013 09:57'!
redirectUrl
	"We support relative locations although they are not the standard (http://en.wikipedia.org/wiki/HTTP_location)"

	| redirectUrl |
	redirectUrl := response location asZnUrl.
	^ redirectUrl isAbsolute
		ifTrue: [ redirectUrl ]
		ifFalse: [ redirectUrl inContextOf: request url ]! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:33'!
request
	"Return the current request that I am building,
	or the previous request that I executed."
	
	^ request ! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:34'!
request: aRequest
	"Replace my current request with aRequest.
	This is a low level operation only needed in special cases."
	
	self validateConnectionForHost: aRequest url host port: aRequest url port.
	request := aRequest! !

!ZnClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/29/2012 17:40'!
resetCookies
	session ifNotNil: [ session resetCookies ]! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 3/2/2014 18:47'!
resetEntity
	"Make sure that the current request does not hold an entity from a previous request.
	This is sometimes necessary after a POST or PUT. This method will be called automatically
	by #method: for certain requests."
	
	(request notNil and: [ request hasEntity ])
		ifTrue: [ request resetEntity: nil ]! !

!ZnClient methodsFor: 'accessing response' stamp: 'SvenVanCaekenberghe 11/8/2011 18:58'!
response 
	"Answer the last response, assuming the current request was executed."

	^ response ! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:33'!
retryDelay
	"Return the number of seconds I will delay retrying a failed request,
	provided I do retry requests. Defaults to 1 seconds."
	
	^ self optionAt: #retryDelay ifAbsent: [ 1 ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:31'!
retryDelay: seconds
	"Set the delay before I will delay retrying a failed request to seconds,
	provided I do retry requests."

	self optionAt: #retryDelay put: seconds! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/12/2014 20:09'!
retryExceptionSet
	^ NetworkError, ZnParseError, ZnCharacterEncodingError, ZnUnknownScheme, ZnPortNotANumber! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/11/2014 15:41'!
sendCookies
	| cookies cookiesString |
	session ifNil: [ ^ self ].
	cookies := (session cookieJar cookiesForUrl: request url).
	cookies isEmpty ifTrue: [ ^ self ]. 
	cookiesString := String streamContents: [ :stream |
		cookies
			do: [ :each | each writeNameAndValueOn: stream ]
			separatedBy: [ stream nextPutAll: '; ' ] ].
	request setCookie: cookiesString! !

!ZnClient methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/8/2011 16:54'!
session
	"Return the current HTTP session object that I am using.
	This session object holds cookies among other things."

	^ session ifNil: [ session := ZnUserAgentSession new ]! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 9/30/2012 20:53'!
setAcceptEncodingGzip
	"Give the server the hint that we accept gzip content encoding.
	When the server complies, important bandwidth reductions are possible."

	self request setAcceptEncodingGzip! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 3/2/2014 18:39'!
setBasicAuthenticationUsername: username password: password
	"Set the username/password for basic authenticationfor the current request."
	
	self request setBasicAuthenticationUsername: username password: password
! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 9/30/2012 20:59'!
setIfModifiedSince: reference
	"Tell the server that we are only interested in the currently requested URI
	if it was modified since reference, a DateAndTime.
	If the URI was not modified since reference, the response will be #isNotModified."

	self request setIfModifiedSince: reference! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/6/2015 21:52'!
setupTLSTo: url
	(ZnNetworkingUtils shouldProxyUrl: url)
		ifTrue: [ | originalRequest |
			"http://www.ietf.org/rfc/rfc2817.txt (section 5)"
			"https://en.wikipedia.org/wiki/HTTP_tunnel#HTTP_CONNECT_tunneling"
			originalRequest := request copy.
			self method: #CONNECT; writeRequest; readResponse.
			response isSuccess ifFalse: [ self error: 'Failed to CONNECT to proxy for TLS/SSL' ].
			connection := ZnNetworkingUtils secureSocketStreamOn: connection socket.
			request := originalRequest ].
	self certificate ifNotNil: [ :certificate | 
		connection sslSession certificateName: certificate ].
	connection connect! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/12/2014 14:32'!
signalHTTPProgress: message
	self signalProgress
		ifTrue: [ HTTPProgress signal: message ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:44'!
signalProgress
	"Return true if I signal HTTPProgress notifications during execution."
	
	^ self optionAt: #signalProgress ifAbsent: [ false ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:45'!
signalProgress: boolean
	"Set whether I will signal HTTPProgress notifications during execution to boolean."
	
	self optionAt: #signalProgress put: boolean! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 11/8/2011 20:58'!
streaming
	"Return whether I will return streaming response objects (see ZnStreamingEntity).
	If true, my users are responsible for actually reading the incoming response entity data stream."
	
	^ self optionAt: #streaming ifAbsent: [ false ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/27/2012 14:26'!
streaming: boolean
	"Set whether I will return streaming response objects (see ZnStreamingEntity) to boolean.
	If true, my users are responsible for actually reading the incoming response entity data stream.
	I cannot do streaming when #isOneShot is true because the stream would be closed too soon."

	(boolean and: [ self isOneShot ])
		ifTrue: [ self error: 'I cannot do streaming when #isOneShot is true.' ].
	self optionAt: #streaming put: boolean! !

!ZnClient methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 8/18/2011 20:27'!
systemPolicy
	self 
		enforceHttpSuccess: true;
		enforceAcceptContentType: true;
		numberOfRetries: 2! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 12/14/2011 14:03'!
timeout
	"Return the timeout in seconds for network socket stream operations
	like connecting, reading and writing. Defaults to a global setting."
	
	^ self optionAt: #timeout ifAbsent: [ ZnNetworkingUtils defaultSocketStreamTimeout ]! !

!ZnClient methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/30/2012 21:31'!
timeout: seconds
	"Set the timeout for network socket stream operations
	like connecting, reading and writing to seconds."

	self optionAt: #timeout put: seconds! !

!ZnClient methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 7/2/2012 16:19'!
uploadEntityFrom: path
	"Set up a ZnStreamingEntity from the binary contents of the file at path.
	The path extension is used to guess the content-type,
	can be overridden using #contentType: if necessary.
	Use the file's length to set the content-length.
	You can then use #put or #post to actually do an upload."
	
	| stream type length |
	stream := ZnFileSystemUtils fileStreamFor: path.
	stream binary.
	type := ZnMimeType forFilenameExtension: (ZnFileSystemUtils extensionFor: path).
	length := ZnFileSystemUtils fileSizeFor: path.
	self entity: (ZnStreamingEntity readFrom: stream usingType: type andLength: length)! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 1/30/2013 19:16'!
url: urlObject
	"Modify the URL of the current request object to urlObject.
	This is done using ZnUrl>>#inContextOf: so that either scheme, 
	host, port or path are set and merged.
	Default the URL using ZnUrl>>#asZnUrlWithDefaults to fall back
	to http or the default http/https ports."
	
	| url |
	url := urlObject asZnUrl.
	request url
		ifNotNil: [ url := url inContextOf: request url ].  
	url := url asZnUrlWithDefaults.  
	self validateConnectionForHost: url host port: url port.   
	request url: url.
	url hasUsername & url hasPassword
		ifTrue: [ self setBasicAuthenticationUsername: url username password: url password ]! !

!ZnClient methodsFor: 'accessing request' stamp: 'SvenVanCaekenberghe 11/8/2011 17:35'!
username: username password: password
	"Set the username/password for basic authenticationfor the current request."
	
	self setBasicAuthenticationUsername: username password: password
! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/12/2011 13:23'!
validateConnectionForHost: host port: port
	(connection isNil or: [ request url isNil ]) 
		ifTrue: [ ^ self ].
	(request url host = host and: [ request url port = port ])
		ifFalse: [ self close ]  ! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/12/2012 21:06'!
withProgressDo: block
	^ ZnSignalProgress value
		ifNil: [
			ZnSignalProgress 
				value: self signalProgress 
				during: [ ^ block value ] ]
		ifNotNil: [ 
			block value ]! !

!ZnClient methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/19/2011 16:55'!
withTimeoutDo: block
	^ ZnConnectionTimeout value
		ifNil: [
			ZnConnectionTimeout 
				value: self timeout 
				during: [ ^ block value ] ]
		ifNotNil: [ 
			block value ]! !

!ZnClient methodsFor: 'private protocol' stamp: 'SvenVanCaekenberghe 6/12/2014 20:04'!
writeRequest
	self signalHTTPProgress: 'Writing request'.
	request writeOn: connection.
	connection flush! !

Object subclass: #ZnCommonLogFormat
	instanceVariableNames: 'combined'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnCommonLogFormat commentStamp: 'SvenVanCaekenberghe 7/1/2015 16:34' prior: 0!
I am ZnCommonLogFormat, I can output ZnServerTransactionEvent objects using Apache Common Log Format (CLF).

https://en.wikipedia.org/wiki/Common_Log_Format
https://httpd.apache.org/docs/trunk/logs.html#common

| formatter |
formatter := ZnCommonLogFormat new.
ZnLogEvent announcer 
  when: ZnServerTransactionEvent 
  do: [ :event | 
    formatter format: event on: Transcript.
    Transcript cr; endEntry ].
!

!ZnCommonLogFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/1/2015 16:31'!
clientIPAddressFrom: event
	^ event request headers 
		at: ZnConstants remoteAddressHeader 
		ifAbsent: [ '-' ]! !

!ZnCommonLogFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/1/2015 14:22'!
combined
	^ combined! !

!ZnCommonLogFormat methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/1/2015 16:24'!
combined: boolean
	"If boolean is true (the default), use the 'Combined' format, 
	adding Referer and User-Agent fields."
	
	combined := boolean! !

!ZnCommonLogFormat methodsFor: 'public' stamp: 'SvenVanCaekenberghe 7/1/2015 16:34'!
format: event on: stream
	"Output a ZnServerTransactionEvent on a text stream in Common Log Format"
	
	stream << (self clientIPAddressFrom: event); << ' - '.
	stream << (self userFrom: event); space.
	self formatTimestamp: event timestamp on: stream.
	self formatRequest: event on: stream.
	self formatResponse: event on: stream. 
	combined ifTrue: [ self formatCombinedFields: event request on: stream ]! !

!ZnCommonLogFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/1/2015 16:09'!
formatCombinedFields: request on: stream
	| referer userAgent |
	referer := request headers at: 'Referer' ifAbsent: [ '-' ].
	userAgent := request headers at: 'User-Agent' ifAbsent: [ '-' ].
	stream space.
	stream nextPut: $"; << referer; nextPut: $".
	stream space.
	stream nextPut: $"; << userAgent; nextPut: $"
! !

!ZnCommonLogFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/1/2015 16:30'!
formatRequest: event on: stream
	| requestLine |
	stream space.
	requestLine := event request requestLine.
	stream << $".
	requestLine printMethodAndUriOn: stream.
	stream space; << requestLine version.
	stream << $"! !

!ZnCommonLogFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/1/2015 16:30'!
formatResponse: event on: stream
	| size |
	stream space.
	stream print: event response code.
	stream space.
	size := event response hasEntity 
		ifTrue: [ event response contentLength ] 
		ifFalse: [ 0 ].
	stream print: size
! !

!ZnCommonLogFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/1/2015 15:35'!
formatTimestamp: timestamp on: stream
	stream nextPut: $[.
	timestamp dayOfMonth printOn: stream base: 10 length: 2 padded: true.
	stream nextPut: $/.
	stream << timestamp monthAbbreviation.
	stream nextPut: $/.
	timestamp year printOn: stream.
	stream << $:.
	timestamp printHMSOn: stream.
	stream space.
	stream nextPut: (timestamp offset positive ifTrue: [ $+ ] ifFalse: [ $- ]).
	timestamp offset hours abs printOn: stream base: 10 length: 2 padded: true.
	stream nextPut: $:.
	timestamp offset minutes abs printOn: stream base: 10 length: 2 padded: true.
	stream nextPut: $]! !

!ZnCommonLogFormat methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 7/1/2015 14:22'!
initialize
	super initialize.
	combined := true.! !

!ZnCommonLogFormat methodsFor: 'private' stamp: 'SvenVanCaekenberghe 7/1/2015 16:31'!
userFrom: event
	^ [ event request basicAuthentication first ]
		on: Error
		do: [ '-' ]! !

Object subclass: #ZnConstants
	instanceVariableNames: ''
	classVariableNames: 'DefaultMaximumEntitySize HTTPStatusCodes'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnConstants commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:01' prior: 0!
I am ZnConstants, holding various constants that we use.

Part of Zinc HTTP Components.!

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:18'!
defaultHTTPVersion
	^ 'HTTP/1.1'! !

!ZnConstants class methodsFor: 'resource limits' stamp: 'SvenVanCaekenberghe 9/5/2012 12:34'!
defaultMaximumEntitySize
	"Return the default maximum entity size to accept from a stream.
	Used by ZnEntity and subclasses. Can be set using #defaultMaximumEntitySize:
	Default value is 16Mb. This affects both client and server code.
	This helps to protect us from malicious content."
	
	^ DefaultMaximumEntitySize ifNil: [ DefaultMaximumEntitySize := 16 * 1024 * 1024 ] ! !

!ZnConstants class methodsFor: 'resource limits' stamp: 'SvenVanCaekenberghe 9/5/2012 12:35'!
defaultMaximumEntitySize: integer
	"Set the default maximum entity size to accept from a stream to integer bytes."
	
	^ DefaultMaximumEntitySize := integer! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/31/2011 08:54'!
defaultServerString 
	^ self frameworkNameAndVersion ! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:49'!
defaultUserAgent
	^ self frameworkNameAndVersion ! !

!ZnConstants class methodsFor: 'accessing' stamp: 'PaulDeBruicker 4/10/2011 11:54'!
faviconBytes
	^ #(0 0 1 0 1 0 16 16 16 0 0 0 0 0 40 1 0 0 22 0 0 0 40 0 0 0 16 0 0 0 32 0 0 0 1 0 4 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 255 255 255 0 0 189 255 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 1 17 17 17 1 16 1 16 1 17 17 17 1 16 1 16 1 16 0 0 1 16 1 16 0 17 0 0 1 16 1 16 0 1 16 0 1 16 1 16 0 0 17 0 1 17 17 16 0 0 1 16 1 1 17 0 0 0 0 17 0 0 0 0 1 17 17 17 0 0 0 0 1 17 17 17 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0) asByteArray! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:36'!
frameworkLicense
	"http://en.wikipedia.org/wiki/MIT_License"
	
	^ 'MIT License'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 17:12'!
frameworkMCVersion
	MCWorkingCopy managersForClass: self class do: [ :each | ^ each ancestors first name ]! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:35'!
frameworkName
	^ 'Zinc HTTP Components'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 10:47'!
frameworkNameAndVersion
	^ self frameworkName, ' ', self frameworkVersion ! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/31/2011 08:58'!
frameworkVersion
	^ '1.0'! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:53'!
httpStatusCodes
	^ HTTPStatusCodes ! !

!ZnConstants class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/6/2010 15:51'!
initialize 
	self initializeHTTPStatusCodes! !

!ZnConstants class methodsFor: 'private' stamp: 'KrisGybels 3/27/2014 17:35'!
initializeHTTPStatusCodes 
	"This is the official list from http://www.iana.org/assignments/http-status-codes"

	HTTPStatusCodes := Dictionary newFromPairs: #(
		100 'Continue'
		101 'Switching Protocols'
		102 'Processing'
		200 'OK'
		201 'Created'
		202 'Accepted'
		203 'Non-Authoritative Information'
		204 'No Content'
		205 'Reset Content'
		206 'Partial Content'
		207 'Multi-Status'
		208 'Already Reported'
		226 'IM Used'
		300 'Multiple Choices'
		301 'Moved Permanently'
		302 'Found'
		303 'See Other'
		304 'Not Modified'
		305 'Use Proxy'
		307 'Temporary Redirect'
		308 'Permanent Redirect'
		400 'Bad Request'
		401 'Unauthorized'
		402 'Payment Required'
		403 'Forbidden'
		404 'Not Found'
		405 'Method Not Allowed'
		406 'Not Acceptable'
		407 'Proxy Authentication Required'
		408 'Request Timeout'
		409 'Conflict'
		410 'Gone'
		411 'Length Required'
		412 'Precondition Failed'
		413 'Payload Too Large'
		414 'URI Too Long'
		415 'Unsupported Media Type'
		416 'Requested Range Not Satisfiable'
		417 'Expectation Failed'
		422 'Unprocessable Entity'
		423 'Locked'
		424 'Failed Dependency'
		426 'Upgrade Required'
		428 'Precondition Required'
		429 'Too Many Requests'
		431 'Request Header Fields Too Large'
		500 'Internal Server Error'
		501 'Not Implemented'
		502 'Bad Gateway'
		503 'Service Unavailable'
		504 'Gateway Timeout'
		505 'HTTP Version Not Supported'
		506 'Variant Also Negotiates (Experimental)'
		507 'Insufficient Storage'
		508 'Loop Detected'
		510 'Not Extended'
		511 'Network Authentication Required')! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/11/2011 18:07'!
knownHTTPMethods
	^ #(
		"Standard HTTP 1.1 Verbs"
		GET PUT POST DELETE HEAD 
		TRACE OPTIONS CONNECT PATCH
		"WebDAV verbs"
		PROPFIND PROPPATCH MKCOL COPY MOVE LOCK UNLOCK)! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:22'!
knownHTTPVersions
	^ #('HTTP/1.1' 'HTTP/1.0')! !

!ZnConstants class methodsFor: 'resource limits' stamp: 'SvenVanCaekenberghe 9/5/2012 12:36'!
maximumEntitySize
	"Return the maximum entity size to accept from a stream.
	Used by ZnEntity and subclasses. Can be set using #maximumEntitySize:
	This affects both client and server code.
	This helps to protect us from malicious content."
	
	^ ZnMaximumEntitySize value 
		ifNil: [ self defaultMaximumEntitySize ]! !

!ZnConstants class methodsFor: 'resource limits' stamp: 'SvenVanCaekenberghe 9/5/2012 12:33'!
maximumEntitySize: integer
	"Set the maximum entity size to accept from a stream to integer bytes."
	
	^ DefaultMaximumEntitySize := integer! !

!ZnConstants class methodsFor: 'resource limits' stamp: 'SvenVanCaekenberghe 1/2/2012 17:45'!
maximumLineLength
	"Return the maximum line length to accept.
	Used by ZnLineReader and thus for reading request/status lines as well as headers.
	This helps to protect us from malicious content."
	
	^ 4096! !

!ZnConstants class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/9/2010 20:19'!
remoteAddressHeader
	^ 'X-Zinc-Remote-Address'! !

Object subclass: #ZnCookie
	instanceVariableNames: 'attributes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCookie commentStamp: 'SvenVanCaekenberghe 12/7/2010 00:01' prior: 0!
I represent an HTTP Cookie token.

Part of Zinc HTTP Components.!

!ZnCookie class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/30/2011 19:46'!
fromString: aString
	^ self new 
		fromString: aString; 
		yourself! !

!ZnCookie class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/30/2011 19:46'!
fromString: aString for: urlObject
	| cookie |
	cookie := self fromString: aString.
	"Ensures a domain value if none provided by the cookie"
	cookie defaultDomain: urlObject host.
	^ cookie! !

!ZnCookie class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 11/10/2012 19:25'!
name: nameString value: value
	^ self new
		name: nameString;
		value: value;
		yourself! !

!ZnCookie methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:05'!
= aCookie
	(aCookie isKindOf: self class) ifFalse: [ ^ false ].
	^ (aCookie name = self name) & (aCookie path = self path) & (aCookie domain = self domain)
	! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/30/2011 19:45'!
defaultDomain: aDomain
	self domain ifNil: [ self domain: aDomain ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:04'!
domain
	^ attributes at: 'domain' ifAbsent: [ nil ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:25'!
domain: aDomain
	attributes at: 'domain' put: aDomain! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:04'!
expires
	^ attributes at: 'expires' ifAbsent: [ nil ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:26'!
expires: anExpiration
	attributes at: 'expires' put: anExpiration! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2013 11:31'!
expiresTimeStamp
	self expires ifNil: [ ^ DateAndTime now + 1 day ].
	^ ZnUtils parseHttpDate: self expires! !

!ZnCookie methodsFor: 'private' stamp: 'SvenVanCaekenberghe 8/18/2011 14:17'!
extraAttributes
	"Answer all attributes except name and value"
	^ attributes associationsSelect: [ :ea | (ea key = 'name' or: [ ea key = 'value' ]) not ]! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/28/2015 18:15'!
fromString: aString
	| tokens val data i |
	tokens := aString substrings: ';'.
	i := tokens first indexOf: $=.
	val := tokens first copyFrom: 1 to: i - 1.
	data := tokens first copyFrom: i + 1 to: tokens first size. 
	self name: val trimBoth; value: data trimBoth.
	2 to: tokens size do: [ :index | 
		self readAttribute: (tokens at: index) ]! !

!ZnCookie methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 8/30/2011 21:43'!
fullString
	^ String streamContents: [ :stream | 
		self writeOn: stream ]! !

!ZnCookie methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 8/30/2011 20:02'!
hash
	^ (self name hash bitXor: self path hash) bitXor: self domain hash! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:03'!
httponly
	"This is not an RFC cookie attribute, but a supplement found in MS Internet Explorer and supported in PHP.
	 See http://msdn.microsoft.com/en-us/library/ms533046.aspx"
	^ attributes at: 'httponly' ifAbsent: [ false ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:28'!
httponly: aBoolean
	attributes at: 'httponly' put: aBoolean! !

!ZnCookie methodsFor: 'initialization' stamp: 'MattKennedy 9/27/2010 20:21'!
initialize 
	attributes := Dictionary new.! !

!ZnCookie methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/10/2013 11:31'!
isExpired
	^ (self expiresTimeStamp asUTC < DateAndTime now asUTC) or: [ self expires = 0 ]! !

!ZnCookie methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/23/2012 14:34'!
isInDomain: urlObject
	| domain |
	domain := (self domain beginsWith: '.')
		ifTrue: [ self domain copyFrom: 2 to: self domain size ]
		ifFalse: [ self domain ].
	^ urlObject host includesSubstring: domain
	! !

!ZnCookie methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/23/2012 14:34'!
isInPath: urlObject
	^ urlObject pathPrintString includesSubstring: self path
	! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
maxage
	^ self expires! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:19'!
name
	^ attributes at: 'name'! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:23'!
name: aName
	attributes at: 'name' put: aName! !

!ZnCookie methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 8/30/2011 21:42'!
nameValueString
	^ String streamContents: [ :stream | 
		self writeNameAndValueOn: stream ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:04'!
path
	^ attributes at: 'path' ifAbsent: [ '/' ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:08'!
path: aPath
	attributes at: 'path' put: aPath! !

!ZnCookie methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/30/2011 21:43'!
printOn: stream
	self writeOn: stream! !

!ZnCookie methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/8/2012 14:16'!
readAttribute: aToken
	| i key data |
	i := aToken indexOf: $=.
	i > 0
		ifTrue: [
			key := (aToken copyFrom: 1 to: i - 1) trimBoth asLowercase.
			data := (aToken copyFrom: i + 1 to: aToken size) trimBoth ]
		ifFalse: [ 
			key := (aToken copyFrom: 1 to: aToken size) trimBoth asLowercase.
			data := true ].
	attributes at: key put: data! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:05'!
secure
	^ attributes at: 'secure' ifAbsent: [ false ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:28'!
secure: aBoolean
	attributes at: 'secure' put: aBoolean! !

!ZnCookie methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:05'!
value
	^ attributes at: 'value' ifAbsent: [ nil ]! !

!ZnCookie methodsFor: 'accessing' stamp: 'MattKennedy 9/27/2010 20:23'!
value: aValue
	attributes at: 'value' put: aValue! !

!ZnCookie methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/30/2011 20:03'!
writeExtraAttributesOn: aStream
	self extraAttributes
		associationsDo: [ :ea | 
			aStream
				nextPutAll: '; ';
				nextPutAll: ea key.
			(ea value isKindOf: Boolean)
				ifFalse: [ 
					aStream
						nextPutAll: '=';
						nextPutAll: ea value ] ]! !

!ZnCookie methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:37'!
writeNameAndValueOn: aStream
	aStream
		nextPutAll: self name;
		nextPutAll: '=';
		nextPutAll: self value! !

!ZnCookie methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/30/2011 21:32'!
writeOn: aStream
	self
		writeNameAndValueOn: aStream;
		writeExtraAttributesOn: aStream ! !

Object subclass: #ZnCookieJar
	instanceVariableNames: 'cookies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCookieJar commentStamp: 'SvenVanCaekenberghe 12/7/2010 00:01' prior: 0!
I represent a container for storing HTTP Cookies for a client session.

Part of Zinc HTTP Components.!

!ZnCookieJar methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:18'!
add: aCookie
	(self removeIfExpired: aCookie) ifTrue: [ ^ nil ].
	self cookies do: [ :cookie |
		cookie = aCookie ifTrue: [ self cookies remove: cookie ] ].
	self cookies add: aCookie
	! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:07'!
cookieAt: aName forUrl: urlObject
	(self cookiesForUrl: urlObject) do: [ :cookie | cookie name = aName ifTrue: [ ^ cookie ] ].
	^ nil! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:07'!
cookies 
	^ cookies ifNil: [ cookies := Bag new ]! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:06'!
cookiesForUrl: urlObject
	^ self cookies select: [ :cookie | 
		(cookie isExpired not) & (cookie isInDomain: urlObject) & (cookie isInPath: urlObject) ]! !

!ZnCookieJar methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:06'!
remove: aCookie
	self cookies remove: aCookie ifAbsent: [ nil ]! !

!ZnCookieJar methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 2/29/2012 17:39'!
removeAll
	cookies ifNotNil: [ cookies removeAll ]! !

!ZnCookieJar methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:07'!
removeIfExpired: aCookie
	^ aCookie isExpired 
		ifTrue: [ 
			self remove: aCookie. 
			true ]
		ifFalse: [
			false ]! !

Object subclass: #ZnCredential
	instanceVariableNames: 'domain realm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:56' prior: 0!
I am ZnCredential, an abstract class representing credentials in a domain and realm.

Part of Zinc HTTP Components.!

ZnCredential subclass: #ZnBasicCredential
	instanceVariableNames: 'username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnBasicCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:58' prior: 0!
I am ZnBasicCredential, a concrete subclass of ZnCredential.
I implement basic HTTP authentication.
I hold a username and password.

Part of Zinc HTTP Components.!

!ZnBasicCredential class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
authName
	^ 'Basic'! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
authTokenForUrl: urlObject method: method
	^ 'Basic ', (ZnUtils encodeBase64: (username, ':', password)) ! !

!ZnBasicCredential methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
isComplete 
	^ username notNil & password notNil ! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
password
	^ password! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
password: anObject
	password := anObject! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
username
	^ username! !

!ZnBasicCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
username: anObject
	username := anObject! !

ZnBasicCredential subclass: #ZnDigestCredential
	instanceVariableNames: 'nonce opaque'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnDigestCredential commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:59' prior: 0!
I am ZnDigestCredential, a concrete subclass of ZnCredential and subclass of ZnBasicCredential.
I implement digest HTTP authentication.
I hold a nounce and opaque.

Part of Zinc HTTP Components.!

!ZnDigestCredential class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
authName
	^ 'Digest'! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
a1
	^ ZnDigestAuthenticator md5Hash: self username, ':', self realm, ':', self password! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
a2ForUrl: urlObject method: method
	"This doesn't support all qop options yet."
	^ ZnDigestAuthenticator md5Hash: method, ':', (self uriPathFrom: urlObject)! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:37'!
authTokenForUrl: urlObject method: method
	| result |
	result := WriteStream on: String new.
	result
		nextPutAll: 'Digest username="';
		nextPutAll: self username;
		nextPutAll: '", ';
		nextPutAll: 'nonce="';
		nextPutAll: self nonce;
		nextPutAll: '", ';
		nextPutAll: 'realm="';
		nextPutAll: self realm;
		nextPutAll: '", ';
		nextPutAll: 'uri="';
		nextPutAll: (self uriPathFrom: urlObject);
		nextPutAll: '", ';
		nextPutAll: 'response="';
		nextPutAll: (self responseForUrl: urlObject method: method);
		nextPutAll: '", '.
	self opaque isNil
		ifFalse: [ 
			result
				nextPutAll: 'opaque="';
				nextPutAll: self opaque;
				nextPutAll: '"' ].
	^ result contents! !

!ZnDigestCredential methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
isComplete 
	^ super isComplete & nonce notNil ! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:08'!
linkToAuthRequest: aHeader
	| data |
	data := self parseAuthRequest: aHeader.
	self nonce: (data at: 'nonce').
	self realm: (data at: 'realm' ifAbsent: [ '' ])! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
nonce
	^ nonce! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
nonce: anObject
	nonce := anObject! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
opaque
	^ opaque! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:48'!
opaque: anObject
	opaque := anObject! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
parseAuthRequest: aHeader
	^ ZnDigestAuthenticator parseAuthRequest: aHeader! !

!ZnDigestCredential methodsFor: 'computation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:21'!
responseForUrl: urlObject method: method
	^ ZnDigestAuthenticator md5Hash: self a1, ':', self nonce, ':', (self a2ForUrl: urlObject method: method)! !

!ZnDigestCredential methodsFor: 'operations' stamp: 'MattKennedy 9/29/2010 16:28'!
setAuthorizationHeader: headers for: urlObject method: method
	headers at: 'Authorization' put: (self authTokenForUrl: urlObject method: method)! !

!ZnDigestCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:08'!
uriPathFrom: urlObject
	^ urlObject pathPrintString ifNil: [ '/' ]! !

!ZnCredential class methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:28'!
authName
	self subclassResponsibility ! !

!ZnCredential class methodsFor: 'accessing class hierarchy' stamp: 'SvenVanCaekenberghe 8/18/2011 14:09'!
findHandlerFor: anAuthHeader
	| type i |
	i := anAuthHeader indexOf: Character space.
	type := anAuthHeader copyFrom: 1 to: i - 1.
	self allSubclassesDo: [ :subclass |
		subclass authName = type ifTrue: [ ^ subclass ] ].
	^ nil! !

!ZnCredential class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/28/2015 18:15'!
findRealmFor: anAuthHeader
	(anAuthHeader substrings: ',') do: [ :token |
		"String>>trimBoth: is a grease method, so not sure if we should be using it."
		(token includesSubstring: 'realm=') 
			ifTrue: [ ^ ((token substrings: '=') second) ] ].
	^ nil! !

!ZnCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:20'!
authToken
	^ self authTokenForUrl: nil method: nil ! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 13:48'!
authTokenForUrl: urlObject method: method
	self subclassResponsibility ! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
domain
	^ domain! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
domain: anObject
	domain := anObject! !

!ZnCredential methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:20'!
isComplete
	^ self subclassResponsibility ! !

!ZnCredential methodsFor: 'operations' stamp: 'MattKennedy 9/29/2010 16:10'!
linkToAuthRequest: aHeader
	"Link this credential to the WWW-Authenticate request in the provided header. 
	Used for authentication methods which require parameters provided by the server
	(e.g. Digest authentication)."
	realm := self class findRealmFor: aHeader! !

!ZnCredential methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:07'!
realm
	^ realm ifNil: [ realm := '' ]! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:27'!
realm: anObject
	realm := anObject! !

!ZnCredential methodsFor: 'accessing' stamp: 'MattKennedy 9/29/2010 13:42'!
setAuthorizationHeader: headers for: urlObject method: method
	headers at: 'Authorization' put: self authToken! !

Object subclass: #ZnDefaultServerDelegate
	instanceVariableNames: 'prefixMap byteArray'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnDefaultServerDelegate commentStamp: 'SvenVanCaekenberghe 6/9/2015 15:36' prior: 0!
I am ZnDefaultServerDelegate.
I function as a delegate for ZnServer, implementing #handleRequest:

I implement responses to the following prefixes:

/echo - an echo text of request information for debugging purposes
/dw-bench - a dynamic html page for benchmarking purposes
/unicode - a unicode test page
/random - a random string (/random/32 for a specific size)
/bytes - bytes according to a pattern (/bytes/32 for a specific size)
/favicon.ico - a Zn favicon
/status - a server status page
/error - force server errors
/session - a simpler session based counter
/help - lists all page prefixes
/ - an html welcome page

Without any matches, I respond with a page not found.

I can be reused without any default handlers mapped using #empty and configured using #map:to: where the second argument can be a block.

Part of Zinc HTTP Components.!

!ZnDefaultServerDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 6/9/2015 15:33'!
empty
	^ self basicNew! !

!ZnDefaultServerDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/10/2010 15:17'!
new
	^ super new initialize! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 8/18/2011 14:09'!
bytes: request
	"Answer a number of bytes according to a pattern as an application/octet-stream.
	Optionally take the size from the URI, as in /bytes/32 for 32 bytes, default to 64 bytes.
	The main goal is to be efficient, esp. in benchmarks, that's why we cache (non-thread-safe)"
	
	| count |
	count := Integer readFrom: request uri lastPathSegment ifFail: [ 64 ].
	(byteArray isNil or: [ byteArray size ~= count ]) 
		ifTrue: [
			byteArray := ByteArray new: count.
			1 to: count - 1 do: [ :each | 
				byteArray at: each put: (#(65 66 67 68) at: (each \\ 4 + 1)) ].
			byteArray at: count put: 10 ].
	^ ZnResponse ok: (ZnEntity bytes: byteArray)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:30'!
dwbench: request
	"Reply with the dynamic DW-Bench HTML page"

	^ ZnResponse ok: (ZnEntity html: self generateDWBench)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 5/27/2013 16:11'!
echoRequest: request	
	"Echo request by generating a plain text response useful for debugging purposes"
	
	| delay info entity |
	delay := Integer readFrom: (request uri queryAt: 'delay' ifAbsent: [ '' ]) ifFail: [ 0 ].
	delay > 0 ifTrue: [ (Delay forSeconds: delay) wait ].  
	info := self generateEchoRequestString: request.
	entity := ZnEntity textCRLF: info.
	^ ZnResponse ok: entity! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/23/2012 18:03'!
errorResponse: request
	"Here we artificially signal an Error at the Smalltalk level,
	optionally with a user defined message."
	
	| message |
	message := request uri queryAt: 'message' ifAbsent: [ 'An artifical error' ].
	Error signal: message! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:39'!
favicon: request
	"Answer a nice favicon for browsers to display"

	| entity |
	entity := ZnEntity with: ZnConstants faviconBytes type: 'image/vnd.microsoft.icon'. 
	^ ZnResponse ok: entity! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 11/30/2015 14:49'!
formTest1: request
	| input page |
	input := request uri queryAt: #input ifAbsent: [ 'input' ].
	page := ZnHtmlOutputStream streamContents: [ :html |
		html page: 'Form Test 1' do: [ 
			html 
				tag: #form 
				attributes: #(action 'form-test-1' 'accept-charset' 'utf-8' method GET) 
				do: [ 
					html 
						str: 'Input'; space;
						tag: #input attributes: { #type. #input. #name. #input. #value. input }; space;
						tag: #input attributes: #(type submit); 
						str: 'for input'; space; tag: #em with: input ] ] ].
	^ ZnResponse ok: (ZnEntity html: page)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 11/30/2015 14:50'!
formTest2: request
	| input page |
	input := 'input'.
	(request hasEntity and: [ request contentType matches: ZnMimeType applicationFormUrlEncoded  ])
		ifTrue: [ input := request entity at: #input ifAbsent: [ 'input' ] ].
	page := ZnHtmlOutputStream streamContents: [ :html |
		html page: 'Form Test 2' do: [ 
			html 
				tag: #form 
				attributes: #(action 'form-test-2' 'accept-charset' 'utf-8' method POST) 
				do: [ 
					html 
						str: 'Input'; space;
						tag: #input attributes: { #type. #input. #name. #input. #value. input }; space;
						tag: #input attributes: #(type submit); 
						str: 'for input'; space; tag: #em with: input ] ] ].
	^ ZnResponse ok: (ZnEntity html: page)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 11/30/2015 15:15'!
formTest3: request
	| contents filename contentType page |
	contents := filename := contentType := ''.
	(request hasEntity and: [ request contentType matches: ZnMimeType multiPartFormData ])
		ifTrue: [ 
			(request entity partNamed: #file ifNone: [ nil ]) 
				ifNotNil: [ :part |
					filename := part fileName.
					contents := part contents.
					contentType := part  contentType.
					contentType isBinary ifTrue: [ contents := contents hex ] ] ].
	page := ZnHtmlOutputStream streamContents: [ :html |
		html page: 'Form Test 3' do: [ 
			html 
				tag: #form 
				attributes: #(action 'form-test-3' 'accept-charset' 'utf-8' 
									enctype 'multipart/form-data' method POST) 
				do: [ 
					html 
						str: 'File'; space;
						tag: #input attributes: #(type file name file); space;
						tag: #input attributes: #(type submit) ];
				tag: #p do: [ html str: 'filename = '; str: filename ];
				tag: #p do: [ html str: 'content-type = '; str: contentType asString ];
				tag: #p do: [ html str: 'contents = '; str: contents asString ] ] ].
	^ ZnResponse ok: (ZnEntity html: page)! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/30/2015 14:06'!
generateDWBench
	"Generate the dynamic DW-Bench HTML page.
	Although the date/time is variable, the page size is constant."
	
	^ ZnHtmlOutputStream streamContents: [ :html |
		html page: 'DW-Bench Dynamic' do: [ 
			html tag: #table attributes: #(border 1) do: [
				1 to: 25 do: [ :row |
					html tag: #tr do: [
						1 to: 25 do: [ :col |
							html tag: #td do: [ html print: row * col ] ] ] ] ].
			html tag: #p do: [ 
				html << Date today yyyymmdd; space; << Time now print24 ] ] ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/7/2013 15:02'!
generateEchoRequestString: request
	"Generate a string echoing the request"
	
	^ String streamContents: [ :stream |
		stream nextPutAll: 'This is Zinc HTTP Components echoing your request !!'; crlf.
		stream nextPutAll: 'Running '; print: request server; crlf.
		stream nextPutAll: request method; nextPutAll: ' request for '.
		request uri printPathQueryFragmentOn: stream.
		stream crlf; nextPutAll: 'with headers'; crlf.
		request headersDo: [ :key :value |
			stream space; nextPutAll: key; nextPutAll: ': '; nextPutAll: value; crlf ].
		request hasEntity ifTrue: [
			stream nextPutAll: ' containing '; print: request entity ] ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/30/2015 14:22'!
generateHelp
	"Generate an HTML page with links to all pages I support"
	
	^ ZnHtmlOutputStream streamContents: [ :html |
		html page: (self class name, ' Help') do: [  	
			html tag: #h3 with: 'Available Pages'.
			html tag: #ul do: [
				prefixMap keys sorted do: [ :each | 
					html tag: #li do: [ 
						html tag: #a attributes: { #href. each } with: each ] ] ] ] ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/30/2015 14:21'!
generateSessionRequest: session
	"Generate an HTML page using a session"
	
	^ ZnHtmlOutputStream streamContents: [ :html |
		html page: 'Session' do: [ 
			html tag: #p do: [ html << 'I am using '; str: session asString ].
			html tag: #p do: [ html << 'Current session id is ' << session id ].
			html tag: #p do: [ html << 'Session hit count ' << (session attributeAt: #hitCount) ] ] ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/30/2015 14:36'!
generateStatus
	"Generate an HTML page with the state of the server"
	
	^ ZnHtmlOutputStream streamContents: [ :html |
		html page: 'ZnServer Status' do: [
			html tag: #p do: [ html << 'I am '; print: ZnCurrentServer value ].
			html tag: #p with: self systemVersionInfo.
			html tag: #pre with: self vmStats.
			html tag: #h1 with: 'Processes'.
			html tag: #ol do: [ 
				self processes do: [ :each |
					html tag: #li do: [ 
						html 
							str: each name; space; nextPut: $[; print: each priority; nextPut: $]; 
							tag: #br; str: each printString ] ] ] ] ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/30/2015 14:28'!
generateUnicodeTest
	"Generate an HTML page containing all Unicode characters with code points between 0 and 16r024F in a nice table"
	
	^ ZnHtmlOutputStream streamContents: [ :html |
		html page: 'Unicode Test Page' do: [ 
			html tag: #table do: [
				0 to: 16r024F by: 8 do: [ :row |
					html tag: #tr do: [
					row to: row + 7 do: [ :each |
						html tag: #td do: [ 
							html 
								nextPutAll: 'U+'; nextPutAll: (each printPaddedWith: $0 to: 4 base: 16);
								nextPut: $=; nextPut: each asCharacter ] ] ] ] ] ] ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/30/2015 16:14'!
generateWelcomePage
	^ ZnHtmlOutputStream streamContents: [ :html |
		html html5; tag: #html do: [
			html tag: #head do: [
				html tag: #title with: 'Zinc HTTP Components'.
				html tag: #style attributes: #(type 'text/css') with: self welcomePageCss ].
		html tag: #body do: [ 
			html tag: #div id: #logo with: #Zn.
			html tag: #h1 with: 'Zinc HTTP Components'.
			html tag: #p with: self welcomePageGreeting.
			self welcomePageLinksOn: html.
			html tag: #h4 with: 'May the Source be with you!!' ] ] ]! !

!ZnDefaultServerDelegate methodsFor: 'public' stamp: 'PaulDeBruicker 4/19/2012 16:02'!
handleRequest: request
	| prefix prefixHandler |
	prefix := request uri isSlash
		ifTrue: [ prefixMap at: '/' ifAbsent: [ nil ] ]
		ifFalse: [ request uri firstPathSegment ].
	prefixHandler := prefixMap at: prefix ifAbsent: [ nil ].
	prefixHandler isNil
		ifFalse: [ 
			^ prefixHandler isSymbol
				ifTrue: [ self perform: prefixHandler with: request ]
				ifFalse: [ prefixHandler value: request ] ].
	^ ZnResponse notFound: request uri! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 3/31/2011 09:17'!
help: request
	"Reply with a dynamic HTML page containing links to all pages I support"

	^ ZnResponse ok: (ZnEntity html: self generateHelp)
	! !

!ZnDefaultServerDelegate methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 11/30/2015 16:22'!
initialize
	self
		map: 'welcome' to: #welcome:;
		map: 'help' to: #help:;
		map: 'echo' to: #echoRequest:;
		map: 'small' to: #small:;
		map: 'dw-bench' to: #dwbench:;
		map: 'favicon.ico' to: #favicon:;
		map: 'random' to: #random:;
		map: 'bytes' to: #bytes:;
		map: 'status' to: #status:;
		map: 'unicode' to: #unicode:;
		map: 'session' to: #sessionRequest:;
		map: 'error' to: #errorResponse:;
		map: 'form-test-1' to: #formTest1:;
		map: 'form-test-2' to: #formTest2:;
		map: 'form-test-3' to: #formTest3:;
		map: '/' to: 'welcome'! !

!ZnDefaultServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/9/2015 15:49'!
map: prefix to: handler
	"Arrange for the receiver to dispatch requests matching prefix to handler.
	Handler can be a symbol naming a method in the receiver, 
	or a block accepting the request and producing the response.
	The special prefix '/' is resolved first and can refer to another prefix."
	
	self prefixMap 
		at: prefix
		put: handler
		! !

!ZnDefaultServerDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/21/2012 10:36'!
prefixMap
	^ prefixMap ifNil: [ prefixMap := Dictionary new ]! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/31/2011 09:46'!
processes
	^ (Process allSubInstances
		reject: [ :each | each isTerminated or: [ each = Processor activeProcess ] ])
		sorted: [ :a :b | a priority >= b priority ] ! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 1/4/2011 14:52'!
random: request
	"Answer a text plain UTF-8 encoded string of random hex characters.
	Optionally take the size from the URI, as in /random/32 for 32 characters.
	Default to 64 characters with a newline at the end"
	
	| count hexString |
	count := Integer readFrom: request uri lastPathSegment ifFail: [ 64 ].
	hexString := String new: count streamContents: [ :stream |
		count - 1 timesRepeat: [ stream nextPut: '0123456789ABCDEF' atRandom ].
		stream lf ].
	^ ZnResponse ok: (ZnEntity text: hexString)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 11/10/2012 23:41'!
sessionRequest: request	
	"An HTML response that uses a session"
	
	| session hitCount |
	session := request session.
	hitCount := session attributeAt: #hitCount ifAbsent: [ 0 ].
	hitCount := hitCount + 1.
	session attributeAt: #hitCount put: hitCount. 
	^ ZnResponse ok: (ZnEntity html: (self generateSessionRequest: session))! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 11/30/2015 16:21'!
small: request
	| page |
	page := ZnHtmlOutputStream streamContents: [ :html |
		html page: #Small do: [ 
			html tag: #p with: 'This is a small HTML document' ] ].
	^ ZnResponse ok: (ZnEntity html: page)! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 3/31/2011 09:18'!
status: request
	"Reply with a dynamic HTML page describing the state of the server"

	^ ZnResponse ok: (ZnEntity html: self generateStatus)
	! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/1/2012 13:55'!
systemVersionInfo
	^ String streamContents: [ :stream |
			stream 
				print: SystemVersion current; 
				nextPutAll: ' - '; 
				nextPutAll: ZnConstants defaultServerString ]
	! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 12/14/2010 10:40'!
unicode: request
	"Answer a Unicode Test HTML page"
	
	^ ZnResponse ok: (ZnEntity html: self generateUnicodeTest)! !

!ZnDefaultServerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/22/2012 14:40'!
value: request
	"I implement the generic #value: message as equivalent to #handleRequest:"
	
	^ self handleRequest: request! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 3/1/2012 13:54'!
vmStats
	^  [ SmalltalkImage current vm statisticsReport ]
		on: MessageNotUnderstood 
		do: [ SmalltalkImage current vmStatisticsReportString ]! !

!ZnDefaultServerDelegate methodsFor: 'responses' stamp: 'SvenVanCaekenberghe 11/30/2015 15:43'!
welcome: request
	"Answer a nice HTML welcome page"
	
	| page entity |
	page := self generateWelcomePage.
	entity := ZnEntity html: page. 
	^ ZnResponse ok: entity! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/30/2015 16:10'!
welcomePageCss
	^ Character space join: 
	'body { color: black; background: white; width: 900px; font-family: Verdana, Arial, Helvetica, sans-serif; font-size: 13px }
p { width: 600px; padding: 0 20px 10px 20px }
ul, ol { width: 600px; padding: 0 5px 5px 30px } 
#logo { color: orange; font-family: Helvetica, sans-serif; font-weight: bold; font-size: 100px }' lines! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/30/2015 16:14'!
welcomePageGreeting
	^ 'Welcome to Zinc HTTP Components, a modern, open-source Smalltalk framework to deal with the HTTP networking protocol.'! !

!ZnDefaultServerDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/30/2015 16:12'!
welcomePageLinksOn: html
	html tag: #ul do: [ 
		html tag: #li do: [ 
			html 
				str: 'Project homepage'; space;  
				tag: #a attributes: #(href 'http://zn.stfx.eu') with: 'http://zn.stfx.eu' ].
		html tag: #li do: [ 
			html 
				str: 'Locally available pages'; space;  
				tag: #a attributes: #(href '/help') with: '/help' ] ]! !

Object subclass: #ZnDispatcherDelegate
	instanceVariableNames: 'dispatcherDictionary'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnDispatcherDelegate commentStamp: 'SvenVanCaekenberghe 5/15/2012 16:03' prior: 0!
I am ZnDispatcherDelegate, doing a straight-forward dispatching to mapped URLs.

ZnDispatcherDelegate API:

(server := ZnServer startDefaultOn: 9090)
	delegate: (ZnDispatcherDelegate new 
		map: '/hello' to: [ :request :response | response entity: (ZnEntity html: '<h1>hello server</h1>') ];
		map: '/counter' to: [ :request :response | counterApplication handleRequest: request response: response ]).!

!ZnDispatcherDelegate methodsFor: 'accessing' stamp: 'NickAger 5/9/2011 16:25'!
dispatcherDictionary
	^ dispatcherDictionary ifNil: [ dispatcherDictionary := Dictionary new ]! !

!ZnDispatcherDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/18/2011 14:16'!
handleRequest: request
	| handler |
	handler := self dispatcherDictionary 
		at: request uri pathPrintString 
		ifAbsent: [ ^ ZnResponse notFound: request uri ].	
	^ handler 
		value: request 
		value: (ZnResponse new
			statusLine: (ZnStatusLine ok);
			headers: ZnHeaders defaultResponseHeaders;
			yourself).

! !

!ZnDispatcherDelegate methodsFor: 'public' stamp: 'NickAger 5/9/2011 16:19'!
map: relativeUrlString to: handler
	self dispatcherDictionary at: relativeUrlString put: handler! !

Object subclass: #ZnEasy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnEasy commentStamp: '<historical>' prior: 0!
I am ZnEasy, a simplified HTTP client for occasional use.

My class side implements a number of convenience methods to do common HTTP client operations.

	ZnEasy get: 'http://zn.stfx.eu/zn/small.html'.

For most requests, I return a response object. 

For a couple of message, I return an image Form.

	ZnEasy getPng: 'http://www.pharo-project.org/images/pharo.png'.

For my implementation I use ZnClient, a full featured HTTP client.

Part of Zinc HTTP Components.!

!ZnEasy class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/8/2011 22:16'!
client
	^ ZnClient new
		beOneShot;
		yourself! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
delete: urlObject
	^ self client
		url: urlObject;
		delete;
		response! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
delete: urlObject username: username password: password
	^ self client
		url: urlObject;
		setBasicAuthenticationUsername: username password: password;
		delete;
		response
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/1/2014 16:10'!
get: urlObject
	"self get: 'http://caretaker.wolf359.be/small.html'."
	
	^ self client
		url: urlObject;
		get;
		response
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
get: urlObject username: username password: password
	^ self client
		url: urlObject;
		setBasicAuthenticationUsername: username password: password;
		get;
		response
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 7/4/2012 17:56'!
getGif: urlObject
	"self getGif: 'http://zn.stfx.eu/zn/Hot-Air-Balloon.gif'."
	
	^ self 
		getImageOfType: ZnMimeType imageGif 
		usingParser: GIFReadWriter 
		fromUrl: urlObject 
	! !

!ZnEasy class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
getImageOfType: mimeType usingParser: parserClass fromUrl: urlObject
	| client |
	(client := self client)
		url: urlObject;
		accept: mimeType;
		enforceHttpSuccess: true;
		enforceAcceptContentType: true;
		get.
	^ (parserClass on: client entity readStream) nextImage
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/1/2014 16:10'!
getJpeg: urlObject
	"self getJpeg: 'http://caretaker.wolf359.be/sun-fire-x2100.jpg'."
	
	^ self 
		getImageOfType: ZnMimeType imageJpeg 
		usingParser: JPEGReadWriter 
		fromUrl: urlObject 
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 8/1/2014 16:17'!
getPng: urlObject
	"self getPng: 'http://pharo.org/files/pharo.png'."
	"(self getPng: 'http://chart.googleapis.com/chart?cht=tx&chl=',
		'a^2+b^2=c^2' urlEncoded) asMorph openInHand."
	
	^ self 
		getImageOfType: ZnMimeType imagePng 
		usingParser: PNGReadWriter 
		fromUrl: urlObject 
	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
head: urlObject
	^ self client
		url: urlObject;
		head;
		response! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
head: urlObject username: username password: password
	^ self client
		url: urlObject;
		setBasicAuthenticationUsername: username password: password;
		head;
		response! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
post: urlObject data: entity
	^ self client
		url: urlObject;
		entity: entity;
		post;
		response! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
post: urlObject data: entity username: username password: password
	^ self client
		url: urlObject;
		setBasicAuthenticationUsername: username password: password;
		entity: entity;
		post;
		response! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
put: urlObject data: entity
	^ self client
		url: urlObject;
		entity: entity;
		put;
		response	! !

!ZnEasy class methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 9/15/2011 16:24'!
put: urlObject data: entity username: username password: password
	^ self client
		url: urlObject;
		setBasicAuthenticationUsername: username password: password;
		entity: entity;
		put;
		response	! !

Object subclass: #ZnEntity
	instanceVariableNames: 'contentType contentLength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:03' prior: 0!
I am ZnEntity, an abstract HTTP Entity.

An entity has a content type (mime-type), a content length (byte length) and a contents.

Entities can encode and decode themselves to and from binary streams.

Part of Zinc HTTP Components.!

ZnEntity subclass: #ZnApplicationFormUrlEncodedEntity
	instanceVariableNames: 'fields representation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnApplicationFormUrlEncodedEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:04' prior: 0!
I am ZnApplicationFormUrlEncodedEntity, a concrete HTTP Entity 
holding a set of fields (key value pairs) that is form URL encoded.
I am a ZnEntity.

Part of Zinc HTTP Components.!

!ZnApplicationFormUrlEncodedEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:41'!
designatedMimeType
	^ ZnMimeType applicationFormUrlEncoded ! !

!ZnApplicationFormUrlEncodedEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/13/2010 22:23'!
withAll: keyedCollection
	^ self new
		addAll: keyedCollection;
		yourself ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/16/2011 23:26'!
add: association
	self at: association key put: association value! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/24/2013 10:52'!
addAll: keyedCollection
	"Note that we use #addAllMulti:"

	self invalidateRepresentation.
	self fields addAllMulti: keyedCollection.
	^ keyedCollection ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/11/2010 21:07'!
at: key
	^ self fields at: key! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 11:23'!
at: key add: value
	self invalidateRepresentation.
	^ self fields at: key add: value! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/25/2013 14:11'!
at: key ifAbsent: block
	^ self fields at: key ifAbsent: block! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:23'!
at: key put: value
	self invalidateRepresentation.
	^ self fields at: key put: value! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/12/2010 10:19'!
computeRepresentation
	representation := String streamContents: [ :stream | self writeRepresentationOn: stream ].
	contentLength := representation size! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/12/2010 10:22'!
contentLength
	contentLength isNil ifTrue: [ self computeRepresentation ].
	^ contentLength ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 16:19'!
contents
	^ self fields! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/11/2010 21:00'!
fields
	^ fields! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 1/31/2011 12:16'!
fieldsDo: block
	self fields keysAndValuesDo: block! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/25/2013 14:07'!
includesField: field
	^ self fields includesKey: field! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 1/31/2011 12:15'!
initialize
	super initialize.
	fields := ZnMultiValueDictionary new.
	self contentType: self class designatedMimeType ! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/12/2010 10:17'!
invalidateRepresentation
	representation := nil.
	contentLength := nil! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:41'!
isEmpty
	^ self fields isEmpty! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printContentsOn: stream
	super printContentsOn: stream.
	self isEmpty
		ifTrue: [ ^ self ].
	stream space.
	fields printElementsOn: stream! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/25/2013 14:32'!
readFrom: stream
	"We parse from stream, limited to content length if available."
	
	| readStream |
	representation := nil.
	contentLength isNil
		ifTrue: [ readStream := ZnCharacterReadStream on: stream encoding: 'ascii' ]
		ifFalse: [ (readStream := ZnLimitedReadStream on: stream limit: contentLength) ascii ].
	fields := ZnResourceMetaUtils parseQueryFrom: readStream! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/18/2013 19:58'!
writeOn: stream
	"In most use cases, contentLength will have been asked first so that a representation is cached"
	
	representation isNil ifTrue: [ self computeRepresentation ].
	ZnUtils nextPutAll: representation on: stream! !

!ZnApplicationFormUrlEncodedEntity methodsFor: 'writing' stamp: 'KrisGybels 6/11/2014 13:48'!
writeRepresentationOn: stream
	self contentType charSet
		ifNotNil: [:charSet|
			ZnResourceMetaUtils
				writeQueryFields: self fields 
				withEncoder: (ZnCharacterEncoder newForEncoding: charSet)
				on: stream] 
		ifNil: [ 
			ZnResourceMetaUtils 
				writeQueryFields: self fields 
				on: stream ]! !

ZnEntity subclass: #ZnByteArrayEntity
	instanceVariableNames: 'bytes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnByteArrayEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:04' prior: 0!
I am ZnByteArrayEntity, a concrete HTTP Entity based on a ByteArray.
It is used to hold binary (non textual) data.
I am a ZnEntity.

Part of Zinc HTTP Components.!

!ZnByteArrayEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/12/2010 11:13'!
bytes: byteArray
	^ (self type: ZnMimeType applicationOctetStream length: byteArray size)
		bytes: byteArray;
		yourself ! !

!ZnByteArrayEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 17:22'!
designatedMimeType
	"I have no designated mime type, I can handle all binary content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnByteArrayEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/4/2013 14:43'!
matches: mimeType
	"I have no designated mime type, I can handle all binary content.
	See #concreteSubclassForType: "
	
	^ false! !

!ZnByteArrayEntity methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:45'!
= other
	^ super = other and: [ self bytes = other bytes ]! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
bytes
	^ bytes! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 15:12'!
bytes: anObject
	bytes := anObject! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 22:05'!
contents
	^ self bytes! !

!ZnByteArrayEntity methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:44'!
hash
	^ super hash bitXor: self bytes hash! !

!ZnByteArrayEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 17:53'!
isEmpty 
	^ self bytes isNil or: [ self bytes isEmpty ]! !

!ZnByteArrayEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:20'!
printContentsOn: stream
	super printContentsOn: stream.
	self bytes notNil ifTrue: [ 
		stream space; print: self bytes ]! !

!ZnByteArrayEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/17/2012 15:00'!
readFrom: stream
	self contentLength isNil
		ifTrue: [ 
			self bytes: (ZnUtils readUpToEnd: stream limit: ZnConstants maximumEntitySize).
			self contentLength: self bytes size ]
		ifFalse: [ | byteArray |
			self contentLength > ZnConstants maximumEntitySize
				ifTrue: [ ZnEntityTooLarge signal ].
			byteArray := ByteArray ofSize: self contentLength.
			self contentLength > ZnUtils streamingBufferSize 
				ifTrue: [
					ZnUtils streamFrom: stream to: byteArray writeStream size: self contentLength ]
				ifFalse: [
					stream next: self contentLength into: byteArray ]. 
			self bytes: byteArray ]! !

!ZnByteArrayEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 19:27'!
readStream
	^ self bytes readStream! !

!ZnByteArrayEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/18/2013 19:54'!
writeOn: stream
	ZnUtils nextPutAll: self bytes on: stream! !

!ZnEntity class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 13:57'!
byteArrayEntityClass
	^ ZnByteArrayEntity ! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:00'!
bytes: bytes
	^ self byteArrayEntityClass bytes: bytes! !

!ZnEntity class methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/4/2013 15:27'!
concreteSubclassForType: mimeType binary: forceBinary
	"Answer the concrete ZnEntity subclass that handles mimeType"
	
	^ self allSubclasses 
		detect: [ :each | 
			each matches: mimeType ]
		ifNone: [ 
			(mimeType isBinary or: [ forceBinary ])
				ifTrue: [ self byteArrayEntityClass ] 
				ifFalse: [ self stringEntityClass ] ]
! !

!ZnEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 17:22'!
designatedMimeType
	"Answer the mime type we are designated to handle.
	See #concreteSubclassForType: "
	
	self subclassResponsibility ! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:01'!
html: html
	^ self stringEntityClass html: html! !

!ZnEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/4/2013 14:43'!
matches: mimeType
	"Return whether a given mimeType can be handled by instances of the receiver"
	
	^ mimeType matches: self designatedMimeType ! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/17/2010 17:07'!
new
	^ super new initialize ! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 10/16/2014 10:42'!
readBinaryFrom: stream usingType: mimeType andLength: length
	"Create and return a new buffered entity of mimeType and length from stream.
	The concrete subclass depends on mimeType, but textual type will remain binary"
	
	| newEntity |
	newEntity := (self concreteSubclassForType: mimeType binary: true) 
						type: mimeType length: length.
	newEntity readBinaryFrom: stream.
	^ newEntity 
! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/7/2012 17:36'!
readFrom: stream usingType: mimeType andLength: length
	"Create and return a new buffered entity of mimeType and length from stream.
	The concrete subclass depends on mimeType"
	
	| newEntity |
	newEntity := (self concreteSubclassForType: mimeType binary: false) 
						type: mimeType length: length.
	newEntity readFrom: stream.
	^ newEntity 
! !

!ZnEntity class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/6/2010 13:48'!
stringEntityClass
	^ ZnStringEntity! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:01'!
text: text
	^ self stringEntityClass text: text! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/6/2013 22:46'!
textCRLF: text
	^ self text: text, String crlf! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/25/2010 22:16'!
type: type
	"Create a new entity with a given mime type and (currently) undetermined length"
	
	^ self new
		contentType: type;
		yourself! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/12/2010 11:06'!
type: type length: length
	"Create a new entity with a given mime type and length"
	
	^ self new
		contentType: type;
		contentLength: length;
		yourself! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/3/2010 14:02'!
with: object
	"Create a new entity holding object.
	Currently works for strings and byte arrays.
	You will probably want to set the mime type to something more specific"
	
	^ object isString 
		ifTrue: [ self text: object ] 
		ifFalse: [ self bytes: object ]! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/17/2010 20:39'!
with: object type: type
	"Create a new entity holding object.
	Currently works for strings and byte arrays.
	The entity is set to mime type"
	
	^ (self with: object)
		contentType: type;
		yourself! !

!ZnEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/11/2011 18:12'!
xml: text
	^ self stringEntityClass xml: text! !

!ZnEntity methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:42'!
= other
	self class = other class ifFalse: [ ^ false ].
	^ self contentType = other contentType and: [ self contentLength = other contentLength ]! !

!ZnEntity methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 5/7/2013 15:58'!
asByteArray
	"Return the ByteArray that is the binary, encoded representation of the receiver"
	
	^ ByteArray streamContents: [ :stream | self writeOn: stream ]! !

!ZnEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/18/2011 12:50'!
close! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:17'!
contentLength 
	"Content length can be nil when it is (currently) unknown."
	
	^ contentLength! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:57'!
contentLength: integer
	contentLength := integer! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:57'!
contentType
	^ contentType ! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/4/2013 15:25'!
contentType: object
	"We only allow assignment compatible with our designated mime type"
	
	| newType |
	newType := object asZnMimeType.
	contentType = newType 
		ifTrue: [ ^ self ]
		ifFalse: [ 
			(self class designatedMimeType isNil or: [ self class matches: newType ])
				ifTrue: [ contentType := newType ] ]! !

!ZnEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/13/2010 22:04'!
contents
	self subclassResponsibility! !

!ZnEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:25'!
hasContentLength
	^ contentLength notNil! !

!ZnEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:26'!
hasContentType
	^ contentType notNil! !

!ZnEntity methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:41'!
hash
	^ self contentType hash bitXor: self contentLength hash! !

!ZnEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 17:51'!
isEmpty 
	self subclassResponsibility ! !

!ZnEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:19'!
printContentTypeAndLengthOn: stream
	self hasContentType ifTrue: [ 
		stream print: self contentType; space ].
	self hasContentLength ifTrue: [
		stream print: self contentLength; nextPut: $B ]! !

!ZnEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 10:56'!
printContentsOn: stream
	self printContentTypeAndLengthOn: stream! !

!ZnEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:24'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self printContentsOn: stream.
	stream nextPut: $)! !

!ZnEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/16/2014 10:53'!
readBinaryFrom: stream
	self readFrom: stream! !

!ZnEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/6/2010 22:02'!
readFrom: stream
	self subclassResponsibility ! !

!ZnEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:30'!
writeOn: stream
	self subclassResponsibility ! !

ZnEntity subclass: #ZnMultiPartFormDataEntity
	instanceVariableNames: 'parts representation'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMultiPartFormDataEntity commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:04' prior: 0!
I am ZnMultiPartFormDataEntity, a concrete HTTP Entity 
holding multi part form data that is encoded according to specific rules.
I am a ZnEntity.

Acknowledgement: some code borrowed from AJP.

Part of Zinc HTTP Components.!

!ZnMultiPartFormDataEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/11/2010 20:42'!
designatedMimeType
	^ ZnMimeType multiPartFormData  ! !

!ZnMultiPartFormDataEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/11/2013 09:25'!
matches: mimeType
	^ mimeType matches: 'multipart/*' asZnMimeType! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/27/2011 10:29'!
addPart: mimePart
	self invalidateRepresentation.
	self parts add: mimePart! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2010 20:11'!
computeRepresentation
	representation := ByteArray streamContents: [ :stream | 
		self writeRepresentationOn: (ZnBivalentWriteStream on: stream) ].
	contentLength := representation size! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/27/2011 17:11'!
contentLength
	contentLength isNil ifTrue: [ self computeRepresentation ].
	^ contentLength! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/11/2013 10:43'!
contentType: object
	"We only allow assignment compatible with our designated mime type.
	The main/sub must be equal but the parameters must be allowed to be different"
	
	| newType |
	newType := object asZnMimeType.
	(self class matches: newType)
		ifTrue: [ contentType := newType ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 19:14'!
contents
	^ self parts! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/31/2011 12:13'!
generateBoundary
	^ String streamContents: [ :stream | | letters |
			stream nextPutAll: 'Boundary-Zn-'.
			letters := 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'.
			8 timesRepeat: [ stream nextPut: letters atRandom ] ]
	! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:17'!
getBoundary
	"Return the multipart/form-data MIME part boundary.
	This should really be there, set externally or internally.
	See #initialize and #mimeTypeWithBoundary"
	
	^ (self contentType parameterAt: 'boundary') withoutQuoting ! !

!ZnMultiPartFormDataEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/5/2010 17:17'!
initialize
	super initialize.
	self contentType: self mimeTypeWithBoundary  
	! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2010 19:15'!
invalidateRepresentation
	representation := nil.
	contentLength := nil! !

!ZnMultiPartFormDataEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 12:43'!
isEmpty
	^ parts isNil or: [ parts isEmpty ]! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/5/2010 17:19'!
mimeTypeWithBoundary
	| mimeType |
	mimeType := self class designatedMimeType copy.
	mimeType parameterAt: 'boundary' put: self generateBoundary.
	^ mimeType! !

!ZnMultiPartFormDataEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/16/2014 10:44'!
parse: bytes boundary: boundary binary: binary
	| next start fullBoundary |
	fullBoundary := '--' asByteArray, boundary.
	"Sometimes there is whitespace in front of the first boundary"
	start := (bytes
		indexOfSubCollection: fullBoundary
		startingAt: 1) + 1.
	next := bytes
		indexOfSubCollection: fullBoundary
		startingAt: start 
		ifAbsent: [ bytes size ].
	[ next < (bytes size - 2) ] whileTrue: [ | partReadStream |
		partReadStream := ReadStream on: bytes from: start + 2 to: next - 3.
		self addPart: (ZnMimePart perform: (binary ifTrue: [ #readBinaryFrom: ] ifFalse: [ #readFrom: ]) with: partReadStream).
		start := next + fullBoundary size.
		next := bytes
			indexOfSubCollection: fullBoundary
			startingAt: start
			ifAbsent: [ bytes size ] ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:12'!
partNamed: fieldName
	^ self partNamed: fieldName ifNone: [ self error: 'Cannot find part named ', fieldName asString ]! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 19:18'!
partNamed: fieldName ifNone: block
	parts isNil ifTrue: [ block value ].
	^ self parts detect: [ :each | each fieldName = fieldName ] ifNone: block! !

!ZnMultiPartFormDataEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 20:07'!
parts
	parts isNil ifTrue: [ parts := OrderedCollection new ].
	^ parts! !

!ZnMultiPartFormDataEntity methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 2/27/2011 20:13'!
partsDo: block
	parts isNil ifTrue: [ ^ self ].
	parts do: block  ! !

!ZnMultiPartFormDataEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printContentsOn: stream
	super printContentsOn: stream.
	self isEmpty
		ifTrue: [ ^ self ].
	stream space.
	self parts printElementsOn: stream! !

!ZnMultiPartFormDataEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/16/2014 10:44'!
readBinaryFrom: stream
	"Switch to streaming implementation later on"
	
	| bytes |
	"We have to use the contentLength instance variable instead of the accessor else a representation will be computed"
	contentLength isNil
		ifTrue: [ 
			bytes := ZnUtils readUpToEnd: stream limit: ZnConstants maximumEntitySize ]
		ifFalse: [
			self contentLength > ZnConstants maximumEntitySize ifTrue: [ ZnEntityTooLarge signal ].
			bytes := ByteArray ofSize: contentLength.
			stream next: contentLength into: bytes ].
	self parse: bytes boundary: self getBoundary asByteArray binary: true! !

!ZnMultiPartFormDataEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/16/2014 10:44'!
readFrom: stream
	"Switch to streaming implementation later on"
	
	| bytes |
	"We have to use the contentLength instance variable instead of the accessor else a representation will be computed"
	contentLength isNil
		ifTrue: [ 
			bytes := ZnUtils readUpToEnd: stream limit: ZnConstants maximumEntitySize ]
		ifFalse: [
			self contentLength > ZnConstants maximumEntitySize ifTrue: [ ZnEntityTooLarge signal ].
			bytes := ByteArray ofSize: contentLength.
			stream next: contentLength into: bytes ].
	self parse: bytes boundary: self getBoundary asByteArray binary: false! !

!ZnMultiPartFormDataEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/18/2013 19:57'!
writeOn: stream
	"We always go via our representation because we need a bivalent stream"
	
	representation isNil ifTrue: [ self computeRepresentation ].
	ZnUtils nextPutAll: representation on: stream! !

!ZnMultiPartFormDataEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:24'!
writeRepresentationOn: stream
	| boundary |
	boundary := self getBoundary.
	self parts do: [ :each |
		stream nextPutAll: '--'; nextPutAll: boundary; nextPutAll: String crlf.
		each writeOn: stream.
		stream nextPutAll: String crlf ].
	stream nextPutAll: '--'; nextPutAll: boundary; nextPutAll: '--'; nextPutAll: String crlf
! !

ZnEntity subclass: #ZnStreamingEntity
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStreamingEntity commentStamp: 'SvenVanCaekenberghe 8/18/2011 14:48' prior: 0!
I am ZnStreamingEntity, a concrete HTTP Entity based on a stream.
I am a ZnEntity.

When a ZnStreamingEntity is created using #readFrom: 
its content type and length are set but no body is actually read.
Instead, stream is set to a ZnLimitedReadStream that clients can/must
use to read from.

When the client is done, either all data must be read or #consumeContent must be called.
A client should normally not close the stream.

A ZnStreamingEntity can also be instanciated with an existing read stream.
During #writeOn: this stream is copied to the output stream.

Part of Zinc HTTP Components.!

!ZnStreamingEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/15/2010 19:23'!
designatedMimeType
	"I have no designated mime type, I can handle all content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnStreamingEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/4/2013 14:43'!
matches: mimeType
	"I have no designated mime type, I can handle all content.
	See #concreteSubclassForType: "
	
	^ false! !

!ZnStreamingEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/15/2010 17:13'!
readFrom: stream usingType: mimeType andLength: length
	"Create and return a new streaming entity of mimeType and length from stream.
	Clients can/must use the embedded ZnLimitedReadStream to read all content, 
	or call #consumeContent when they are done. They should not close the stream."
	
	^ (self type: mimeType length: length)
		readFrom: stream;
		yourself
! !

!ZnStreamingEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 8/18/2011 12:51'!
close
	stream ifNotNil: [ 
		stream close. 
		stream := nil ]! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:10'!
consumeContent
	"To be improved later on using some kind of larger buffer"
	
	[ self stream atEnd ] 
		whileFalse: [ self stream next ]! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/31/2012 12:59'!
contents
	"Since this is a non-repeatable entity, 
	#contents will #consumeContent, 
	this can thus only be done once"
	
	^ ZnUtils readUpToEnd: self stream limit: ZnConstants maximumEntitySize! !

!ZnStreamingEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 18:25'!
isEmpty 
	"We actually don't know"
	
	^ false! !

!ZnStreamingEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/13/2012 20:27'!
readFrom: readStream
	self contentLength isNil
		ifTrue: [ 
			self stream: readStream ]
		ifFalse: [ | limitedStream |
			limitedStream := ZnLimitedReadStream on: readStream limit: self contentLength.
			self stream: limitedStream ]! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:14'!
readStream 
	^ self stream! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:04'!
stream 
	^ stream! !

!ZnStreamingEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/15/2010 17:04'!
stream: readStream
	stream := readStream! !

!ZnStreamingEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 7/20/2012 12:01'!
writeOn: outputStream
	(self contentLength isNil or: [ self contentLength = 0 ])
		ifTrue: [ 
			ZnUtils 
				streamFrom: self stream 
				to: outputStream ]
		ifFalse: [
			ZnUtils 
				streamFrom: self stream 
				to: outputStream 
				size: self contentLength ].
	stream close! !

ZnEntity subclass: #ZnStringEntity
	instanceVariableNames: 'string encoder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStringEntity commentStamp: '<historical>' prior: 0!
I am ZnStringEntity, a concrete HTTP Entity based on a String.
It is used to hold textual (non-binary) data.
I am a ZnEntity.

Optionally, an encoding is used to convert to and from bytes.
The default encoding it UTF-8.

Note that content length is the encoded byte count, not the number of characters in the string.

Part of Zinc HTTP Components.!

!ZnStringEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:58'!
designatedMimeType
	"I have no designated mime type, I can handle all non-binary content.
	See #concreteSubclassForType: "
	
	^ nil! !

!ZnStringEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/2/2010 12:22'!
html: string
	^ (self type: ZnMimeType textHtml)
		string: string;
		yourself! !

!ZnStringEntity class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/4/2013 14:44'!
matches: mimeType
	"I have no designated mime type, I can handle all non-binary content.
	See #concreteSubclassForType: "
	
	^ false! !

!ZnStringEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/2/2010 12:22'!
text: string
	^ (self type: ZnMimeType textPlain)
		string: string;
		yourself! !

!ZnStringEntity class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 12/11/2011 18:12'!
xml: string
	^ (self type: ZnMimeType applicationXml)
		string: string;
		yourself! !

!ZnStringEntity methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:44'!
= other
	^ super = other and: [ self string = other string ]! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/23/2013 12:27'!
computeContentLength
	self string isNil ifTrue: [ ^ self ].
	contentLength := self encoder encodedByteCountForString: string! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/27/2011 10:26'!
contentLength
	contentLength isNil ifTrue: [ self computeContentLength ].
	^ contentLength! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/14/2014 15:40'!
contentType: object
	"We only allow assignment compatible with our designated mime type"
	
	| newType |
	newType := object asZnMimeType.
	contentType = newType 
		ifTrue: [ ^ self ]
		ifFalse: [ 
			newType isBinary
				ifFalse: [ 
					contentType := newType.
					self invalidateEncoder ] ]! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
contents
	^ self string! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:59'!
encoder
	self initializeEncoder.
	^ encoder! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 15:11'!
encoder: anEncoding
	(encoder notNil and: [ anEncoding ~= encoder ]) ifTrue: [ self invalidateContentLength ].
	encoder := anEncoding! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/14/2010 12:09'!
hasEncoder
	^ encoder notNil! !

!ZnStringEntity methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:43'!
hash
	^ super hash bitXor: self string hash! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/2/2015 14:44'!
initializeEncoder
	"If the content-type specifies an encoding, use that.
	If not, use the value of ZnDefaultCharacterEncoder,
	if that is not set, fall back to UTF-8"
	
	| charSet newEncoder |
	self hasEncoder
		ifTrue: [ ^ self ].
	newEncoder := (charSet := contentType charSet) isNil
		ifTrue: [ ZnDefaultCharacterEncoder value ifNil: [ ZnUTF8Encoder new ] ]
		ifFalse: [ 
			(ZnCharacterEncoder newForEncoding: charSet)
				beLenient;
				yourself ].
	self encoder: newEncoder! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/2/2010 12:20'!
invalidateContentLength
	self contentLength: nil! !

!ZnStringEntity methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/14/2014 15:04'!
invalidateEncoder
	self encoder: nil! !

!ZnStringEntity methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/1/2011 17:54'!
isEmpty 
	^ self string isNil or: [ self string isEmpty ] ! !

!ZnStringEntity methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:20'!
printContentsOn: stream
	super printContentsOn: stream.
	self string notNil ifTrue: [ 
		stream space; nextPutAll: self string ]! !

!ZnStringEntity methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/13/2013 13:53'!
readFrom: stream
	| buffer totalRead read readStream stringStream total |
	total := self contentLength.
	readStream := total ifNotNil: [ ZnLimitedReadStream on: stream limit: total ] ifNil: [ stream ].
	buffer := String new: (ZnUtils streamingBufferSize min: (total ifNil: [ ZnUtils streamingBufferSize ])).
	stringStream := nil.
	totalRead := 0.
	self initializeEncoder.
	[ readStream atEnd ] whileFalse: [
		[ read := encoder readInto: buffer startingAt: 1 count: buffer size fromStream: readStream ]
			on: ZnByteStringBecameWideString 
			do: [ :notification | 
					buffer := notification wideString.
					stringStream ifNotNil: [ | wideString position |
						position := stringStream position.
						wideString := WideString from: stringStream originalContents.
						stringStream on: wideString; setFrom: position + 1 to: position ].
					notification resume ].
		totalRead := totalRead + read.
		totalRead > ZnConstants maximumEntitySize
			ifTrue: [ ZnEntityTooLarge signal ].
		stringStream ifNil: [ 
			readStream atEnd
				ifTrue: [ ^ self string: (buffer copyFrom: 1 to: read) ]
				ifFalse: [ stringStream := (total ifNil: [ buffer species new ] ifNotNil: [ buffer species new: total ]) writeStream ] ].
		stringStream next: read putAll: buffer startingAt: 1.
		ZnUtils signalProgress: totalRead total: total ].
	self string: (stringStream ifNil: [ String new ] ifNotNil: [ stringStream contents ])! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
readStream
	^ self string readStream! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/1/2010 20:57'!
string
	^ string! !

!ZnStringEntity methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/3/2010 15:11'!
string: aString
	(string notNil and: [ aString ~= string ]) ifTrue: [ self invalidateContentLength ].
	string := aString! !

!ZnStringEntity methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:25'!
writeOn: stream
	| totalWritten toWrite total  |
	self initializeEncoder.
	totalWritten := 0.
	total := string size.
	[ totalWritten < total ] whileTrue: [ 
		toWrite := ZnUtils streamingBufferSize min: (total - totalWritten).
		encoder next: toWrite putAll: string startingAt: totalWritten + 1 toStream: stream.
		totalWritten := totalWritten + toWrite.
		ZnUtils signalProgress: totalWritten total: total ]! !

Object subclass: #ZnEntityReader
	instanceVariableNames: 'headers stream streaming binary allowReadingUpToEnd'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnEntityReader commentStamp: 'SvenVanCaekenberghe 4/7/2012 16:37' prior: 0!
I am ZnEntityReader, I help reading ZnEntities from a stream given meta data in headers.

I deal with chunking and gzip decoding.

I have several options:
	- to read streaming entities, where the client has to do the actual reading
	- to read binary entities, where textual content is not interpreted
	- to allow reading up to end, when there is no content length set

Part of Zinc HTTP Components.!

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:07'!
allowReadingUpToEnd
	allowReadingUpToEnd := true! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:07'!
allowsReadingUpToEnd
	^ allowReadingUpToEnd notNil and: [ allowReadingUpToEnd ]! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 4/7/2012 16:37'!
binary
	binary := true! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/20/2011 12:39'!
canReadContent
	^ self hasContentLength or: [ self isChunked or: [ self allowsReadingUpToEnd ] ]! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:10'!
contentLength
	^ (self headers includesKey: 'Content-Length')
		ifTrue: [ self headers contentLength ]
		ifFalse: [ nil ]! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/24/2015 14:40'!
contentType
	^ (self headers includesKey: 'Content-Type') 
		ifTrue: [ self headers contentType ] 
		ifFalse: [ ZnMimeType default ]! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:35'!
hasContentLength
	^ self headers includesKey: 'Content-Length'! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:23'!
headers
	^ headers! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:22'!
headers: znHeaders
	headers := znHeaders! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/7/2012 16:37'!
isBinary
	^ binary notNil and: [ binary ]! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/28/2013 12:02'!
isChunked
	| transferEncoding |
	transferEncoding := self headers at: 'Transfer-Encoding' ifAbsent: [ ^ false ].
	^ transferEncoding = 'chunked' or: [ transferEncoding beginsWith: 'chunked' ]! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 13:55'!
isGzipped
	| contentEncoding |
	contentEncoding := self headers at: 'Content-Encoding' ifAbsent: [ ^ false ].
	^ contentEncoding = 'gzip'! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2010 16:45'!
isIdentityEncoded
	^ self isGzipped not! !

!ZnEntityReader methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/25/2010 23:03'!
isStreaming
	^ streaming notNil and: [ streaming ]! !

!ZnEntityReader methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/2/2015 14:59'!
readEntity
	| entity |
	self canReadContent ifFalse: [ ^ nil ].
	entity := self readEntityFromStream.
	^ entity isEmpty
		ifTrue: [ nil ]
		ifFalse: [ entity ]    ! !

!ZnEntityReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/28/2013 13:17'!
readEntityFromStream
	| entity decodedEntityLength chunkedStream isChunked |
	chunkedStream := (isChunked := self isChunked) ifTrue: [ stream := ZnChunkedReadStream on: stream ] ifFalse: [ nil ].
	self isGzipped 
		ifTrue: [ 
			self hasContentLength ifTrue: [ stream := ZnLimitedReadStream on: stream limit: self contentLength ].
			decodedEntityLength := nil.
			stream := GZipReadStream on: stream ]
		ifFalse: [ decodedEntityLength := self contentLength ].
	entity := self readFrom: stream usingType: self contentType andLength: decodedEntityLength.
	isChunked ifTrue: [ 
		self isIdentityEncoded ifTrue: [ entity contentLength: stream totalSize ].
		chunkedStream extraHeaders ifNotNil: [ :extraHeaders | self headers addAll: extraHeaders ] ].
	^ entity! !

!ZnEntityReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 4/7/2012 16:42'!
readFrom: readStream usingType: contentType andLength: length
	| entityClass selector |
	entityClass := self isStreaming 
		ifTrue: [ ZnStreamingEntity ]
		ifFalse: [ ZnEntity ].
	selector := self isBinary 
		ifTrue: [ #readBinaryFrom:usingType:andLength: ]
		ifFalse: [ #readFrom:usingType:andLength: ].
	^ entityClass perform: selector 
		with: readStream
		with: contentType 
		with: length! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:23'!
stream
	^ stream! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:22'!
stream: readStream
	stream := readStream ! !

!ZnEntityReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/25/2010 22:31'!
streaming
	streaming := true! !

Object subclass: #ZnEntityWriter
	instanceVariableNames: 'headers stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnEntityWriter commentStamp: '<historical>' prior: 0!
I am ZnEntityWriter, I help writing ZnEntities to a stream given meta data in headers.

I deal with chunking and gzip encoding.

Part of Zinc HTTP Components.!

!ZnEntityWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/24/2015 14:40'!
contentType
	^ (self headers includesKey: 'Content-Type') 
		ifTrue: [ self headers contentType ] 
		ifFalse: [ ZnMimeType default ]! !

!ZnEntityWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 12:47'!
headers
	^ headers! !

!ZnEntityWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 12:47'!
headers: znHeaders
	headers := znHeaders! !

!ZnEntityWriter methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/28/2013 12:02'!
isChunked
	| transferEncoding |
	transferEncoding := self headers at: 'Transfer-Encoding' ifAbsent: [ ^ false ].
	^ transferEncoding = 'chunked' or: [ transferEncoding beginsWith: 'chunked' ]! !

!ZnEntityWriter methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/18/2013 13:01'!
isGzipped
	| contentEncoding |
	contentEncoding := self headers at: 'Content-Encoding' ifAbsent: [ ^ false ].
	^ contentEncoding = 'gzip'! !

!ZnEntityWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 12:47'!
stream
	^ stream! !

!ZnEntityWriter methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/18/2013 12:47'!
stream: writeStream
	stream := writeStream! !

!ZnEntityWriter methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/2/2015 14:59'!
writeEntity: entity
	| actualStream gzipStream bufferedStream chunkedStream isChunked |
	actualStream := stream.
	chunkedStream := gzipStream := bufferedStream := nil.
	(isChunked := self isChunked)
		ifTrue: [ 
			chunkedStream := actualStream := ZnChunkedWriteStream on: actualStream ].
	self isGzipped 
		ifTrue: [ 
			gzipStream := actualStream := GZipWriteStream on: actualStream ]
		ifFalse: [ 
			(isChunked and: [ self contentType isBinary not ])
				ifTrue: [ bufferedStream := actualStream := ZnUtils bufferedWriteStreamOn: actualStream ] ].
	entity writeOn: actualStream.
	gzipStream 
		ifNotNil: [ gzipStream finish ].
	bufferedStream 
		ifNotNil: [ bufferedStream finish ].
	chunkedStream
		ifNotNil: [ chunkedStream finish ]
	! !

Object subclass: #ZnHeaders
	instanceVariableNames: 'headers'
	classVariableNames: 'CommonHeaders'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnHeaders commentStamp: 'SvenVanCaekenberghe 1/18/2011 10:57' prior: 0!
I am ZnHeaders.
I am a collection of HTTP headers.
I can be used for generating and parsing.

Header names are normalized and used case insensitive.
Header values can be multi-valued.

Part of Zinc HTTP Components.!

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 21:12'!
defaultRequestHeaders
	^ self new
		at: 'User-Agent' put: ZnConstants defaultUserAgent;
		at: 'Accept' put: '*/*';
		yourself! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/31/2011 08:54'!
defaultResponseHeaders
	^ self new
		at: 'Server' put: ZnConstants defaultServerString;
		at: 'Date' put: ZnUtils httpDate;
		yourself! !

!ZnHeaders class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 5/28/2013 11:52'!
initialize
	CommonHeaders := (
		#(
			'Content-Length' 'Content-Type' 'Date' 'Server' 'Connection' 'User-Agent' 'Host'
			'Accept' 'Accept-Language' 'Accept-Encoding'
			'Referer' 'Dnt'
			'Cookie' 'Set-Cookie' 'Authorization' 'WWW-Authenticate' 'Proxy-Authorization'
			'Content-Encoding' 'Transfer-Encoding' 'Location'
			'If-Modified-Since' 'Content-Disposition' 
			'X-Zinc-Remote-Address' 'X-Forwarded-Server' 'X-Forwarded-For' 'X-Forwarded-Host' ) 
			collect: [ :each | ZnUtils capitalizeString: each ]) asSet! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 17:07'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/27/2011 14:41'!
requestHeadersFor: url
	| headers |
	headers := self defaultRequestHeaders.
	headers request: url.
	^ headers! !

!ZnHeaders class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/13/2010 21:45'!
withAll: keyedCollection
	^ self new
		addAll: keyedCollection;
		yourself ! !

!ZnHeaders methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:34'!
= other
	self class = other class ifFalse: [ ^ false ].
	^ self headers = other headers! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/29/2013 14:22'!
acceptEntityDescription: entity
	"Take over the content type and length from entity"
	
	entity 
		ifNotNil: [
			self
				contentType: entity contentType;
				contentLength: entity contentLength ]
		ifNil: [
			self 
				clearContentType; 
				clearContentLength ] ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/7/2013 12:20'!
addAll: keyedCollection
	"Note that we use #addAllMulti:"

	keyedCollection isEmpty
		ifFalse: [ self headers addAllMulti: keyedCollection ].
	^ keyedCollection! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:25'!
at: headerName
	"Return the value stored under headerName,
	a String for single-valued headers, an Array of Strings for multi-valued headers"
	
	^ self headers at: (self normalizeHeaderKey: headerName)! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/2/2012 17:27'!
at: headerName add: value
	"Store value under headerName, optionally turning it into a multi-valued header 
	when a value was already present"
	
	^ self headers at: (self normalizeHeaderKey: headerName) add: value 
		
! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:16'!
at: headerName ifAbsent: block
	"Like #at: but executes block when the headerName is not found"
	
	self isEmpty ifTrue: [ ^ block value ].
	^ self headers at: (self normalizeHeaderKey: headerName) ifAbsent: block! !

!ZnHeaders methodsFor: 'accessing' stamp: 'PaulDeBruicker 5/12/2012 10:02'!
at: headerName ifPresent: block
	"Like #at: but executes block when the headerName is found"

	^ self headers at: (self normalizeHeaderKey: headerName) ifPresent: block! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:26'!
at: headerName put: value
	"Store value under headerName, replace existing entries"
	
	^ self headers at: (self normalizeHeaderKey: headerName) put: value! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:49'!
at: headerName put: value ifPresentMerge: binaryBlock
	"Store value under headerName, when there is an existing entry,
	stored the result of evaluating binaryBlock with old and new value"
	
	| normalizedKey existingValue newValue |
	normalizedKey := self normalizeHeaderKey: headerName.
	existingValue := self headers at: normalizedKey ifAbsent: [ nil ].
	newValue := existingValue isNil
		ifTrue: [ value ] 
		ifFalse: [ binaryBlock value: existingValue value: value ].
	^ self headers at: normalizedKey put: newValue 
		

! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/20/2013 12:59'!
clearContentLength
	self removeKey: 'Content-Length' ifAbsent: [] ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/20/2013 12:59'!
clearContentType
	self removeKey: 'Content-Type' ifAbsent: [] ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/20/2011 13:48'!
contentLength
	"We allow multiple content-length headers provided they are identical."
	
	| value |
	(value := self headers at: 'Content-Length') isString
		ifFalse: [
			value asSet size = 1
				ifTrue: [ value := value first ]
				ifFalse: [ self error: 'Multiple, different Content-Length headers are not allowed' ] ].
	^ Integer readFrom: value ifFail: [ self error: 'Illegal HTTP Content Length' ] ! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/29/2013 14:21'!
contentLength: object
	self at: 'Content-Length' put: object asString! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/24/2015 14:34'!
contentType
	| headerValue |
	headerValue := (self headers at: 'Content-Type') trimBoth.
	headerValue isEmpty ifTrue: [ ^ ZnMimeType default ].
	^ ZnMimeType fromString: headerValue! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/29/2013 14:21'!
contentType: object
	self at: 'Content-Type' put: object asString! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/11/2013 10:03'!
extendHeaderAt: key from: line
	"The value of a continuation header line is concatenated, 
	keeping the whitespace, but without the CRLF"

	| existingValue newValue |
	existingValue := self at: key.
	newValue := existingValue isArray
		ifTrue: [ | last |
			last := existingValue size.
			existingValue at: last put: ((existingValue at: last) , line).
			existingValue ]
		ifFalse: [ existingValue , line ].
	self at: key put: newValue! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:12'!
hasContentLength
	^ self includesKey: 'Content-Length'! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:11'!
hasContentType
	^ self includesKey: 'Content-Type'! !

!ZnHeaders methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:35'!
hash
	^ self headers hash! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 1/2/2012 17:24'!
headers
	headers isNil ifTrue: [ headers := ZnMultiValueDictionary new ].
	^ headers! !

!ZnHeaders methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/21/2010 12:14'!
headersDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	self isEmpty ifTrue: [ ^ self ].
	self headers keysAndValuesDo: [ :headerKey :headerValue |
		headerValue isArray
			ifTrue: [ 
				headerValue do: [ :each | 
					block value: headerKey value: each ] ]
			ifFalse: [
				block value: headerKey value: headerValue ] ]! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:13'!
includesKey: headerName
	^ self isEmpty not and: [ self headers includesKey: (self normalizeHeaderKey: headerName) ]
	! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/10/2010 20:29'!
isDescribingEntity
	"Do I include enough information to describe an entity (i.e. content length and type) ?"
	
	^ (self headers includesKey: 'Content-Type') 
		and: [ self headers includesKey: 'Content-Length' ]
! !

!ZnHeaders methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/6/2010 17:09'!
isEmpty 
	^ headers isNil or: [ self headers isEmpty ]! !

!ZnHeaders methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/26/2010 18:13'!
keysAndValuesDo: block
	"Execute a two argument (key, value) block for each header.
	Multi-valued headers are handled transparently."

	self headersDo: block! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/28/2013 11:59'!
normalizeHeaderKey: string
	"Test string to see if it has proper header key capitalization,
	if true, return string, if not, return a properly capitalized copy"
	
	"Optimization: if string is a know,, properly capitalized header, use it as is"
	(CommonHeaders includes: string) 
		ifTrue: [ ^ string ].
	^ (ZnUtils isCapitalizedString: string)
		ifTrue: [ string ] 
		ifFalse: [ ZnUtils capitalizeString: string ] ! !

!ZnHeaders methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 6/12/2014 15:58'!
postCopy
	headers := headers copy! !

!ZnHeaders methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 9/20/2010 17:20'!
printOn: stream
	super printOn: stream.
	self isEmpty ifFalse: [ self headers printElementsOn: stream ]
	! !

!ZnHeaders methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/22/2015 13:53'!
readFrom: stream
	| line reader |
	reader := ZnLineReader on: stream.
	[ (line := reader nextLine) isEmpty ] whileFalse: [ | key |
		key := self readOneHeaderFrom: line readStream.
		"Continuation header lines start with a space or tab"
		[ stream atEnd not and: [ #[ 32 9 ] includes: stream peek asInteger ] ]
			whileTrue: [ 
				self extendHeaderAt: key from: reader nextLine ] ] ! !

!ZnHeaders methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/22/2012 10:27'!
readOneHeaderFrom: stream
	| key value |
	key := stream upTo: $:.
	[ stream peek == Character space ] whileTrue: [ stream next ].
	value := stream upToEnd.
	self at: key add: value.
	^ key! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/14/2010 13:36'!
removeKey: headerName	
	^ self headers removeKey: (self normalizeHeaderKey: headerName)! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:16'!
removeKey: headerName ifAbsent: block
	self isEmpty ifTrue: [ ^ block value ].
	^ self headers removeKey: (self normalizeHeaderKey: headerName) ifAbsent: block! !

!ZnHeaders methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/6/2015 16:23'!
request: url
	"Setup the receiver to request url"
	
	(url isNil or: [ url hasHost not ]) ifTrue: [ ^ self ].  
	self at: 'Host' put: url authorityWithPort.
	(ZnNetworkingUtils proxyAuthorizationHeaderValueToUrl: url)
		ifNotNil: [ :value | self at: 'Proxy-Authorization' put: value ].    
  ! !

!ZnHeaders methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 5/14/2013 11:29'!
unlimited
	self headers unlimited! !

!ZnHeaders methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:39'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	self headersDo: [ :key :value |
		writeStream nextPutAll: key; nextPut: $:; space; nextPutAll: value; nextPutAll: String crlf ]! !

Object subclass: #ZnHtmlOutputStream
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnHtmlOutputStream commentStamp: 'SvenVanCaekenberghe 11/30/2015 22:06' prior: 0!
I am ZnHtmlOutputStream. I wrap another character write stream to offer a richer API for generating correct HTML markup.

See https://en.wikipedia.org/wiki/HTML

My streaming protocol contains the traditional write stream operations. These are raw and do not do any conversions/escaping.

My html protocols contains a rich API for generating correct HTML. 

String streamContents: [ :out | | html |
	html := ZnHtmlOutputStream on: out.
	html html5.
	html tag: #html do: [ 
		html tag: #body do: [
			html tag: #div class: #main do: [
				html tag: #p with: 'Hello World & Universe !!'.
				html tag: #hr.
				html 
					tag: #em 
					attributes: #(class big id 1 disable nil) 
					with: 'The END' ] ] ] ].
	
ZnHtmlOutputStream streamContents: [ :html |
	html page: 'Hello World' do: [ 
		html tag: #div class: #main do: [
			html tag: #p with: 'Hello World & Universe !!' ] ] ]

Part of Zinc HTTP Components.!

!ZnHtmlOutputStream class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 11/25/2015 10:39'!
on: writeStream
	^ self new
		on: writeStream;
		yourself! !

!ZnHtmlOutputStream class methodsFor: 'convenience' stamp: 'SvenVanCaekenberghe 11/30/2015 22:04'!
streamContents: block
	"Execute block with a ZnHtmlOutputStream as argument to generate HTML, returning the resulting string"
	
	"ZnHtmlOutputStream streamContents: [ :html |
		html page: 'Hello World' do: [ 
			html tag: #p with: 'Hello World & Universe !!' ] ]"
	
	^ String streamContents: [ :out |
			block value: (self on: out) ]! !

!ZnHtmlOutputStream methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 11/25/2015 15:50'!
<< collection
	^ self nextPutAll: collection asString! !

!ZnHtmlOutputStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/25/2015 10:40'!
close
	stream close! !

!ZnHtmlOutputStream methodsFor: 'private html' stamp: 'SvenVanCaekenberghe 11/25/2015 11:23'!
closeTag: tag
	stream
		nextPut: $<;
		nextPut: $/;
		nextPutAll: tag;
		nextPut: $>! !

!ZnHtmlOutputStream methodsFor: 'html' stamp: 'SvenVanCaekenberghe 11/30/2015 21:41'!
comment: string
	"Write an HTML/XML comment consisting of string, as in <!!-- string -->"
	
	stream
		nextPutAll: '<!!-- ';
		nextPutAll: string;
		nextPutAll: ' -->'! !

!ZnHtmlOutputStream methodsFor: 'html' stamp: 'SvenVanCaekenberghe 11/30/2015 21:42'!
escape: string
	"Write string, escaping characters as needed for regular text"
	
	string do: [ :each | self escapeCharacter: each ]! !

!ZnHtmlOutputStream methodsFor: 'html' stamp: 'SvenVanCaekenberghe 11/30/2015 21:50'!
escapeAttributeValue: string
	"Write string, escaping characters as needed for the value of an attribute"

	string do: [ :each |
		each == $"
			ifTrue: [ stream nextPutAll: '&quot;' ]
			ifFalse: [ stream nextPut: each ] ] ! !

!ZnHtmlOutputStream methodsFor: 'html' stamp: 'SvenVanCaekenberghe 11/30/2015 21:43'!
escapeCharacter: char
	"Write char, escaping it as needed for regular text"

	char == $<
		ifTrue: [ stream nextPutAll: '&lt;' ]
		ifFalse: [ 
			char == $&
				ifTrue: [ stream nextPutAll: '&amp;' ]
				ifFalse: [ stream nextPut: char ] ]! !

!ZnHtmlOutputStream methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 11/25/2015 10:43'!
flush
	stream flush! !

!ZnHtmlOutputStream methodsFor: 'html' stamp: 'SvenVanCaekenberghe 11/30/2015 21:50'!
format: string with: args
	"Like String>>#format: format the string template using the args given, esacaping characters when needed for regular text"

	| input currentChar |
	input := string readStream.
	[ input atEnd ] whileFalse: [
		(currentChar := input next) == ${
			ifTrue: [ | expression index | 
				expression := input upTo: $}.
				index := Integer readFrom: expression ifFail: [ expression ].
				self escape: (args at: index) asString ]
			ifFalse: [
				currentChar == $\
					ifTrue: [ input atEnd ifFalse: [ self escapeCharacter: stream next ] ]
					ifFalse: [ self escapeCharacter: currentChar ] ] ]! !

!ZnHtmlOutputStream methodsFor: 'html' stamp: 'SvenVanCaekenberghe 11/30/2015 21:44'!
html5
	"Write the standard HTML5 DOCTYPE tag"
	
	stream nextPutAll: '<!!DOCTYPE html>'! !

!ZnHtmlOutputStream methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 11/25/2015 10:43'!
next: count putAll: collection	
	self 
		next: count 
		putAll: collection 
		startingAt: 1! !

!ZnHtmlOutputStream methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 11/25/2015 10:46'!
next: count putAll: collection startingAt: offset
	"Write count characters from collection starting at offset."
	
	stream 
		next: count 
		putAll: collection 
		startingAt: offset! !

!ZnHtmlOutputStream methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 11/25/2015 10:46'!
nextPut: object
	^ stream nextPut: object! !

!ZnHtmlOutputStream methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 11/25/2015 10:43'!
nextPutAll: collection
	self 
		next: collection size 
		putAll: collection 
		startingAt: 1! !

!ZnHtmlOutputStream methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/25/2015 10:40'!
on: writeStream
	stream := writeStream! !

!ZnHtmlOutputStream methodsFor: 'private html' stamp: 'SvenVanCaekenberghe 11/25/2015 11:21'!
openTag: tag
	stream
		nextPut: $<;
		nextPutAll: tag;
		nextPut: $>! !

!ZnHtmlOutputStream methodsFor: 'private html' stamp: 'SvenVanCaekenberghe 11/25/2015 15:45'!
openTag: tag attribute: name value: value
	stream
		nextPut: $<;
		nextPutAll: tag;
		space; 
		nextPutAll: name.
	value ifNotNil: [
		stream nextPutAll: '="'.
		self escapeAttributeValue: value asString.
		stream nextPut: $" ].
	stream nextPut: $>! !

!ZnHtmlOutputStream methodsFor: 'private html' stamp: 'SvenVanCaekenberghe 11/25/2015 16:04'!
openTag: tag attributes: attributes
	stream
		nextPut: $<;
		nextPutAll: tag.
	self renderAttributes: attributes.
	stream nextPut: $>! !

!ZnHtmlOutputStream methodsFor: 'html' stamp: 'SvenVanCaekenberghe 11/30/2015 21:51'!
page: title do: block
	"Write out a standard page using title, then execute block"
	
	"ZnHtmlOutputStream streamContents: [ :html | html page: 'Hello' do: [ html tag: #p with: 'World' ] ]"

	self html5; tag: #html do: [ 
		self
			tag: #head do: [ self tag: #title with: title ];
			tag: #body do: [ 
				self tag: #h1 with: title.
				block value ] ]! !

!ZnHtmlOutputStream methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 11/25/2015 10:43'!
print: object
	object printOn: self! !

!ZnHtmlOutputStream methodsFor: 'private html' stamp: 'SvenVanCaekenberghe 11/25/2015 16:03'!
renderAttributes: attributes
	attributes pairsDo: [ :name :value | 
		stream space; nextPutAll: name.
		value ifNotNil: [
			stream nextPutAll: '="'.
			self escapeAttributeValue: value asString.
			stream nextPut: $" ] ]! !

!ZnHtmlOutputStream methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 11/25/2015 10:43'!
space
	self nextPut: Character space! !

!ZnHtmlOutputStream methodsFor: 'html' stamp: 'SvenVanCaekenberghe 11/30/2015 21:45'!
str: string
	"Write string, escaping characters as needed for regular text"

	self escape: string! !

!ZnHtmlOutputStream methodsFor: 'html tags' stamp: 'SvenVanCaekenberghe 11/30/2015 21:55'!
tag: tag
	"Write a standalone, self closing HTML tag"
	
	"<tag/>"
	
	stream
		nextPut: $<;
		nextPutAll: tag;
		nextPut: $/;
		nextPut: $>! !

!ZnHtmlOutputStream methodsFor: 'html tags' stamp: 'SvenVanCaekenberghe 11/30/2015 21:55'!
tag: tag attributes: attributes
	"Write a standalone, self closing HTML tag with attributes, a collection of alternating keys and values"
	
	"<tag attr1=""value1"" ... attrN=""valueN""/>"

	stream
		nextPut: $<;
		nextPutAll: tag.
	self renderAttributes: attributes.
	stream
		nextPut: $/;
		nextPut: $>! !

!ZnHtmlOutputStream methodsFor: 'html tags' stamp: 'SvenVanCaekenberghe 11/30/2015 21:55'!
tag: tag attributes: attributePairs do: block
	"Write an HTML tag with attributes, a collection of alternating keys and values, executing block to generate enclosed content" 
	
	"<tag attr1=""value1"" ... attrN=""valueN""> ... </tag>"

	self openTag: tag attributes: attributePairs.
	block value.
	self closeTag: tag! !

!ZnHtmlOutputStream methodsFor: 'html tags' stamp: 'SvenVanCaekenberghe 11/30/2015 21:56'!
tag: tag attributes: attributePairs with: string
	"Write an HTML tag with attributes, a collection of alternating keys and values, using the escaped string as content" 
	
	"<tag attr1=""value1"" ... attrN=""valueN"">string</tag>"

	self openTag: tag attributes: attributePairs.
	self escape: string.
	self closeTag: tag ! !

!ZnHtmlOutputStream methodsFor: 'html tags' stamp: 'SvenVanCaekenberghe 11/30/2015 21:57'!
tag: tag class: cssClass do: block
	"Write an HTML tag with class cssClass, executing block to generate enclosed content" 
	
	"<tag class=""cssClass""> ... </tag>"

	self openTag: tag attribute: #class value: cssClass.
	block value.
	self closeTag: tag! !

!ZnHtmlOutputStream methodsFor: 'html tags' stamp: 'SvenVanCaekenberghe 11/30/2015 21:58'!
tag: tag class: cssClass with: string
	"Write an HTML tag with class cssClass, using the escaped string as content" 
	
	"<tag class=""cssClass"">string</tag>"

	self openTag: tag attribute: #class value: cssClass.
	self escape: string.
	self closeTag: tag ! !

!ZnHtmlOutputStream methodsFor: 'html tags' stamp: 'SvenVanCaekenberghe 11/30/2015 22:01'!
tag: tag do: block
	"Write an HTML tag, executing block to generate enclosed content" 
	
	"<tag>...</tag>"

	self openTag: tag.
	block value.
	self closeTag: tag! !

!ZnHtmlOutputStream methodsFor: 'html tags' stamp: 'SvenVanCaekenberghe 11/30/2015 21:59'!
tag: tag id: cssId do: block
	"Write an HTML tag with id cssId, executing block to generate enclosed content" 
	
	"<tag id=""cssId""> ... </tag>"

	self openTag: tag attribute: #id value: cssId.
	block value.
	self closeTag: tag! !

!ZnHtmlOutputStream methodsFor: 'html tags' stamp: 'SvenVanCaekenberghe 11/30/2015 22:00'!
tag: tag id: cssId with: string
	"Write an HTML tag with id cssId, using the escaped string as content" 
	
	"<tag id=""cssId"">string</tag>"

	self openTag: tag attribute: #id value: cssId.
	self escape: string.
	self closeTag: tag ! !

!ZnHtmlOutputStream methodsFor: 'html tags' stamp: 'SvenVanCaekenberghe 11/30/2015 22:01'!
tag: tag with: string
	"Write an HTML tag using the escaped string as content" 
	
	"<tag>string</tag>"

	self openTag: tag.
	self escape: string.
	self closeTag: tag ! !

!ZnHtmlOutputStream methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/25/2015 10:40'!
wrappedStream
	^ stream! !

Object subclass: #ZnImageExampleDelegate
	instanceVariableNames: 'image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnImageExampleDelegate commentStamp: '<historical>' prior: 0!
I am ZnImageExampleDelegate.
I am a implementation of the web app in 'Building and deploying your first web app in Pharo'.

I serve an image that web clients can change by uploading a new one.

  ZnServer startDefaultOn: 1701.
  ZnImageExampleDelegate installInDefaultServer.
  ZnServer stopDefault.

I implement the following API or resources

	GET /image - returns an HTML page showing our image and a form to upload a new image
	GET /image?raw=true - directly serves the last uploaded image
	POST /image - handler accepting a multipart form data entity with a file part containing GIF, JPEG or PNG bytes
	
Part of Zinc HTTP Components.!

!ZnImageExampleDelegate class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 7/6/2013 00:12'!
installInDefaultServer
	"Assuming the default server has the default delegate, 
	install ourself under /image"
	
	ZnServer default delegate 
		map: 'image' to: self new! !

!ZnImageExampleDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/26/2014 23:40'!
downloadPharoLogo
	^ ZnClient new beOneShot
		get: 'http://pharo.org/files/pharo.png';
		entity! !

!ZnImageExampleDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/6/2013 00:01'!
form
	^ self formForImageEntity: self image
	! !

!ZnImageExampleDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/6/2013 00:10'!
formForImageEntity: imageEntity
	"Parse the actual bytes of imageEntity and return a Form."
	
	| imageType parserClassName parserClass parser |
	imageType := imageEntity contentType sub.
	parserClassName := imageType asUppercase, #ReadWriter.
	parserClass := Smalltalk globals at: parserClassName asSymbol.
	parser := parserClass on: imageEntity readStream.
	^ parser nextImage
	! !

!ZnImageExampleDelegate methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 7/6/2013 00:07'!
handleGetRequest: request
	"GET /image - Serve an HTML page showing our image and a form to upload a new one.
	GET /image?raw=true - Serve the last image directly"
	
	^ (request uri queryAt: #raw ifAbsent: [ nil ])
		ifNil: [ 
			ZnResponse ok: (ZnEntity html: self html) ]
		ifNotNil: [ 
			ZnResponse ok: self image ]! !

!ZnImageExampleDelegate methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 7/6/2013 11:42'!
handlePostRequest: request
	"POST /image <multipart form data with file part> - change the image and show the result.
	Do full error checking before accepting the newly uploaded image"
	
	| part newImage badRequest |
	badRequest := [ ^ ZnResponse badRequest: request ].
	(request hasEntity and: [ request contentType matches: ZnMimeType multiPartFormData ])
		ifFalse: badRequest.
	part := request entity 
		partNamed: #file 
		ifNone: badRequest.
	newImage := part entity.
	(newImage notNil and: [ newImage contentType matches: 'image/*' asZnMimeType ])
		ifFalse: badRequest.
	[ self formForImageEntity: newImage ] on: Error do: badRequest.
	image := newImage.
	^ ZnResponse redirect: #image
	! !

!ZnImageExampleDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 7/6/2013 11:43'!
handleRequest: request
	"Dispatch between GET and POST on /image"
	
	request uri path = #image
		ifTrue: [ 
			request method = #GET
				ifTrue: [ 
					^ self handleGetRequest: request ].
			request method = #POST
				ifTrue: [ 
					^ self handlePostRequest: request ] ].
	^ ZnResponse notFound: request uri! !

!ZnImageExampleDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/30/2015 13:43'!
html
	^ ZnHtmlOutputStream streamContents: [ :html |
		 html page: 'Image' do: [
			html 
				tag: #src attributes: #(src. 'image?raw=true');
				tag: #br;
				tag: #form attributes: #(enctype 'multipart/form-data' action image method POST) do: [
					html
						tag: #h3 with: 'Change the image:';
						tag: #input attributes: #(type file name file);
						tag: #input attributes: #(type submit value Upload) ] ] ]! !

!ZnImageExampleDelegate methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/6/2013 00:09'!
image
	"Return the current image entity.
	Use a default when not yet set."
	
	^ image ifNil: [ image := self downloadPharoLogo ]! !

!ZnImageExampleDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 7/5/2013 14:28'!
value: request
	"I implement the generic #value: message as equivalent to #handleRequest:"
	
	^ self handleRequest: request! !

Object subclass: #ZnLineReader
	instanceVariableNames: 'stream buffer position limit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Streaming'!

!ZnLineReader commentStamp: 'SvenVanCaekenberghe 9/27/2010 17:31' prior: 0!
I am ZnLineReader and I help reading CRLF terminated lines from a stream.
I can work on ascii and binary streams doing conversions as necessary.
I limit lines to a maximum length.
I work with a reusable buffer.

Part of Zinc HTTP Components.!

!ZnLineReader class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/27/2010 17:15'!
on: stream
	^ self new
		on: stream;
		yourself! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/28/2011 14:36'!
growBuffer
	| newBuffer |
	buffer size >= limit 
		ifTrue: [
			(ZnLineTooLong limit: limit) signal ].
	newBuffer := String new: buffer size * 2.
	newBuffer replaceFrom: 1 to: buffer size with: buffer.
	buffer := newBuffer ! !

!ZnLineReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 17:30'!
limit: size
	limit := size! !

!ZnLineReader methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 19:05'!
nextLine
	self reset.
	[ self processNext isNil ] whileFalse.
	^ buffer copyFrom: 1 to: position ! !

!ZnLineReader methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 1/2/2012 17:51'!
on: readStream
	stream := readStream.
	buffer := String new: 64.
	self 
		limit: ZnConstants maximumLineLength; 
		reset! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/12/2011 14:07'!
processNext
	| item |
	stream atEnd ifTrue: [ ^ nil ].
	(item := stream next) ifNil: [ ^ nil ].
	stream isBinary ifTrue: [ item := item asCharacter ].
	item == Character lf
		ifTrue: [
			(position > 0 and: [ (buffer at: position) == Character cr ])
				ifTrue: [
					position := position - 1.
					^ nil ] ].
	self store: item.
	^ item
			
	! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 17:29'!
reset
	position := 0! !

!ZnLineReader methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/27/2010 18:43'!
store: item
	position >= buffer size ifTrue: [ self growBuffer ].
	position := position + 1.
	buffer at: position put: item		
	! !

Object subclass: #ZnMessage
	instanceVariableNames: 'headers entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMessage commentStamp: 'SvenVanCaekenberghe 12/6/2010 12:46' prior: 0!
I am ZnMessage, an abstract class representing an HTTP Message.

Part of Zinc HTTP Components.!

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 4/7/2012 16:41'!
readBinaryFrom: stream
	^ self new
		readBinaryFrom: stream;
		yourself! !

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:56'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 12:18'!
readHeaderFrom: stream
	^ self new
		readHeaderFrom: stream;
		yourself! !

!ZnMessage class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:56'!
readStreamingFrom: stream
	^ self new
		readStreamingFrom: stream;
		yourself! !

!ZnMessage methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:38'!
= other
	self class = other class ifFalse: [ ^ false ].
	^ self headers = other headers and: [ self entity = other entity ]! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/14/2012 10:37'!
clearEntity
	"Set my entity to nil. 
	Unlike #resetEntity:, my content type and length remain unchanged."

	entity ifNotNil: [
		entity close.
		entity := nil ]! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:17'!
contentLength
	self hasEntity ifTrue: [ ^ self entity contentLength ].
	(self hasHeaders and: [ self headers hasContentLength ]) ifTrue: [ ^ self headers contentLength ].
	^ nil! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:15'!
contentType
	self hasEntity ifTrue: [ ^ self entity contentType ].
	(self hasHeaders and: [ self headers hasContentType ]) ifTrue: [ ^ self headers contentType ].
	^ nil! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/28/2010 13:03'!
contents
	^ entity isNil ifTrue: [ nil ] ifFalse: [ entity contents ]! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 22:05'!
cookies
	self subclassResponsibility ! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:34'!
entity
	^ entity! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/19/2011 10:09'!
entity: object
	"Set my entity to object. Unless I already am describing an entity using my content type and length
	set my content type and length as defined by object."

	entity := object.
	self headers isDescribingEntity ifFalse: [
		self headers acceptEntityDescription: object ]! !

!ZnMessage methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/1/2011 19:02'!
entityReaderOn: stream
	^ ZnEntityReader new
		headers: self headers;
		stream: stream;
		yourself! !

!ZnMessage methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/18/2013 12:51'!
entityWriterOn: stream
	^ ZnEntityWriter new
		headers: self headers;
		stream: stream;
		yourself! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/21/2010 12:02'!
hasEntity
	^ self entity notNil! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 20:32'!
hasHeaders
	^ headers notNil and: [ self headers isEmpty not ]! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/10/2012 18:58'!
hasSession
	"Return if there currently is a server session.
	This only returns a value during #handleRequest:"

	^ ZnCurrentServerSession value notNil! !

!ZnMessage methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:37'!
hash
	^ self headers hash bitXor: self entity hash! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:34'!
headers
	headers isNil ifTrue: [ headers := ZnHeaders new ].
	^ headers! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers: object
	headers := object! !

!ZnMessage methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/21/2010 12:04'!
headersDo: twoArgumentBlock
	self hasHeaders
		ifTrue: [ self headers headersDo: twoArgumentBlock ]! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:51'!
isConnectionClose
	| value |
	value := self headers at: 'Connection' ifAbsent: [ ^ false ].
	^ value sameAs: 'close'! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:51'!
isConnectionKeepAlive
	| value |
	value := self headers at: 'Connection' ifAbsent: [ ^ false ].
	^ value sameAs: 'keep-alive'! !

!ZnMessage methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 6/12/2014 15:56'!
postCopy
	headers := headers copy.
	"Note that we don't copy the entity, see also #resetEntity: and ZnClient>>#resetEntity"! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 4/7/2012 16:42'!
readBinaryFrom: stream
	| entityReader |
	self readHeaderFrom: stream.
	(entityReader := self entityReaderOn: stream)
		binary.
	self entity: entityReader readEntity! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/28/2011 10:31'!
readEntityFrom: stream
	self entity: (self entityReaderOn: stream) readEntity
	! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/28/2011 10:31'!
readFrom: stream
	self readHeaderFrom: stream.
	self readEntityFrom: stream
	! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:32'!
readHeaderFrom: stream
	self headers: (ZnHeaders readFrom: stream)
	! !

!ZnMessage methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 12:17'!
readStreamingFrom: stream
	| entityReader |
	self readHeaderFrom: stream.
	(entityReader := self entityReaderOn: stream)
		streaming.
	self entity: entityReader readEntity
	! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/19/2011 10:26'!
resetEntity: object
	"Set my entity to object. Always set my content type and length as defined by object,
	even if I am already describing an entity using my content type and length.
	See also: #clearEntity"

	(entity ~= object and: [ entity notNil ]) 
		ifTrue: [ entity close ].
	entity := object.
	self headers acceptEntityDescription: object! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 17:32'!
server
	"Return the current server.
	This only returns a value during #handleRequest:"

	^ ZnCurrentServer value! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 19:02'!
session
	"Return the current server session.
	This only returns a value during #handleRequest:"

	^ ZnCurrentServerSession value! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 22:32'!
setConnectionClose
	self headers at: 'Connection' put: 'close'! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 13:58'!
setConnectionKeepAlive
	self headers at: 'Connection' put: 'keep-alive'! !

!ZnMessage methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/29/2013 14:37'!
setContentType: mimeType
	"Explicitly set my Content-Type to mimeType.
	I normally do this automatically by taking the #contentType from my entity."
	
	self headers contentType: mimeType! !

!ZnMessage methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/20/2012 16:07'!
wantsConnectionClose
	"Return if the HTTP protocol should close the connection after processing the receiver"
	
	^ false! !

!ZnMessage methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/18/2013 12:52'!
writeOn: stream
	| bivalentWriteStream |
	bivalentWriteStream := ZnBivalentWriteStream on: stream.
	self headers writeOn: bivalentWriteStream.
	bivalentWriteStream nextPutAll: String crlf.
	self hasEntity ifTrue: [ 
		(self entityWriterOn: bivalentWriteStream) writeEntity: self entity ]! !

!ZnMessage methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 1/24/2013 10:00'!
writeToTranscript
	"Write the receiver on the Transcript like when sent or received over the network.
	Useful for debugging, obviously"
	
	self writeOn: Transcript.
	Transcript flush! !

ZnMessage subclass: #ZnRequest
	instanceVariableNames: 'requestLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnRequest commentStamp: 'SvenVanCaekenberghe 1/18/2011 10:58' prior: 0!
I am ZnRequest, representing an HTTP Request, 
consisting of a request line, headers and an optional entity (body).
I am a ZnMessage.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:07'!
delete: urlObject
	^ self method: #DELETE url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/27/2011 14:57'!
empty
	^ self new
		requestLine: ZnRequestLine empty;
		headers: ZnHeaders defaultRequestHeaders;
		yourself! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:00'!
get: urlObject
	^ self method: #GET url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:07'!
head: urlObject
	^ self method: #HEAD url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/4/2011 14:01'!
method: method url: urlObject
	| url |
	url := urlObject asZnUrl.
	^ self new
		requestLine: (ZnRequestLine method: method uri: url);
		headers: (ZnHeaders requestHeadersFor: url);
		yourself! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/3/2015 14:38'!
options: urlObject
	^ self method: #OPTIONS url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/3/2015 14:38'!
patch: urlObject
	^ self method: #PATCH url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:08'!
post: urlObject
	^ self method: #POST url: urlObject! !

!ZnRequest class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/21/2010 11:08'!
put: urlObject
	^ self method: #PUT url: urlObject! !

!ZnRequest methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:39'!
= other
	^ super = other and: [ self requestLine = other requestLine ]! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/7/2013 09:06'!
absoluteUrl
	^ self host 
		ifNil: [ self url ]
		ifNotNil: [ :hostUrl | self url inContextOf: hostUrl ]! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/17/2011 16:28'!
accept
	^ self headers at: 'Accept'! !

!ZnRequest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/19/2013 20:35'!
acceptsEncodingGzip
	| value |
	value := self headers at: 'Accept-Encoding' ifAbsent: [ ^ false ].
	^ value includesSubstring: 'gzip'! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/23/2012 22:05'!
authorization
	^ self headers at: 'Authorization'! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/23/2012 22:12'!
basicAuthentication
	| authorization username password |
	authorization := self authorization findTokens: ' '.
	(authorization size = 2 and: [ authorization first = 'Basic' ]) ifFalse: [ 
		^ NotFound signal: 'Incomplete Basic Authentication' ].
	authorization := (ZnUtils decodeBase64: authorization second) findTokens: ':'.
	authorization size = 2 ifFalse: [
		^ NotFound signal: 'Incomplete Basic Authentication' ].
	^ authorization! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 22:41'!
cookies
	| value cookies |
	value := self headers at: 'Cookie' ifAbsent: [ ^ #() ].
	cookies := value isString
		ifTrue: [ value findTokens: #($;) ]
		ifFalse: [ value ].
	^ cookies collect: [ :each | ZnCookie fromString: each ]! !

!ZnRequest methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:37'!
hash
	^ super hash bitXor: self requestLine hash! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/7/2013 11:44'!
host
	| host scheme |
	host := self headers at: 'Host' ifAbsent: [ ^ nil ].
	scheme :=ZnCurrentServer value ifNil: [ #http ] ifNotNil: [ :server | server scheme ].
	^ ZnUrl fromString: host defaultScheme: scheme! !

!ZnRequest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:33'!
isHttp10
	^ self requestLine isHttp10 ! !

!ZnRequest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:33'!
isHttp11
	^ self requestLine isHttp11! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/7/2013 10:52'!
mergedFields
	"Return a new ZnMultiValueDictionary containing all query fields, if any,
	merged with all ZnApplicationFormUrlEncodedEntity fields, if any,
	preserving multi values for identical keys"

	| fields |
	fields := ZnMultiValueDictionary new.
	self uri hasQuery
		ifTrue: [ fields addAllMulti: self uri query ].
	(self contentType = ZnMimeType applicationFormUrlEncoded and: [ self hasEntity ])
		ifTrue: [ fields addAllMulti: self entity fields ].
	^ fields! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/8/2010 10:35'!
method
	^ self requestLine method ! !

!ZnRequest methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/27/2011 14:45'!
method: method
	self requestLine method: method! !

!ZnRequest methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 6/12/2014 15:56'!
postCopy
	super postCopy.
	requestLine := requestLine copy! !

!ZnRequest methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:00'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self requestLine printMethodAndUriOn: stream.
	stream nextPut: $)! !

!ZnRequest methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:33'!
readHeaderFrom: stream
	self requestLine: (ZnRequestLine readFrom: stream).
	super readHeaderFrom: stream
	! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/7/2013 09:07'!
relativeUrl
	^ self url asRelativeUrl ! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:06'!
requestLine
	^ requestLine ! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 21:06'!
requestLine: object
	requestLine := object! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 19:04'!
session
	"Return the current server session.
	If necessary a new session is created.
	This only returns a value during #handleRequest:"

	^ super session
		ifNil: [ 
			| session |
			session := self server sessionFor: self.
			ZnCurrentServerSession value: session.
			session ]! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 20:00'!
setAccept: object
	self headers at: 'Accept' put: object asString! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/27/2010 19:28'!
setAcceptEncodingGzip
	self headers at: 'Accept-Encoding' put: 'gzip'! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/23/2012 22:04'!
setAuthorization: authorization
	self headers at: 'Authorization'  put: authorization! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/23/2012 22:05'!
setBasicAuthenticationUsername: username password: password
	(username isNil | password isNil)
		ifTrue: [
			self headers removeKey: 'Authorization' ifAbsent: []]
		ifFalse: [
			self setAuthorization: 'Basic ', (ZnUtils encodeBase64: (username, ':', password)) ]
! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 2/29/2012 19:49'!
setCookie: cookie
	self headers at: 'Cookie' put: cookie
! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/28/2011 10:56'!
setIfModifiedSince: reference
	self headers at: 'If-Modified-Since' put: (ZnUtils httpDate: reference)! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/8/2010 10:36'!
uri
	^ self requestLine uri ! !

!ZnRequest methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/21/2010 21:48'!
url
	^ self uri ! !

!ZnRequest methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 7/27/2011 14:44'!
url: url
	self requestLine uri: url.
	self headers request: self url! !

!ZnRequest methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/20/2012 16:09'!
wantsConnectionClose
	"Return if the HTTP protocol should close the connection after processing the receiver.
	Overwritten to return true in case of the presense of an explicit connection close request header
	or the absense of a keep alive header when using the old HTTP 1.0 protocol."

	^ self isConnectionClose 
		or: [ self isHttp10 and: [ self isConnectionKeepAlive not ] ]! !

!ZnRequest methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:36'!
writeOn: stream
	| bivalentWriteStream |
	bivalentWriteStream := ZnBivalentWriteStream on: stream.
	self requestLine writeOn: bivalentWriteStream.
	super writeOn: bivalentWriteStream! !

ZnMessage subclass: #ZnResponse
	instanceVariableNames: 'statusLine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnResponse commentStamp: 'SvenVanCaekenberghe 1/18/2011 10:59' prior: 0!
I am ZnResponse, representing an HTTP Response 
consisting of a status line, headers and an optional entity (body).
I am a ZnMessage.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:17'!
badRequest: request
	| message |
	message := String streamContents: [ :out |
		out << 'Bad Request '; << request method; space.
		request uri printPathQueryFragmentOn: out ].
	^ self 
		badRequest: request 
		entity: (ZnEntity textCRLF: message)! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:15'!
badRequest: request entity: entity
	^ (self statusLine: ZnStatusLine badRequest)
		entity: entity;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:17'!
created: url
	^ self 
		created: url 
		entity: (ZnEntity textCRLF: 'Created ' , url asString)! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:22'!
created: url entity: entity
	^ (self statusLine: ZnStatusLine created)
		setLocation: url asString;
		entity: entity;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:18'!
methodNotAllowed: request
	| message |
	message := String streamContents: [ :out |
		out << 'Method Not Allowed '; << request method; space.
		request uri printPathQueryFragmentOn: out ].
	^ self 
		methodNotAllowed: request
		entity: (ZnEntity textCRLF: message)! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:17'!
methodNotAllowed: request entity: entity
	^ (self statusLine: ZnStatusLine methodNotAllowed)
		entity: entity;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/3/2015 15:08'!
noContent
	^ self statusLine: ZnStatusLine noContent! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:18'!
notFound: url
	^ self
		notFound: url
		entity: (ZnEntity textCRLF: 'Not Found ' , url asZnUrl pathQueryFragmentPrintString)! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:18'!
notFound: url entity: entity
	^ (self statusLine: ZnStatusLine notFound)
		entity: entity;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 16:37'!
notModified
	^ self statusLine: ZnStatusLine notModified! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 16:30'!
ok: entity
	^ (self statusLine: ZnStatusLine ok)
		entity: entity;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:23'!
redirect: url
	^ self
		redirect: url
		entity: (ZnEntity textCRLF: 'Redirect ' , url asString)! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:22'!
redirect: url entity: entity
	^ (self statusLine: ZnStatusLine redirect)
		setLocation: url asString;
		entity: entity;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:24'!
serverError: string
	^ self serverErrorWithEntity: (ZnEntity textCRLF: string)! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:24'!
serverErrorWithEntity: entity
	^ (self statusLine: ZnStatusLine internalServerError)
		entity: entity;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 16:24'!
statusCode: httpResponeCode
	^ self statusLine: (ZnStatusLine code: httpResponeCode)! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 16:23'!
statusLine: statusLine
	^ self new
		statusLine: statusLine;
		headers: ZnHeaders defaultResponseHeaders;
		yourself! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'MattKennedy 9/30/2010 15:46'!
unauthorized
	^ self unauthorized: 'Basic realm=ZincHTTPComponents'! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:21'!
unauthorized: authString
	^ self
		unauthorized: authString
		entity: (ZnEntity textCRLF: 'Unauthorized')! !

!ZnResponse class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2015 22:20'!
unauthorized: authString entity: entity
	^ (self statusLine: ZnStatusLine unauthorized)
		setWWWAuthenticate: authString;
		entity: entity;
		yourself! !

!ZnResponse methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:39'!
= other
	^ super = other and: [ self statusLine = other statusLine ]! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 21:44'!
addCookie: cookie
	self headers at: 'Set-Cookie' add: cookie fullString
! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/16/2010 20:44'!
code
	^ self statusLine code! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 21:53'!
cookies
	| value cookies |
	value := self headers at: 'Set-Cookie' ifAbsent: [ ^ #() ].
	cookies := value isString
		ifTrue: [ Array with: value ]
		ifFalse: [ value ].
	^ cookies collect: [ :each | ZnCookie fromString: each ]! !

!ZnResponse methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/1/2011 19:02'!
entityReaderOn: stream
	"Reading undefined content is allowed for responses"
	
	^ (super entityReaderOn: stream)  
		allowReadingUpToEnd;
		yourself! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/19/2013 21:08'!
hasContentEncoding
	^ self headers includesKey: 'Content-Encoding'! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 5/19/2013 21:09'!
hasTransferEncoding
	^ self headers includesKey: 'Transfer-Encoding'! !

!ZnResponse methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:37'!
hash
	^ super hash bitXor: self status hash! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 1/12/2011 10:03'!
isAuthenticationRequired
	^ self code = 401 ! !

!ZnResponse methodsFor: 'testing' stamp: 'ChristopheDemarey 12/11/2014 10:27'!
isBadRequest
	^ self code = 400! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/14/2011 14:43'!
isCreated
	^ self code = 201! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 4/20/2012 10:26'!
isError
	^self code > 399! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/28/2011 10:28'!
isInformational
	^ self code between: 100 and: 199! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/28/2011 10:27'!
isNoContent
	^ self code = 204! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/10/2012 20:05'!
isNotFound
	^ self code = 404 ! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 3/21/2011 10:45'!
isNotModified
	^ self code = 304! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/27/2012 21:31'!
isRedirect
	^ #(301 302 303 307) includes: self code! !

!ZnResponse methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/28/2010 12:38'!
isSuccess
	^ #(200 201 202 204) includes: self code! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:31'!
location
	^ self headers at: 'Location'! !

!ZnResponse methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 6/12/2014 15:57'!
postCopy
	super postCopy.
	statusLine := statusLine copy! !

!ZnResponse methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 12/14/2010 12:54'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self statusLine printCodeAndReasonOn: stream.
	self hasEntity ifTrue: [ 
		stream space.
		self entity printContentTypeAndLengthOn: stream ].
	stream nextPut: $)
	! !

!ZnResponse methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/28/2011 10:36'!
readEntityFrom: stream
	(self isInformational or: [ self isNoContent or: [ self isNotModified ] ])
		ifFalse: [ super readEntityFrom: stream ]! !

!ZnResponse methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/21/2010 11:34'!
readHeaderFrom: stream
	self statusLine: (ZnStatusLine readFrom: stream).
	super readHeaderFrom: stream
	! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/7/2013 10:51'!
setConnectionCloseFor: request
	request wantsConnectionClose 
		ifTrue: [ self setConnectionClose ]! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/19/2013 20:24'!
setContentEncodingGzip
	self headers at: 'Content-Encoding' put: 'gzip'! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:34'!
setKeepAliveFor: request
	(request isHttp10 and: [ request isConnectionKeepAlive ])
		ifTrue: [ self setConnectionKeepAlive ]
	! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 15:19'!
setLocation: uriString
	self headers at: 'Location' put: uriString! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/20/2013 13:02'!
setTransferEncodingChunked
	self headers 
		at: 'Transfer-Encoding' put: 'chunked';
		clearContentLength! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/17/2010 15:19'!
setWWWAuthenticate: string
	self headers at: 'WWW-Authenticate' put: string! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/7/2010 18:35'!
status
	^ self statusLine code! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:36'!
statusLine
	^ statusLine! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 22:36'!
statusLine: object
	statusLine := object! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/20/2012 16:55'!
useConnection: connection
	"Hook method that can be overwritten to give subclasses the chance to 
	keep using connection in the current thread/process after the server wrote the response"! !

!ZnResponse methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/14/2010 11:36'!
writeOn: stream
	| bivalentWriteStream |
	bivalentWriteStream := ZnBivalentWriteStream on: stream.
	self statusLine writeOn: bivalentWriteStream.
	super writeOn: bivalentWriteStream! !

!ZnResponse methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/12/2011 13:28'!
wwwAuthenticate
	^ self headers at: 'WWW-Authenticate'! !

Object subclass: #ZnMimePart
	instanceVariableNames: 'headers entity'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnMimePart commentStamp: 'SvenVanCaekenberghe 10/5/2010 19:20' prior: 0!
I am ZnMimePart.
I hold headers and an optional entity.
I am used in MultiPartMimeEncoding.

Acknowledgement: some code borrowed from AJP.

Part of Zinc HTTP Components.!

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/16/2011 23:28'!
fieldName: fieldName entity: entity
	^ self new
		setContentDisposition: 'form-data;name="', fieldName, '"';
		entity: entity;
		yourself! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 10/9/2014 15:27'!
fieldName: fieldName fileName: fileName entity: entity
	"Pathnames are often silenty encoded using UTF-8,
	this is a no-op for ASCII, but will fail on Latin-1 and others"

	| encodedFileName |
	encodedFileName := fileName utf8Encoded asString.
	^ self new
		setContentDisposition: 'form-data;name="', fieldName, '";filename="', encodedFileName, '"';
		entity: entity;
		yourself! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/2/2012 16:10'!
fieldName: fieldName fileNamed: fileName
	| mimeType size fileStream entity baseName |
	mimeType := ZnMimeType forFilenameExtension: (ZnFileSystemUtils extensionFor: fileName).
	fileStream := ZnFileSystemUtils oldFileStreamFor: fileName.
	mimeType isBinary ifTrue: [ fileStream binary ].
	size := ZnFileSystemUtils fileSizeFor: fileName.
	(entity := ZnStreamingEntity type: mimeType length: size)
		stream: fileStream.
	baseName := ZnFileSystemUtils baseNameFor: fileName.
	^ self fieldName: fieldName fileName: baseName entity: entity
	! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 1/29/2012 19:27'!
fieldName: fieldName value: fieldValue
	^ self
		fieldName: fieldName
		entity: (ZnEntity with: fieldValue)! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 10/16/2014 10:50'!
readBinaryFrom: stream
	^ self new
		readBinaryFrom: stream;
		yourself! !

!ZnMimePart class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/30/2010 19:57'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnMimePart methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:41'!
= other
	self class = other class ifFalse: [ ^ false ].
	^ self headers = other headers and: [ self entity = other entity ]! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/3/2010 16:12'!
contentDisposition
	^ self headers at: 'Content-Disposition' ifAbsent: [ nil ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/6/2010 09:30'!
contentDispositionValues
	| contentDisposition values start semiColonIndex |
	(contentDisposition := self contentDisposition) isNil ifTrue: [ ^ #() ].
	values := OrderedCollection new: 3.
	start := 1.
	semiColonIndex := contentDisposition indexOf: $; startingAt: start ifAbsent: [ 0 ].
	[ semiColonIndex isZero ] whileFalse: [
		| value |
		value := contentDisposition copyFrom: start to: semiColonIndex - 1.
		values add: (ZnUtils trimString: value).
		start := semiColonIndex + 1.
		semiColonIndex := contentDisposition indexOf: $; startingAt: start ifAbsent: [ 0 ] ].
	values add: (ZnUtils trimString: (contentDisposition copyFrom: start to: contentDisposition size)).
	^ values! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contentLength
	self hasEntity ifTrue: [ ^ self entity contentLength ].
	(self hasHeaders and: [ self headers hasContentLength ]) ifTrue: [ ^ self headers contentLength ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contentType
	self hasEntity ifTrue: [ ^ self entity contentType ].
	(self hasHeaders and: [ self headers hasContentType ]) ifTrue: [ ^ self headers contentType ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
contents
	^ entity isNil ifTrue: [ nil ] ifFalse: [ entity contents ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/5/2010 17:01'!
detectContentDispositionValue: aString
	self contentDispositionValues do: [ :each | 
		((each beginsWith: aString)
			and: [ each size > (aString size + 1)
			and: [ (each at: aString size + 1) = $= ] ]) ifTrue: [
				^ (each copyAfter: $=) withoutQuoting ] ].
	^ nil! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
entity
	^ entity! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/19/2011 10:09'!
entity: object
	entity := object.
	self headers isDescribingEntity ifFalse: [
		self headers acceptEntityDescription: object ]! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/3/2010 16:10'!
entityReaderOn: stream
	^ ZnEntityReader new
		headers: self headers;
		stream: stream;
		allowReadingUpToEnd;
		yourself! !

!ZnMimePart methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/18/2013 12:53'!
entityWriterOn: stream
	^ ZnEntityWriter new
		headers: self headers;
		stream: stream;
		yourself! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/5/2010 17:03'!
fieldName
	^ self detectContentDispositionValue: 'name'! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 18:54'!
fieldValue
	^ self hasEntity 
		ifTrue: [ self entity contents ] 
		ifFalse: [ nil ] ! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/29/2011 14:17'!
fieldValueString
	^ self fieldValue
		ifNil: [ String empty ]
		ifNotNil: [ :value | value asString ]! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/10/2014 09:32'!
fileName
	"Pathnames are often silenty encoded using UTF-8,
	this is a no-op for ASCII, but will fail on Latin-1 and others"

	^ (self detectContentDispositionValue: 'filename')
		ifNotNil: [ :encodedFileName | encodedFileName asByteArray utf8Decoded ]! !

!ZnMimePart methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
hasEntity
	^ self entity notNil! !

!ZnMimePart methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/4/2010 13:32'!
hasHeaders
	^ headers notNil and: [ self headers isEmpty not ]! !

!ZnMimePart methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:40'!
hash
	^ self headers hash bitXor: self entity hash! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers
	headers isNil ifTrue: [ headers := ZnHeaders new ].
	^ headers! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headers: object
	headers := object! !

!ZnMimePart methodsFor: 'enumerating' stamp: 'SvenVanCaekenberghe 9/30/2010 19:53'!
headersDo: twoArgumentBlock
	self hasHeaders
		ifTrue: [ self headers headersDo: twoArgumentBlock ]! !

!ZnMimePart methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 6/12/2014 15:58'!
postCopy
	headers := headers copy.
	"Note that we don't copy the entity, see also #resetEntity: and ZnClient>>#resetEntity"! !

!ZnMimePart methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 6/11/2013 11:24'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	self hasEntity ifTrue: [ 
		self entity printContentTypeAndLengthOn: stream ].
	stream nextPut: $)
	! !

!ZnMimePart methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/16/2014 10:46'!
readBinaryFrom: stream
	| entityReader |
	self headers: (ZnHeaders readFrom: stream).
	(entityReader := self entityReaderOn: stream)
		binary.
	self entity: entityReader readEntity! !

!ZnMimePart methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/30/2010 19:54'!
readFrom: stream
	self headers: (ZnHeaders readFrom: stream).
	self entity: (self entityReaderOn: stream) readEntity
	! !

!ZnMimePart methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/4/2010 18:49'!
setContentDisposition: value
	^ self headers at: 'Content-Disposition' put: value ! !

!ZnMimePart methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 5/18/2013 12:54'!
writeOn: stream
	self headers writeOn: stream.
	stream nextPutAll: String crlf.
	self hasEntity ifTrue: [ 
		(self entityWriterOn: stream) writeEntity: self entity ]! !

Object subclass: #ZnNetworkingUtils
	instanceVariableNames: 'socketStreamClass secureSocketStreamClass'
	classVariableNames: 'Default DefaultSocketStreamTimeout'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnNetworkingUtils commentStamp: '<historical>' prior: 0!
I am ZnNetworkingUtils holding various utilities related to HTTP networking.

My class side is a general API.

My instance side is a factory.

ZnNetworkingUtils default socketStreamClass: SocketStream.
ZnNetworkingUtils default socketStreamClass: ZdcSocketStream.

Part of Zinc HTTP Components.!

!ZnNetworkingUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2011 22:08'!
default
	^ Default ifNil: [ Default := self new ]! !

!ZnNetworkingUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 5/2/2011 22:08'!
default: instance
	Default := instance ! !

!ZnNetworkingUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 6/18/2011 22:44'!
defaultSocketStreamTimeout
	"Global default timeout in seconds for SocketStream IO"
	
	^ DefaultSocketStreamTimeout! !

!ZnNetworkingUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 6/18/2011 22:45'!
defaultSocketStreamTimeout: seconds
	"Global default timeout in seconds for SocketStream IO"
	
	^ DefaultSocketStreamTimeout := seconds! !

!ZnNetworkingUtils class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 5/18/2013 20:03'!
initialize
	"The default will try to use ZdcSocketStream and ZdcSecureSocketStream"
	 
	Default := self new.
	DefaultSocketStreamTimeout := 30! !

!ZnNetworkingUtils class methodsFor: 'converting' stamp: 'SvenVanCaekenberghe 1/7/2011 19:42'!
ipAddressToString: byteArray
	^ String streamContents: [ :stream |
			byteArray 
				do: [ :each | stream print: each ] 
				separatedBy: [ stream nextPut: $. ] ]! !

!ZnNetworkingUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 3/24/2011 08:05'!
listenBacklogSize
	"Server socket backlog size (number of pending connection waiting to be accepted)"
	
	^ 32! !

!ZnNetworkingUtils class methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:43'!
proxyAuthorizationHeaderValueToUrl: url
	"Answer the Proxy-Authorization header value for HTTP access to url, or nil if none"
	
	^ self default proxyAuthorizationHeaderValueToUrl: url  ! !

!ZnNetworkingUtils class methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 10/23/2015 16:33'!
proxyUrl
	"Answer the host:port URL the HTTP/S proxy"
	
	^ self default proxyUrl! !

!ZnNetworkingUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 10/6/2015 17:14'!
secureSocketStreamOn: socket
	^ self default 
		secureSocketStreamOn: socket  ! !

!ZnNetworkingUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 5/2/2011 22:40'!
serverSocketOn: port
	^ self default 
		serverSocketOn: port  ! !

!ZnNetworkingUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 3/12/2012 19:22'!
serverSocketOn: port interface: address
	^ self default 
		serverSocketOn: port interface: address ! !

!ZnNetworkingUtils class methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:41'!
shouldProxyUrl: url
	"Answer if we should proxy HTTP access to url."
	
	^ self default shouldProxyUrl: url  ! !

!ZnNetworkingUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 3/24/2011 08:05'!
socketBufferSize
	"Size in bytes for Sockets and SocketStream IO buffers"

	^ 4096! !

!ZnNetworkingUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 5/2/2011 22:40'!
socketStreamOn: socket
	^ self default 
		socketStreamOn: socket  ! !

!ZnNetworkingUtils class methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 3/2/2014 22:42'!
socketStreamTimeout
	"Access the current timeout in seconds for SocketStream IO"
	
	^ ZnConnectionTimeout value 
		ifNil: [ self defaultSocketStreamTimeout ]! !

!ZnNetworkingUtils class methodsFor: 'networking' stamp: 'SvenVanCaekenberghe 5/2/2011 22:40'!
socketStreamToUrl: url
	^ self default
		socketStreamToUrl: url  ! !

!ZnNetworkingUtils methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/18/2012 09:30'!
bufferSize
	^ self class socketBufferSize! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:54'!
httpProxyPassword
	"Return the password for proxy authorization"
	
	^ NetworkSystemSettings proxyPassword ! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:51'!
httpProxyPort
	"Return the port of the proxy server to use"
	
	^ NetworkSystemSettings httpProxyPort! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:54'!
httpProxyServer
	"Return the hostname of the proxy server to use"
	
	^ NetworkSystemSettings httpProxyServer! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'MarcoSchmidt 6/17/2011 15:25'!
httpProxyUser
	"Return the user for proxy authorization"
	
	^ NetworkSystemSettings proxyUser! !

!ZnNetworkingUtils methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 5/20/2013 15:31'!
initialize
	super initialize.
	self socketStreamClass: (Smalltalk globals at: #ZdcSocketStream ifAbsent: [ SocketStream ]).
	self secureSocketStreamClass: (Smalltalk globals at: #ZdcSecureSocketStream ifAbsent: [ nil ])! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:55'!
isProxyAuthorizationRequired
	"Does the proxy require authorization ?"
	
	^ self isProxySet and: [ NetworkSystemSettings useNetworkAuthentification ]! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:55'!
isProxySet
	"Should a proxy be used ?"
	
	^ NetworkSystemSettings useHTTPProxy ! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 6/16/2011 22:49'!
proxyAuthorizationHeaderValueToUrl: url
	"Answer the Proxy-Authorization header value for HTTP access to url, or nil if none"
	
	^ (self isProxyAuthorizationRequired and: [ self shouldProxyUrl: url ])
		ifTrue: [ 'Basic ', (ZnUtils encodeBase64: (self httpProxyUser, ':', self httpProxyPassword)) ] 
		ifFalse: [ nil ]  ! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 10/23/2015 16:31'!
proxyUrl
	"System settings do currently not support HTTPS proxies."

	^ ZnUrl new
		scheme: #http;
		host: self httpProxyServer;
		port: self httpProxyPort;
		yourself! !

!ZnNetworkingUtils methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/13/2011 21:45'!
secureSocketStreamClass
	^ secureSocketStreamClass ifNil: [ 
		secureSocketStreamClass := Smalltalk globals at: #ZdcSecureSocketStream ifAbsent: [ nil ] ]! !

!ZnNetworkingUtils methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/10/2011 13:43'!
secureSocketStreamClass: anObject
	secureSocketStreamClass := anObject! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 10/6/2015 17:14'!
secureSocketStreamOn: socket
	| stream |
	stream := self secureSocketStreamClass on: socket.
	self setSocketStreamParameters: stream.  
	^ stream! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/2/2011 22:41'!
serverSocketOn: port
	| socket |
	socket := Socket newTCP.
	self setServerSocketOptions: socket.
	socket listenOn: port backlogSize: self class listenBacklogSize.
	socket isValid
		ifFalse: [ self error: 'Cannot create socket on port ', port printString ].
	^ socket! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 3/12/2012 19:22'!
serverSocketOn: port interface: address
	| socket |
	socket := Socket newTCP.
	self setServerSocketOptions: socket.
	socket 
		listenOn: port 
		backlogSize: self class listenBacklogSize 
		interface: address.
	socket isValid
		ifFalse: [ self error: 'Cannot create socket on port ', port printString ].
	^ socket! !

!ZnNetworkingUtils methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/2/2011 22:41'!
setServerSocketOptions: socket
	socket
		setOption: 'TCP_NODELAY' value: 1;
		setOption: 'SO_SNDBUF' value: self class socketBufferSize;
		setOption: 'SO_RCVBUF' value: self class socketBufferSize ! !

!ZnNetworkingUtils methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/18/2012 09:30'!
setSocketStreamParameters: stream
	stream 
		binary;
		shouldSignal: true;
		autoFlush: false;
		bufferSize: self bufferSize;
		timeout: self timeout.
! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 4/8/2014 16:11'!
shouldProxyUrl: url
	"Even when a proxy is set, some URLs should not be proxied.
	There is no setting for this in the image.
	We exclude localhost and explicit exceptions from being proxied."
	
	self isProxySet ifFalse: [ ^ false ].
	^ url isLocalHost not
		and: [ 
			[ (NetworkSystemSettings isAnExceptionFor: url) not ]
				on: MessageNotUnderstood
				do: [ true ] ]! !

!ZnNetworkingUtils methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2011 13:47'!
socketStreamClass
	^ socketStreamClass ifNil: [ socketStreamClass := SocketStream ]! !

!ZnNetworkingUtils methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/10/2011 13:15'!
socketStreamClass: anObject
	socketStreamClass := anObject! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 11/10/2011 13:16'!
socketStreamOn: socket
	| stream |
	stream := self socketStreamClass on: socket.
	self setSocketStreamParameters: stream.  
	^ stream! !

!ZnNetworkingUtils methodsFor: 'proxy' stamp: 'SvenVanCaekenberghe 10/23/2015 16:32'!
socketStreamToProxy
	^ self socketStreamToUrlDirectly: self proxyUrl! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 8/11/2011 19:37'!
socketStreamToUrl: url
	url hasHost ifFalse: [ ZnMissingHost signal ].  
	^ (self shouldProxyUrl: url)
		ifTrue: [ self socketStreamToProxy ] 
		ifFalse: [ self socketStreamToUrlDirectly: url ]! !

!ZnNetworkingUtils methodsFor: 'public' stamp: 'SvenVanCaekenberghe 10/17/2014 13:23'!
socketStreamToUrlDirectly: url
	| stream address |
	address := NetNameResolver addressForName: url host timeout: self timeout. 
	stream := (self streamClassForScheme: url scheme) 
		openConnectionToHost: address
		port: url portOrDefault
		timeout: self timeout.  
	self setSocketStreamParameters: stream. 
	"note that for TLS/SSL, #connect should be called"
	^ stream! !

!ZnNetworkingUtils methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/19/2012 16:09'!
streamClassForScheme: scheme
	(#(http ws) includes: scheme) ifTrue: [ 
		^ self socketStreamClass ].
	(#(https wss) includes: scheme) ifTrue: [ 
		^ self secureSocketStreamClass ifNil: [
			self error: 'No secure socket stream class set or available' ] ].
	(ZnUnknownScheme scheme: scheme) signal! !

!ZnNetworkingUtils methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/18/2012 09:30'!
timeout
	^ self class socketStreamTimeout! !

Object subclass: #ZnReadEvalPrintDelegate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnReadEvalPrintDelegate commentStamp: '<historical>' prior: 0!
I am ZnReadEvalPrintDelegate, I export a REPL Web Service.
You can use this service to work with a headless image.

	ZnReadEvalPrintDelegate startInServerOn: 1701.
	
	ZnClient new
		url: 'http://localhost:1701/repl';
		contents: '42 factorial';
		post.
	
Web Service API:

	POST /repl <some Smalltalk code>
	 
Here is an example terminal session:

$ curl http://localhost:1701/repl
# Pharo Smalltalk REPL. POST expressions to evaluate
# Here is one way (type ctrl-d to end input)
curl -X POST -H'Content-Type:text/plain' --data-binary @- http://localhost:1701/repl

$ curl -X POST -H'Content-Type:text/plain' -d '42 factorial' http://localhost:1701/repl
1405006117752879898543142606244511569936384000000000

$ curl -X POST -H'Content-Type:text/plain' --data-binary @- http://localhost:1701/repl
{ (1 to: 10) sum. (1 to: 10) average }
{55. (11/2)}

There is error handling as well:

$ curl -X POST -H'Content-Type:text/plain' --data-binary @- http://localhost:1701/repl
1 plus: 2
MessageNotUnderstood: SmallInteger>>plus:
SmallInteger(Object)>>doesNotUnderstand: #plus:
	Receiver: 1
	Arguments and temporary variables: 
		aMessage: 	plus: 2
		exception: 	MessageNotUnderstood: SmallInteger>>plus:
		resumeValue: 	nil
	Receiver's instance variables: 
1
UndefinedObject>>DoIt
Compiler>>evaluate:in:to:notifying:ifFail:logged:
Compiler>>evaluate:in:to:notifying:ifFail:
Compiler>>evaluate:in:to:
ZnReadEvalPrintDelegate>>evaluate: in Block: [| result |...
BlockClosure>>on:do:
ZnReadEvalPrintDelegate>>evaluate: in Block: [:out | [| result |...
String class(SequenceableCollection class)>>new:streamContents:

WARNING: never open this service beyond your local network !!
This service gives you absolute control over and access to everything in your image. 
For example, the following will kill your image:

$ curl -X POST -H'Content-Type:text/plain' -d 'Smalltalk quitPrimitive' http://localhost:1701/repl

Part of Zinc HTTP Components. !

!ZnReadEvalPrintDelegate class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/15/2013 22:05'!
startInServerOn: port
	"Start a new server bound to port on the local network running a REPL web service"

	"self startInServerOn: 1701"
	
	 ^ (ZnServer on: port)
			bindingAddress: NetNameResolver loopBackAddress;
			delegate: self new;
			start;
			yourself! !

!ZnReadEvalPrintDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/29/2015 14:48'!
evaluate: string
	| output |
	output := String streamContents: [ :out | 
		[ 	| compiler result |
			compiler := [ self class compiler ] 
								on: MessageNotUnderstood 
								do: [ Smalltalk at: #Compiler "for Pharo 2.0" ].		
			result := compiler evaluate: string. 
			out print: result; cr ]
			on: Error
			do: [ :exception |
				out print: exception; cr.
				exception signalerContext printDetails: out.
				exception signalerContext sender debugStack: 8 on: out ].
		out cr ].
	^ String lf join: output lines! !

!ZnReadEvalPrintDelegate methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 5/16/2013 09:51'!
handleGetRequest: request
	^ ZnResponse ok: (ZnEntity text: (self helpText format: { request server port })) ! !

!ZnReadEvalPrintDelegate methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 5/16/2013 09:54'!
handlePostRequest: request
	^ ZnResponse ok: (ZnEntity text: (self evaluate: request entity contents))! !

!ZnReadEvalPrintDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/16/2013 09:52'!
handleRequest: request
	request uri firstPathSegment = #repl
		ifTrue: [ 
			request method = #GET
				ifTrue: [ 
					^ self handleGetRequest: request ].
			(request method = #POST and: [ request hasEntity and: [ request contentType isBinary not ] ])
				ifTrue: [ 
					^ self handlePostRequest: request ] ].
	^ ZnResponse notFound: request uri! !

!ZnReadEvalPrintDelegate methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/15/2013 21:53'!
helpText
	^ '# Pharo Smalltalk REPL. POST expressions to evaluate\# Here is one way (type ctrl-d to end input)\curl -X POST -H''Content-Type:text/plain'' --data-binary @- http://localhost:{1}/repl\' 
			withCRs 
				replaceAll: Character cr with: Character lf; 
				yourself! !

!ZnReadEvalPrintDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/15/2013 13:44'!
value: request
	"I implement the generic #value: message as equivalent to #handleRequest:"
	
	^ self handleRequest: request! !

Object subclass: #ZnRequestLine
	instanceVariableNames: 'method uri version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnRequestLine commentStamp: '<historical>' prior: 0!
I am an HTTP Request Line.
I consist of an HTTP method, an URI and an HTTP protocol version.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnRequestLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 7/27/2011 14:58'!
empty
	^ self new
		version: ZnConstants defaultHTTPVersion;
		yourself ! !

!ZnRequestLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:18'!
method: method uri: uri
	^ self new
		method: method;
		uri: uri;
		version: ZnConstants defaultHTTPVersion;
		yourself ! !

!ZnRequestLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 13:50'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnRequestLine methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 11:19'!
= other
	self class = other class ifFalse: [ ^ false ].
	^ self method = other method
		and: [ self uri asRelativeUrl = other uri asRelativeUrl
			and: [ self version = other version ] ]! !

!ZnRequestLine methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:33'!
hash
	^ (self method hash bitXor: self uri hash) bitXor: self version hash! !

!ZnRequestLine methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:30'!
isHttp10
	^ self version = 'HTTP/1.0'! !

!ZnRequestLine methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/14/2010 14:31'!
isHttp11
	^ self version = 'HTTP/1.1'! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
method
	^ method! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 14:04'!
method: string
	method := ZnConstants knownHTTPMethods 
		detect: [ :each | each = string ] 
		ifNone: [ (ZnUnknownHttpMethod method: string) signal ] ! !

!ZnRequestLine methodsFor: 'copying' stamp: 'SvenVanCaekenberghe 6/12/2014 15:59'!
postCopy
	uri := uri copy! !

!ZnRequestLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printMethodAndUriOn: stream
	(self method notNil and: [ self uri notNil ])
		ifFalse: [ ^ self ].
	stream
		nextPutAll: self method;
		space.
	self uri printPathQueryFragmentOn: stream! !

!ZnRequestLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:34'!
printOn: stream
	super printOn: stream.
	(self method notNil and: [ self uri notNil ])
		ifFalse: [ ^ self ].
	stream nextPut: $(.
	self printMethodAndUriOn: stream.
	stream nextPut: $)! !

!ZnRequestLine methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 9/27/2010 19:25'!
readFrom: stream
	| line lineStream |
	line := (ZnLineReader on: stream) nextLine.
	lineStream := line readStream.
	self method: (lineStream upTo: Character space).
	self uri: (lineStream upTo: Character space).
	self version: (lineStream upToEnd)! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
uri
	^ uri! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/30/2013 17:19'!
uri: anURI
	uri := anURI asZnUrl.
	uri enforceKnownScheme! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2011 13:56'!
uriPathQueryFragment
	^ String streamContents: [ :stream | 
			self uri printPathQueryFragmentOn: stream ]! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 13:45'!
version
	^ version! !

!ZnRequestLine methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 14:04'!
version: string
	version := ZnConstants knownHTTPVersions 
		detect: [ :each | each = string ] 
		ifNone: [ (ZnUnknownHttpVersion version: string) signal ] ! !

!ZnRequestLine methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 10/6/2015 21:21'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	writeStream nextPutAll: self method; space.
	(ZnNetworkingUtils shouldProxyUrl: self uri)
		ifTrue: [ 
			self uri hasSecureScheme 
				ifTrue: [
					"See also ZnClient>>#setupTLSTo:"
					method = #CONNECT
						ifTrue: [ writeStream nextPutAll: uri authorityWithPort ]
						ifFalse: [ self uri printPathQueryFragmentOn: writeStream ] ]
				ifFalse: [ self uri printOn: writeStream ] ]
		ifFalse: [ self uri printPathQueryFragmentOn: writeStream ].
	writeStream space; nextPutAll: self version; nextPutAll: String crlf ! !

Object subclass: #ZnServer
	instanceVariableNames: 'options sessionManager'
	classVariableNames: 'AlwaysRestart ManagedServers'
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnServer commentStamp: '<historical>' prior: 0!
I am ZnServer, an abstract superclass of HTTP Servers.
I am a facade for controlling a default server implementation.

I delegate my public class protocol methods to #defaultServerClass.

  ZnServer startDefaultOn: 1701.
  ZnClient new get: 'http://localhost:1701'.

Subclasses can register with me to have start/stop sent to them on System startUp/shutDown.
The default server instance will be registered automatically when it is started.

Part of Zinc HTTP Components.!

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 11/30/2015 17:08'!
adoptAsDefault: server
	"Adopt server as the default instance that we manage.
	If there was a previous default, stop it.
	Delegate to the default server class."
	
	self defaultServerClass adoptAsDefault: server! !

!ZnServer class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/18/2012 13:49'!
alwaysRestart
	^ AlwaysRestart ifNil: [ AlwaysRestart := true ]! !

!ZnServer class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/18/2012 13:37'!
alwaysRestart: boolean
	"Set if managed servers should always restart on image save, as opposed to only when quiting."

	^ AlwaysRestart := boolean! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:27'!
default
	"Return the default instance that we manage.
	Delegate to the default server class."
	
	^ self defaultServerClass default! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:03'!
defaultOn: portNumber
	"Return the default instance on a given port,
	Keep a reference to it in a class instance variable.
	If there was no previous default instance, create a new one.
	If there was a previous default instance, reuse it:
	if it was running stop it, change the port if necessary.
	Delegate to the default server class."

	^ self defaultServerClass defaultOn: portNumber ! !

!ZnServer class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 3/4/2012 20:05'!
defaultServerClass
	"Return the default ZnServer subclass to use"
	
	^ ZnManagingMultiThreadedServer ! !

!ZnServer class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 9/18/2012 13:49'!
initialize
	ManagedServers := IdentitySet new.
	AlwaysRestart := true.
	Smalltalk addToStartUpList: self.
	Smalltalk addToShutDownList: self! !

!ZnServer class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/19/2011 12:17'!
managedServers
	^ ManagedServers ifNil: [ ManagedServers := IdentitySet new ]! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:28'!
on: portNumber
	"Instantiate a new listener on a given port,
	send #start to it to start listening.
	Delegate to the default server class."

	^ self defaultServerClass on: portNumber ! !

!ZnServer class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 6/19/2011 12:17'!
register: server
	"Arrange for server to be sent start/stop on system startUp/shutDown"
	
	self managedServers add: server! !

!ZnServer class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 9/18/2012 13:38'!
shutDown: quiting
	"Our system shutDown hook: stop all servers we manage"
	
	(quiting or: [ self alwaysRestart ])
		ifTrue: [
			self managedServers do: [ :each | each stop: false ] ]! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:04'!
startDefaultOn: portNumber
	"Start and return the default instance on a given port.
	Keep a reference to it in a class instance variable.
	If there was no previous default instance, create a new one.
	If there was a previous default instance, reuse it:
	if it was running stop and start it, effectively restarting it.
	Change the port if necessary.
	Starting the default server will register it automatically.
	Delegate to the default server class."

	^ self defaultServerClass startDefaultOn: portNumber ! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:30'!
startOn: portNumber
	"Instanciate and return a new listener on a given port and start listening.
	Delegate to the default server class."

	^ self defaultServerClass startOn: portNumber ! !

!ZnServer class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 9/29/2012 20:51'!
startUp: resuming
	"Our system startUp hook: start all servers we manage.
	We do this using deferred startup actions to allow normal error handling."

	(resuming or: [ self alwaysRestart ])
		ifTrue: [
			Smalltalk addDeferredStartupAction: [ 
				self managedServers do: [ :each | each start ] ] ]! !

!ZnServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:29'!
stopDefault
	"Stop the default instance, if any, and clear it.
	Return the stopped instance, if any.
	Delegate to the default server class."

	^ self defaultServerClass stopDefault ! !

!ZnServer class methodsFor: 'system startup' stamp: 'SvenVanCaekenberghe 8/18/2011 14:10'!
unregister: server
	"No longer send server start/stop on system startUp/shutDown"
	
	self managedServers remove: server ifAbsent: [ ]! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 11:17'!
authenticator
	"Return the optional authenticator,
	the object that will be sent #authenticateRequest:do:
	to authenticate or refuse the requests. 
	When authentication succeeds, the block should be executed,
	when authentication fails, a appropriate response should be returned.
	If there is no authenticator, all requests will pass"
	
	^ self optionAt: #authenticator ifAbsent: [ nil ]! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 11:17'!
authenticator: object
	"Set the object that will be sent #authenticateRequest:do:
	to authenticate or refuse the requests. Can be nil.
	When authentication succeeds, the block should be executed,
	when authentication fails, a appropriate response should be returned"

	self optionAt: #authenticator put: object! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 10:07'!
bindingAddress
	"Return the interface address we are (or will be) listening on.
	Nil means that we are (or will be) listening on all/any interfaces."
	
	^ self optionAt: #bindAddress ifAbsent: [ nil ] ! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 10:07'!
bindingAddress: address
	"Set the interface address we will be listening on.
	Specify nil to listen on all/any interfaces, the default.
	Address must be a 4 element ByteArray, like #[127 0 0 1].
	Cannot be changed after the server is already running."
	
	self optionAt: #bindAddress put: address! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 1/31/2013 11:44'!
debugMode
	"Return whether we are in debug mode, the default is false."
	
	^ self optionAt: #debugMode ifAbsent: [ false ] ! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 1/31/2013 11:44'!
debugMode: boolean
	"Set my debug mode, the default being false.
	In debug mode, Smalltalk Error during #handleRequest: will raise a Debugger.
	When not in debug mode, a Smalltalk Error during #handleRequest: will result in an HTTP Server Error response."
	
	^ self optionAt: #debugMode put: boolean! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 11:17'!
delegate
	"Return the optional delegate, 
	the object that will be sent #handleRequest: to handle a request and produce a response.
	The default delegate is ZnDefaultServerDelegate"
	
	^ self optionAt: #delegate ifAbsentPut: [ ZnDefaultServerDelegate new ]! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 11:17'!
delegate: object
	"Set the delegate to object. Can be nil.
	This will be sent #handleRequest: to handle a request and produce a response"
	
	self optionAt: #delegate put: object! !

!ZnServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/15/2010 21:22'!
isListening
	"Return true when I have a valid server socket listening at the correct port"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/15/2010 15:26'!
isRunning
	"Return true when I am running"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2013 13:15'!
localUrl
	"Return a ZnUrl to access me."
	
	^ ZnUrl new
		scheme: self scheme;
		host: NetNameResolver loopBackName;
		port: self port;
		yourself! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 1/31/2013 11:44'!
logServerErrorDetails
	"Return whether we log server error details including a short strack trace, the default is true."
	
	^ self optionAt: #debugMode ifAbsent: [ true ] ! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 1/31/2013 11:45'!
logServerErrorDetails: boolean
	"Set whether we log server error details including a short strack trace, the default is true."
	
	self optionAt: #debugMode put: boolean! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 12:47'!
maximumEntitySize
	"Return the maximum entity size in bytes that I will read from a stream before signalling ZnEntityTooLarge"
	
	^ self optionAt: #maximumEntitySize ifAbsent: [ ZnConstants defaultMaximumEntitySize ]! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 12:47'!
maximumEntitySize: integer
	"Set the maximum entity size in bytes that I will read from a stream before signalling ZnEntityTooLarge"
	
	^ self optionAt: #maximumEntitySize put: integer! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/5/2012 09:15'!
optionAt: key ifAbsent: block
	"Return my option/settings stored under key.
	Execute block if I have no such option/setting.
	This is a generic interface, see my options protocol for specific usages."

	options ifNil: [ ^ block value ].
	^ options at: key ifAbsent: block! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/5/2012 11:15'!
optionAt: key ifAbsentPut: block
	"Return my option/settings stored under key.
	If I have no such option/setting, store the result of evaluating block as new value and return it.
	This is a generic interface, see my options protocol for specific usages."

	^ options at: key ifAbsentPut: block! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/5/2012 09:15'!
optionAt: key put: value
	"Set my option/setting identified by key to be value.
	This is a generic interface, see my options protocol for specific usages."
	
	options ifNil: [ options := Dictionary new ].
	options at: key put: value! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 09:19'!
port
	"Return the integer port number we are (or will be) listening on"
	
	^ self optionAt: #port ifAbsent: [ 1701 ] ! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 09:18'!
port: integer
	"Set the port number we will be listening on.
	Cannot be changed after the server is already running."
	
	(self isRunning and: [ self port ~= integer ]) 
		ifTrue: [ self error: 'Stop me before changing my port' ].
	self optionAt: #port put: integer! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/15/2010 15:25'!
process
	"Return the process that is running my main listening loop.
	Will be nil when I am not running"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 11:17'!
reader
	"Return a block that when given a stream reads an entity from it."
	
	^ self optionAt: #reader ifAbsentPut: [ [ :stream | ZnRequest readFrom: stream ] ]! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 9/5/2012 11:17'!
reader: block
	"Customize how entities are read from a stream, see #reader"
	
	self optionAt: #reader put: block! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/19/2011 12:00'!
register
	"Ask for start/stop to be sent to me on System startUp/shutDown"
	
	self class register: self! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 12/31/2012 16:58'!
route
	"Return the route of the server.
	This is a short identification string to be appended at the end of server session ids, separated by a dot.
	Routes are used by load balancers and proxies to correctly implement session affiinity or stickyness.
	The default is nil, meaning that no route has to be appended."
	
	^ self optionAt: #route ifAbsent: [ nil ]! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 12/31/2012 16:57'!
route: object
	"Set the route of the server.
	This is a short identification string to be appended at the end of server session ids, separated by a dot.
	Routes are used by load balancers and proxies to correctly implement session affiinity or stickyness.
	The default is nil, meaning that no route has to be appended."

	self optionAt: #route put: object! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2013 13:15'!
scheme
	^ #http! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/15/2010 15:25'!
serverSocket
	"Return the server socket that I am using.
	Will be nil when I am not running"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 1/4/2013 13:40'!
serverUrl
	"Return the explicitely set external server URL, if any. Defaults to nil."
	
	^ self optionAt: #serverUrl ifAbsent: [ nil ]! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 1/4/2013 13:40'!
serverUrl: urlObject
	"Set the explicit external server URL to urlObject. Defaults to nil.
	urlObject should be a ZnUrl or a String that parses correctly to one.
	See also #url."
	
	^ self optionAt: #serverUrl put: urlObject asZnUrl! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 19:17'!
sessionFor: request
	"Bind an existing session to request or create a new session"
	
	^ self sessionManager sessionFor: request! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 19:17'!
sessionManager
	"Return my session manager"

	^ sessionManager ifNil: [ sessionManager := ZnServerSessionManager new ]! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/15/2010 15:25'!
start
	"Start me. I will start listening on my port for incoming HTTP connections.
	If I am running, I will first stop and thus effectively restart"
	
	self subclassResponsibility! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:19'!
stop
	"Stop me. I will stop listening on my port for incoming HTTP connections.
	Does nothing when I am not running"
	
	self stop: true ! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:19'!
stop: unregister
	"Stop me. I will stop listening on my port for incoming HTTP connections.
	If unregister is true, unregister me from the list of managed instances.
	Does nothing when I am not running"
	
	self subclassResponsibility ! !

!ZnServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/19/2011 12:00'!
unregister
	"Ask for start/stop to no longer be sent to me on System startUp/shutDown"

	self class unregister: self! !

!ZnServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 1/4/2013 13:52'!
url
	"Return the base external URL (a new ZnUrl instance) to access me.
	This defaults to #localUrl but can be set explicitely using the #serverUrl option.
	Missing elements from #serverUrl are merged in from #localUrl."

	^ self serverUrl 
		ifNil: [ self localUrl ] 
		ifNotNil: [ :serverUrl | serverUrl inContextOf: self localUrl ]! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/19/2013 20:10'!
useGzipCompressionAndChunking
	"Return whether we should try to use gzip content encoding and chunked transfer encoding, the default is false."
	
	^ self optionAt: #useGzipCompressionAndChunking ifAbsent: [ false ] ! !

!ZnServer methodsFor: 'options' stamp: 'SvenVanCaekenberghe 5/19/2013 20:11'!
useGzipCompressionAndChunking: boolean
	"Set whether we should try to use gzip content encoding and chunked transfer encoding, the default being false."
	
	self optionAt: #useGzipCompressionAndChunking put: boolean! !

ZnServer subclass: #ZnSingleThreadedServer
	instanceVariableNames: 'process serverSocket logLevel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

ZnSingleThreadedServer class
	instanceVariableNames: 'Default'!

!ZnSingleThreadedServer commentStamp: '<historical>' prior: 0!
I am ZnSingleThreadedServer.
I can be customized with a delegate (#handleRequest:) and an authenticator (#authenticateRequest:do:).

  ZnSingleThreadedServer startDefaultOn: 1701.
  ZnSingleThreadedServer default authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
  ZnClient new username: 'foo' password: 'secret'; get: 'http://localhost:1701'.

I use ZnDefaultServerDelegate when no other delegate is set.
I am single threaded, I run in a single process.
I close connections after each request/response cycle.

Part of Zinc HTTP Components.!

ZnSingleThreadedServer class
	instanceVariableNames: 'Default'!

ZnSingleThreadedServer subclass: #ZnMultiThreadedServer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnMultiThreadedServer commentStamp: '<historical>' prior: 0!
I am ZnMultiThreadedServer.
I inherit most features from ZnSingleThreadedServer.

  ZnMultiThreadedServer startDefaultOn: 1701.
  ZnMultiThreadedServer default authenticator: (ZnBasicAuthenticator username: 'foo' password: 'secret').
  ZnClient new username: 'foo' password: 'secret'; get: 'http://localhost:1701'.

I am multi threaded, I fork a new process for each incoming request.
I try to keep connections alive in each process.

Part of Zinc HTTP Components.!

ZnMultiThreadedServer subclass: #ZnManagingMultiThreadedServer
	instanceVariableNames: 'lock connections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnManagingMultiThreadedServer commentStamp: '<historical>' prior: 0!
ZnManagingMultiThreadedServer is a ZnMultiThreadedServer that manages the socket stream connections used by the worker threads that are spawned.

  ZnManagingMultiThreadedServer startDefaultOn: 1701.

I keep track of all the connections that I spawn so that I can close them when I am stopped.

Part of Zinc HTTP Components.!

!ZnManagingMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/13/2014 13:40'!
closeConnections
	self connections isEmpty ifTrue: [ ^ self ].
	self lock critical: [
		self connections do: [ :each | 
			self logConnectionClosed: each.
			[ each close ] 
				on: Exception 
				do: [ ] ].
		self connections removeAll ]! !

!ZnManagingMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/13/2014 13:40'!
closeSocketStream: socketStream
	super closeSocketStream: socketStream.
	self lock critical: [ 
		self connections remove: socketStream ifAbsent: [ ] ]! !

!ZnManagingMultiThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/5/2011 14:14'!
connections
	^ connections ifNil: [ connections := OrderedCollection new ]! !

!ZnManagingMultiThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/5/2011 14:14'!
lock
	^ lock ifNil: [ lock := Mutex new ]! !

!ZnManagingMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/5/2011 14:17'!
socketStreamOn: socket
	| socketStream |
	socketStream := super socketStreamOn: socket.
	self lock critical: [ 
		self connections add: socketStream ].
	^ socketStream! !

!ZnManagingMultiThreadedServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 10/10/2012 10:16'!
stop: unregister
	super stop: unregister.
	self closeConnections! !

!ZnMultiThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 11/7/2013 10:52'!
augmentResponse: response forRequest: request
	"Our handler has produced response for request, manipulate the response before writing it"

	super augmentResponse: response forRequest: request.
	response setKeepAliveFor: request.
	response setConnectionCloseFor: request! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/13/2014 15:19'!
closeSocketStream: socketStream
	[ socketStream close ] 
		on: Exception 
		do: [ ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/4/2011 09:46'!
exceptionSet: classNames
	^ classNames
		inject: ExceptionSet new
		into: [ :set :each |
			(Smalltalk includesKey: each)
				ifTrue: [ set add: (Smalltalk at: each); yourself ]
				ifFalse: [ set ] ]! !

!ZnMultiThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 6/13/2014 16:49'!
executeOneRequestResponseOn: stream
	"Execute one HTTP request / response cycle on stream in 3 steps
	#readRequest: #handleRequest: and #writeResponse:on:
	Return true when the outer loop we are in should stop."

	| request response timing|
	timing := ZnServerTransactionTiming new.
	(request := self readRequestSafely: stream timing: timing)
		ifNil: [ ^ true ]
		ifNotNil: [ 
			response := self handleRequest: request timing: timing.
			self augmentResponse: response forRequest: request.
			self writeResponseSafely: response on: stream timing: timing.
			self logServerTransactionRequest: request response: response timing: timing.
			response useConnection: stream ].
	^ request wantsConnectionClose or: [ response wantsConnectionClose ]! !

!ZnMultiThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 6/13/2014 14:30'!
executeRequestResponseLoopOn: stream
	"Execute the HTTP request / response loop on stream one cycle at a time
	until the other end indicates it wants to stop or times out"

	ZnCurrentServer 
		value: self 
		during: [ 
			[ self executeOneRequestResponseOn: stream ] whileFalse ]! !

!ZnMultiThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 12/10/2010 16:00'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	self initializeServerSocket.
	[ [ 
		serverSocket isValid 
			ifFalse: [
				"will trigger #ifCurtailed: block and destroy socket"
				^ self listenLoop ].
		self serveConnectionsOn: serverSocket ] repeat ]

		ifCurtailed: [ self releaseServerSocket ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/14/2013 10:43'!
readRequestBadExceptionSet
	"Return the set of exceptions which, when they occur while reading a request,
	are interpreted as equivalent to a request parse error or bad request."

	^ self
		exceptionSet:
			#(#ZnParseError #ZnCharacterEncodingError #ZnUnknownScheme #ZnPortNotANumber #ZnTooManyDictionaryEntries #ZnEntityTooLarge)! !

!ZnMultiThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 6/13/2014 21:55'!
readRequestSafely: stream timing: timing
	"Read request from stream, returning nil when the connection is closed or times out"

	^ [ 
		[ self readRequest: stream timing: timing ]
			on: self readRequestBadExceptionSet
			do: [ :exception | 
				self logServerReadError: exception.
				self writeResponseBad: exception on: stream timing: timing.
				nil ] ]
			on: self readRequestTerminationExceptionSet
			do: [ :exception | 
				self logServerReadError: exception.
				nil ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/27/2013 14:57'!
readRequestTerminationExceptionSet
	"Return the set of exceptions which, when they occur while reading a request,
	are interpreted as equivalent to a timeout or connection close."

	^ self
		exceptionSet:
			#(#ConnectionClosed #ConnectionTimedOut #PrimitiveFailed)! !

!ZnMultiThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 6/13/2014 15:50'!
serveConnectionsOn: listeningSocket
	"We wait up to acceptWaitTimeout seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and #executeRequestResponseLoopOn: on it"
		 
	| stream socket |
	socket := listeningSocket waitForAcceptFor: self acceptWaitTimeout.
	socket ifNil: [ ^ self noteAcceptWaitTimedOut ].
	stream := self socketStreamOn: socket.
	[ [ [ self executeRequestResponseLoopOn: stream ]
		ensure: [ self logConnectionClosed: stream. self closeSocketStream: stream ] ]
			ifCurtailed: [ socket destroy ] ]
				forkAt: Processor lowIOPriority
				named: self workerProcessName ! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/13/2010 16:33'!
workerProcessName
	^ String streamContents: [ :stream |
		stream nextPutAll: self class name; nextPutAll: ' HTTP worker' ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/13/2014 16:50'!
writeResponseBad: exception on: stream timing: timing
	"When we failed to parse a request, write a bad request response before closing."

	| response |
	response := ZnResponse new
		statusLine: ZnStatusLine badRequest;
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity textCRLF: 'Bad Request ' , exception printString);
		yourself.
	response setConnectionClose.
	self writeResponseSafely: response on: stream timing: timing! !

!ZnMultiThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 6/13/2014 21:56'!
writeResponseSafely: response on: stream timing: timing
	"Write response to stream, when the connection is closed or times out we ignore this"

	[ self writeResponse: response on: stream timing: timing ]
		on: self writeResponseTerminationExceptionSet
		do: [ :exception | self logServerWriteError: exception ]! !

!ZnMultiThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 5/14/2013 10:39'!
writeResponseTerminationExceptionSet
	"Return the set of exceptions which, when they occur while writing a response,
	are interpreted as equivalent to a timeout or connection close."

	^ self exceptionSet: #(#ConnectionClosed #ConnectionTimedOut #PrimitiveFailed)! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 11/30/2015 17:09'!
adoptAsDefault: server
	"Adopt server as the default instance that we manage.
	If there was a previous default, stop it.
	Note that this is a class instance variable."
	
	self stopDefault.
	Default := server! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:30'!
default
	"Return the default instance that we manage.
	Note that this is a class instance variable."
	
	^ Default! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:59'!
defaultOn: portNumber
	"Return the default instance on a given port,
	Keep a reference to it in a class instance variable.
	If there was no previous default instance, create a new one.
	If there was a previous default instance, reuse it:
	if it was running stop it, change the port if necessary."

	^ Default 
		ifNil: [
			Default := self on: portNumber ] 
		ifNotNil: [ 
			Default stop; port: portNumber; yourself ]! !

!ZnSingleThreadedServer class methodsFor: 'class initialization' stamp: 'SvenVanCaekenberghe 4/20/2011 12:12'!
initialize
	Default := nil! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:34'!
on: aNumber
	"Instantiate and return a new listener on a given port,
	send #start to it to start listening."

	^ self new 
		port: aNumber; 
		yourself! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 12:04'!
startDefaultOn: portNumber
	"Start and return the default instance on a given port.
	Keep a reference to it in a class instance variable.
	If there was no previous default instance, create a new one.
	If there was a previous default instance, reuse it:
	if it was running stop and start it, effectively restarting it.
	Change the port if necessary.
	Starting the default server will register it automatically."

	^ (self defaultOn: portNumber) 
		start; 
		yourself! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:37'!
startOn: portNumber
	"Instanciate and return a new listener on a given port and start listening."

	^ (self on: portNumber) 
		start;
		yourself! !

!ZnSingleThreadedServer class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/22/2011 11:59'!
stopDefault
	"Stop and unregister the Default instance, if any, and clear it.
	Return the stopped instance, if any."
	
	| server |
	(server := Default) ifNotNil: [ 
		Default stop.  
		Default := nil ].
	^ server! !

!ZnSingleThreadedServer methodsFor: 'constants' stamp: 'SvenVanCaekenberghe 12/31/2012 16:08'!
acceptWaitTimeout
	"How many seconds to wait for a server socket listening for an accept ?"
	
	^ 300! !

!ZnSingleThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 5/19/2013 21:12'!
augmentResponse: response forRequest: request
	"Our handler has produced response for request, manipulate the response before writing it"
	
	(self useGzipCompressionAndChunking and: [ response contentType isBinary not ])
		ifTrue: [ 
			(request acceptsEncodingGzip and: [ response hasContentEncoding not ]) 
				ifTrue: [ response setContentEncodingGzip ].
			response hasTransferEncoding
				ifFalse: [ response setTransferEncodingChunked ] ]! !

!ZnSingleThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 1/10/2013 15:12'!
authenticateAndDelegateRequest: request
	"Handle request and return a response.
	If we have a delegate, pass the responsability.
	If we have no delegate, we return a page not found.
	Make sure to pass via our authenticator."

	^ self
		authenticateRequest: request
		do: [ 
			self delegate
				ifNil: [ ZnResponse notFound: request uri ]
				ifNotNil: [ :delegate | delegate handleRequest: request ] ]! !

!ZnSingleThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 1/10/2013 15:13'!
authenticateRequest: request do: block
	"Validate request and execute block.
	When our authenticator is not nil, pass the responsability"

	^ self authenticator
		ifNil: [ block value ]
		ifNotNil: [ :authenticator | authenticator authenticateRequest: request do: block ]! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/13/2014 13:30'!
closeDelegate
	self delegate
		ifNotNil: [ :delegate | 
			(delegate respondsTo: #close)
				ifTrue: [ delegate close ] ]! !

!ZnSingleThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 6/13/2014 16:47'!
executeOneRequestResponseOn: stream
	"Execute one single HTTP request / response cycle on stream in 3 steps:
	#readRequest: #handleRequest: and #writeResponse:on:
	Indicate that we intend to close the connection afterwards"

	| request response timing|
	ZnCurrentServer
		value: self
		during: [ 
			timing := ZnServerTransactionTiming new.
			request := self readRequest: stream timing: timing.
			response := self handleRequest: request timing: timing.
			response setConnectionClose.
			self writeResponse: response on: stream timing: timing.
			self logServerTransactionRequest: request response: response timing: timing ]! !

!ZnSingleThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 6/13/2014 16:46'!
handleRequest: request timing: timing
	"Handle request and return a response.
	Do logging and timing. Set a dynamic variable referencing the server.
	If necessary do session management in the response."

	| response initialMilliseconds |
	initialMilliseconds := Time millisecondClockValue.
	ZnCurrentServerSession value: nil.
	response := self handleRequestProtected: request.
	request method = #HEAD
		ifTrue: [ response clearEntity ].
	timing handlerDuration: (self logRequest: request response: response handledStarted: initialMilliseconds).
	ZnCurrentServerSession value 
		ifNotNil: [ :session | self sessionManager setSession: session in: response ].
	^ response! !

!ZnSingleThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 7/2/2015 23:20'!
handleRequestProtected: request
	"Handle request and return a response.
	If a ZnRespond notification is signaled, return its response directly.
	If an Error is thrown, return a HTTP Server Error response."

	^ [ [ self authenticateAndDelegateRequest: request ]
			on: ZnRespond 
			do: [ :notification | notification response ] ]
		on: Error
		do: [ :exception | 
			self debugMode
				ifTrue: [ exception pass ]
				ifFalse: [
					self logServerHandlerError: exception.
					ZnResponse serverError: exception printString ] ]! !

!ZnSingleThreadedServer methodsFor: 'initialization' stamp: 'MarcusDenker 10/5/2014 18:05'!
initialize
	self loggingOn ! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/13/2014 11:14'!
initializeServerSocket
	"Set up a new server socket and listen on it"
	
	serverSocket := self bindingAddress
		ifNil: [ ZnNetworkingUtils serverSocketOn: self port ]
		ifNotNil: [ ZnNetworkingUtils serverSocketOn: self port interface: self bindingAddress ].
	self logServerSocketBound! !

!ZnSingleThreadedServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/15/2010 21:24'!
isListening
	"Return true when I have a valid server socket listening at the correct port"
	
	^ self serverSocket notNil
		and: [ self serverSocket isValid and: [ self serverSocket localPort = self port ] ]! !

!ZnSingleThreadedServer methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:57'!
isRunning
	"Return true when I am running"
	
	^ self process notNil and: [ self serverSocket notNil ]! !

!ZnSingleThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 12/10/2010 16:00'!
listenLoop
	"We create a listening Socket, then wait for a connection.
	After each connection we also check that the listening Socket is still valid 
	- if not we just make a recursive call to this method to start over."

	self initializeServerSocket.
	[ [ 
		serverSocket isValid
			 ifFalse: [
				"will trigger #ifCurtailed: block and destroy socket"
				^ self listenLoop ].
		self serveConnectionOn: serverSocket ] repeat ]

		ifCurtailed: [ self releaseServerSocket ]! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 2/11/2015 15:28'!
logConnectionAccepted: socket
	logLevel < 3 ifTrue: [ ^ nil ].
	^ (self newLogEvent: ZnConnectionAcceptedEvent)
		address: ([ socket remoteAddress ] on: Error do: [ nil ]);
		emit! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 2/11/2015 15:29'!
logConnectionClosed: socketStream
	logLevel < 3 ifTrue: [ ^ nil ].
	socketStream socket ifNil: [ ^ nil ].
	^ (self newLogEvent: ZnServerConnectionClosedEvent)
		address: ([ socketStream socket remoteAddress] on: Error do: [ nil ]);
		emit! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 6/13/2014 10:19'!
logLevel: integer
	"Set the log level to integer.
	0 - no logging
	1 - simplified transaction logging
	2 - detailed transaction logging
	3 - log everything"
	
	logLevel := integer! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 16:37'!
logRequest: request response: response handledStarted: initialMilliseconds
	^ logLevel < 3
		ifTrue: [ Time millisecondsSince: initialMilliseconds ]
		ifFalse: [ 
			(self newLogEvent: ZnRequestResponseHandledEvent)
				request: request;
				response: response;
				duration: (Time millisecondsSince: initialMilliseconds);
				emit;
				duration ]! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 16:35'!
logRequestRead: request started: initialMilliseconds
	^ logLevel < 3
		ifTrue: [ Time millisecondsSince: initialMilliseconds ]
		ifFalse: [ 
			(self newLogEvent: ZnRequestReadEvent)
				request: request;
				duration: (Time millisecondsSince: initialMilliseconds);
				emit;
				duration ]! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 16:40'!
logResponseWritten: response started: initialMilliseconds
	^ logLevel < 3
		ifTrue: [ Time millisecondsSince: initialMilliseconds ]
		ifFalse: [ 
			(self newLogEvent: ZnResponseWrittenEvent)
				response: response;
				duration: (Time millisecondsSince: initialMilliseconds);
				emit;
				duration ]! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/16/2014 11:39'!
logServerGeneric: subject
	^ (self newLogEvent: ZnServerGenericLogEvent)
		subject: subject;
		emit! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 21:54'!
logServerHandlerError: exception
	^ (self newLogEvent: ZnServerHandlerErrorEvent)
		exception: exception;
		emit! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 21:59'!
logServerReadError: exception
	logLevel < 3 ifTrue: [ ^ nil ].
	^ (self newLogEvent: ZnServerReadErrorEvent)
		exception: exception;
		emit! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 2/11/2015 15:31'!
logServerSocketBound
	logLevel < 1 ifTrue: [ ^ nil ].
	^ (self newLogEvent: ZnServerSocketBoundEvent)
		address: ([ serverSocket address ] on: Error do: [ nil ]);
		port: ([ serverSocket port ] on: Error do: [ nil ]);
		emit! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 16:17'!
logServerSocketReleasedAddress: address port: port
	logLevel < 1 ifTrue: [ ^ nil ].
	^ (self newLogEvent: ZnServerSocketReleasedEvent)
		address: address;
		port: port;
		emit! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 16:54'!
logServerTransactionRequest: request response: response timing: timing
	| logEventClass |
	logLevel < 1 ifTrue: [ ^ nil ].
	logEventClass := logLevel = 1
		ifTrue: [ ZnSimplifiedServerTransactionEvent ]
		ifFalse: [ ZnServerTransactionEvent ].
	^ (self newLogEvent: logEventClass)
		request: request;
		response: response;
		timing: timing;
		emit! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 21:59'!
logServerWriteError: exception
	logLevel < 3 ifTrue: [ ^ nil ].
	^ (self newLogEvent: ZnServerWriteErrorEvent)
		exception: exception;
		emit! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 16:17'!
logStarted
	logLevel < 1 ifTrue: [ ^ nil ].
	^ (self newLogEvent: ZnServerStartedEvent)
		description: self serverProcessName;
		emit! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 16:17'!
logStopped
	logLevel < 1 ifTrue: [ ^ nil ].
	^ (self newLogEvent: ZnServerStoppedEvent)
		description: self serverProcessName;
		emit! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 6/16/2014 14:59'!
logToTranscript
	"Setup logging to the Transcript.
	If logging was completely off, turn it on"

	logLevel isZero ifTrue: [ self loggingOn ].
	ZnLogEvent logToTranscript ! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 6/13/2014 10:20'!
loggingOff
	"Turn logging completely off - do not generate log events."
	
	self logLevel: 0! !

!ZnSingleThreadedServer methodsFor: 'logging' stamp: 'SvenVanCaekenberghe 6/13/2014 10:20'!
loggingOn
	"Turn full logging on - generate all log events"
	
	self logLevel: 3! !

!ZnSingleThreadedServer methodsFor: 'private logging' stamp: 'SvenVanCaekenberghe 6/13/2014 10:53'!
newLogEvent: logEventClass
	^ logEventClass new
		serverId: self route;
		yourself! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/13/2014 13:31'!
noteAcceptWaitTimedOut
	"The accept wait on a server socket timed out (see #acceptWaitTimeout).
	This can be used as an opportunity to do some periodic tasks."
	
	self periodicTasks! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 7/1/2011 14:07'!
onRequestRespond: block
	"Convenience method to quickly set up a delegate.
	On any incoming request, block will be executed with the request 
	as argument to produce a valid response."
	
	self delegate: (ZnValueDelegate with: block)  ! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 11/11/2012 20:07'!
periodicTasks
	"Every #acceptWaitTimeout seconds this method is called.
	Note that during this time the server is not actively listening for connections."
	
	sessionManager ifNotNil: [ sessionManager cleanupInvalidSessions ]! !

!ZnSingleThreadedServer methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 1/10/2013 15:17'!
printOn: stream
	super printOn: stream.
	stream nextPut: $(.
	stream nextPutAll: (self isRunning ifTrue: [ 'running' ] ifFalse: [ 'stopped' ]).
	self port ifNotNil: [ :port | stream space; print: port ].
	self bindingAddress ifNotNil: [ :bindingAddress | stream space; print: bindingAddress ].
	stream nextPut: $)! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:54'!
process
	"Return the process that is running my main listening loop.
	Will be nil when I am not running"
	
	^ process! !

!ZnSingleThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 6/13/2014 16:44'!
readRequest: stream timing: timing
	"Read a request from stream.
	We add a virtual header containing the remote IP address of the client"

	| request initialMilliseconds |
	initialMilliseconds := Time millisecondClockValue.
	request := self withMaximumEntitySizeDo: [ self reader value: stream ].
	request headers 
		at: ZnConstants remoteAddressHeader
		put: (ZnNetworkingUtils ipAddressToString: stream socket remoteAddress).
	timing requestDuration:  (self logRequestRead: request started: initialMilliseconds).
	^ request! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 2/11/2015 15:34'!
releaseServerSocket
	"Release our server socket"
	
	| address port |
	serverSocket ifNil: [ ^ self ].
	address := [ serverSocket address ] on: Error do: [ nil ].
	port := [ serverSocket port ]on: Error do: [ nil ].
	(Delay forMilliseconds: 10) wait.
	serverSocket destroy.
	serverSocket := nil.
	self logServerSocketReleasedAddress: address port: port ! !

!ZnSingleThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 6/13/2014 13:42'!
serveConnectionOn: listeningSocket
	"We wait up to acceptWaitTimeout seconds for an incoming connection.
	If we get one we wrap it in a SocketStream and #executeOneRequestResponseOn: on it.
	We do not fork a worker thread/process but stay in the current one for just one request/response cycle."

	| stream socket |
	socket := listeningSocket waitForAcceptFor: self acceptWaitTimeout.
	socket ifNil: [ ^ self noteAcceptWaitTimedOut ].
	stream := self socketStreamOn: socket.
	[ 
	[ self executeOneRequestResponseOn: stream ]
		ensure: [ 
			self logConnectionClosed: stream.
			stream close ] ]
		ifCurtailed: [ 
			socket destroy ]! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/13/2010 16:42'!
serverProcessName
	^ String streamContents: [ :stream |
		stream nextPutAll: self class name; nextPutAll: ' HTTP port '; print: self port ]! !

!ZnSingleThreadedServer methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/10/2010 13:54'!
serverSocket
	"Return the server socket that I am using.
	Will be nil when I am not running"
	
	^ serverSocket! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/13/2014 11:44'!
socketStreamOn: socket
	self logConnectionAccepted: socket.
	^ ZnNetworkingUtils socketStreamOn: socket! !

!ZnSingleThreadedServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/13/2014 11:00'!
start
	"Start me. I will start listening on my port for incoming HTTP connections.
	If I am running, I will first stop and thus effectively restart"
	
	self stop.
	self class default = self ifTrue: [ self register ].
	process := [ [ self listenLoop ] repeat ]
		forkAt: Processor highIOPriority 
		named: self serverProcessName.
	self logStarted.
! !

!ZnSingleThreadedServer methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/13/2014 11:00'!
stop: unregister
	"Stop me. I will stop listening on my port for incoming HTTP connections.
	If unregister is true, unregister me from the list of managed instances.
	Does nothing when I am not running"
	
	self isRunning ifFalse: [ ^ self ].
	process terminate. 
	process := nil.
	unregister ifTrue: [ self unregister ].
	self closeDelegate.
	self logStopped! !

!ZnSingleThreadedServer methodsFor: 'private' stamp: 'SvenVanCaekenberghe 9/5/2012 12:49'!
withMaximumEntitySizeDo: block
	^ ZnMaximumEntitySize value
		ifNil: [
			ZnMaximumEntitySize 
				value: self maximumEntitySize  
				during: [ ^ block value ] ]
		ifNotNil: [ 
			block value ]! !

!ZnSingleThreadedServer methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 6/13/2014 16:45'!
writeResponse: response on: stream timing: timing
	"Write response to stream and flush the stream"
	
	| initialMilliseconds |
	initialMilliseconds := Time millisecondClockValue.
	response writeOn: stream.
	stream flush.
	timing responseDuration: (self logResponseWritten: response started: initialMilliseconds)  ! !

Object subclass: #ZnServerSession
	instanceVariableNames: 'id attributes created modified'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnServerSession commentStamp: '<historical>' prior: 0!
I am ZnServerSession.
I represent an HTTP session in the context of a ZnServer.

To get a reference to the current session, send #session to a ZnMessage (request or response).

Part of Zinc HTTP Components.!

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/11/2012 20:06'!
age
	^ DateAndTime now - created! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 17:28'!
attributeAt: key
	^ attributes at: key! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 17:28'!
attributeAt: key ifAbsent: block
	^ attributes at: key ifAbsent: block! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/7/2012 12:21'!
attributeAt: key ifAbsentPut: block
	^ attributes at: key ifAbsentPut: block! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 17:29'!
attributeAt: key put: value
	^ attributes at: key put: value! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2012 23:53'!
attributeKeys
	^ attributes keys! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/11/2012 08:01'!
created
	^ created! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 17:27'!
id
	^ id! !

!ZnServerSession methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 11/10/2012 17:27'!
id: anObject
	id := anObject! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/11/2012 20:08'!
inactivity
	^ DateAndTime now - modified ! !

!ZnServerSession methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 11/11/2012 08:01'!
initialize
	super initialize.
	attributes := SmallDictionary new.
	created := modified := DateAndTime now! !

!ZnServerSession methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 11/11/2012 20:08'!
isValid
	^ self inactivity < 1 hour! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/11/2012 08:01'!
modified
	^ modified! !

!ZnServerSession methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 11/10/2012 17:30'!
printOn: stream
	super printOn: stream.
	id
		ifNotNil: [ 
			stream
				nextPut: $(;
				print: id;
				nextPut: $) ]! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/20/2012 23:35'!
removeAttribute: key
	attributes removeKey: key ifAbsent: [ ] ! !

!ZnServerSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/11/2012 08:01'!
touch
	modified := DateAndTime now! !

Object subclass: #ZnServerSessionManager
	instanceVariableNames: 'sessions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnServerSessionManager commentStamp: '<historical>' prior: 0!
I am ZnServerSessionManager.
I manage a collection of ZnServerSession on behalf of a ZnServer.

Session ids are stored in cookies.

Part of Zinc HTTP Components.!

!ZnServerSessionManager methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/31/2012 16:45'!
cleanupInvalidSessions
	"Remove all expired sessions"
	
	sessions keysAndValuesRemove: [ :key :value | value isValid not ]! !

!ZnServerSessionManager methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 19:33'!
cookieName
	^ 'session-id'! !

!ZnServerSessionManager methodsFor: 'initialization' stamp: 'SvenVanCaekenberghe 11/10/2012 19:06'!
initialize
	super initialize.
	sessions := Dictionary new! !

!ZnServerSessionManager methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/31/2012 16:34'!
newSessionForRequest: request
	| id session |
	id := self newSessionId.
	session := ZnServerSession new.
	session id: id.
	sessions at: id put: session.
	^ session! !

!ZnServerSessionManager methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/31/2012 16:46'!
newSessionId
	"Create a new session id.
	If the server has a route, append it at the end with a dot as separator."
	
	^ ZnCurrentServer value route
		ifNil: [ UUID new asString36 ]
		ifNotNil: [ :route | 
			String
				streamContents: [ :stream | 
					stream
						<< UUID new asString36;
						nextPut: $.;
						<< route ] ]! !

!ZnServerSessionManager methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/31/2012 16:45'!
removeSessionWithId: id
	"Remove the session with id.
	If there is no such session, do nothing"
	
	^ sessions removeKey: id ifAbsent: [ nil ]! !

!ZnServerSessionManager methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/25/2012 21:26'!
sessionFor: request
	"Bind an existing session to request or create a new session"

	| id |
	id := self sessionIdFromRequest: request.
	sessions
		at: id
		ifPresent: [ :session | 
			session isValid
				ifTrue: [ ^ session touch; yourself ]
				ifFalse: [ sessions removeKey: id ] ].
	^ self newSessionForRequest: request! !

!ZnServerSessionManager methodsFor: 'private' stamp: 'SvenVanCaekenberghe 12/31/2012 16:40'!
sessionIdFromRequest: request
	| cookie |
	cookie := request cookies 
		detect: [ :each | each name = self cookieName ] 
		ifNone: [ ^ nil ].
	^ cookie value! !

!ZnServerSessionManager methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 11/10/2012 22:57'!
setSession: session in: response
	"Bind session in response"

	| cookie |
	session ifNil: [ ^ self ].
	(cookie := ZnCookie name: self cookieName value: session id)
		path: '/'.
	response addCookie: cookie! !

Object subclass: #ZnServerTransactionTiming
	instanceVariableNames: 'requestDuration handlerDuration responseDuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Logging'!

!ZnServerTransactionTiming commentStamp: 'SvenVanCaekenberghe 6/13/2014 16:00' prior: 0!
I am ZnServerTransactionTiming.

I hold timing information related to the handling of an HTTP transaction by a server.
!

!ZnServerTransactionTiming methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:00'!
handlerDuration
	^ handlerDuration! !

!ZnServerTransactionTiming methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:00'!
handlerDuration: anObject
	handlerDuration := anObject! !

!ZnServerTransactionTiming methodsFor: 'initialize' stamp: 'SvenVanCaekenberghe 6/13/2014 16:01'!
initialize
	super initialize.
	requestDuration := handlerDuration := responseDuration := 0! !

!ZnServerTransactionTiming methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:00'!
requestDuration
	^ requestDuration! !

!ZnServerTransactionTiming methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:00'!
requestDuration: anObject
	requestDuration := anObject! !

!ZnServerTransactionTiming methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:00'!
responseDuration
	^ responseDuration! !

!ZnServerTransactionTiming methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 16:00'!
responseDuration: anObject
	responseDuration := anObject! !

!ZnServerTransactionTiming methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 6/13/2014 21:49'!
totalDuration
	"requestDuration is unreliable because it contains idle wait time"
	
	^ handlerDuration + responseDuration ! !

Object subclass: #ZnStatusLine
	instanceVariableNames: 'version code reason'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Core'!

!ZnStatusLine commentStamp: '<historical>' prior: 0!
I am an HTTP Status Line.
I consist of an HTTP protocol version, an integer HTTP status code and an HTTP reason string.
I can be used for generating and parsing.

Part of Zinc HTTP Components.!

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:00'!
badRequest
	^ self code: 400! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:48'!
code: code
	^ self new
		code: code;
		version: ZnConstants defaultHTTPVersion;
		yourself! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:05'!
created
	^ self code: 201! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:10'!
forbidden
	^ self code: 403! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:09'!
found
	^ self code: 302! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:02'!
internalServerError
	^ self code: 500! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:09'!
methodNotAllowed
	^ self code: 405! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:09'!
movedPermanently
	^ self code: 301! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/3/2015 15:03'!
noContent
	^ self code: 204! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 08:59'!
notFound
	^ self code: 404! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:08'!
notImplemented
	^ self code: 501! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:08'!
notModified
	^ self code: 304! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 08:58'!
ok
	^ self code: 200! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 9/6/2010 15:47'!
readFrom: stream
	^ self new
		readFrom: stream;
		yourself! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:07'!
redirect
	^ self code: 302! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:07'!
seeOther
	^ self code: 303! !

!ZnStatusLine class methodsFor: 'instance creation' stamp: 'NickAger 5/10/2011 09:02'!
unauthorized
	^ self code: 401! !

!ZnStatusLine methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:30'!
= other
	self class = other class ifFalse: [ ^ false ].
	^ self code = other code 
		and: [ self version = other version
			and: [ self reason = other reason ] ]! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:49'!
code
	^ code! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'DamienPollet 8/4/2011 14:04'!
code: integer
	(ZnConstants httpStatusCodes includesKey: integer) 
		ifFalse: [ 
			(ZnUnknownHttpStatusCode code: integer) signal ].
	code := integer! !

!ZnStatusLine methodsFor: 'comparing' stamp: 'SvenVanCaekenberghe 5/22/2013 10:33'!
hash
	^ (self code hash bitXor: self version hash) bitXor: self reason hash! !

!ZnStatusLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 10/12/2015 11:35'!
printCodeAndReasonOn: stream
	self code
		ifNotNil: [
			stream print: self code; space; nextPutAll: self reason ]! !

!ZnStatusLine methodsFor: 'printing' stamp: 'SvenVanCaekenberghe 10/12/2015 11:36'!
printOn: stream
	super printOn: stream.
	self code ifNil: [ ^ self ].
	stream nextPut: $(.
	self printCodeAndReasonOn: stream.
	stream nextPut: $)! !

!ZnStatusLine methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 10/12/2015 11:36'!
readFrom: stream
	| line lineStream httpCode |
	line := (ZnLineReader on: stream) nextLine.
	lineStream := line readStream.
	self version: (lineStream upTo: Character space).
	httpCode := lineStream upTo: Character space.
	httpCode := Integer readFrom: httpCode ifFail: [ (ZnUnknownHttpStatusCode code: httpCode) signal ].
	self code: httpCode.
	self reason: (lineStream upToEnd)
! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/12/2015 11:42'!
reason
	reason ifNil: [ 
		reason := ZnConstants httpStatusCodes 
			at: self code 
			ifAbsent: [ 'Unknown Status Code' ] ].
	^ reason! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 16:29'!
reason: string
	reason := string! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 9/6/2010 15:46'!
version
	^ version! !

!ZnStatusLine methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 10/12/2015 11:38'!
version: string
	version := ZnConstants knownHTTPVersions 
		detect: [ :each | each = string ] 
		ifNone: [ (ZnUnknownHttpVersion version: string) signal ] ! !

!ZnStatusLine methodsFor: 'writing' stamp: 'SvenVanCaekenberghe 12/3/2010 20:40'!
writeOn: stream
	| writeStream |
	writeStream := ZnBivalentWriteStream on: stream.
	writeStream nextPutAll: self version; space.
	self code printOn: writeStream.
	writeStream space; nextPutAll: self reason; nextPutAll: String crlf ! !

Object subclass: #ZnTestRunnerDelegate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnTestRunnerDelegate commentStamp: '<historical>' prior: 0!
I am ZnTestRunnerDelegate, a web service interface to run unit tests.

	ZnTestRunnerDelegate startInServerOn: 1701.
	
	ZnEasy get: 'http://localhost:1701/sunit/ZnUtilsTests'.
	ZnEasy get: 'http://localhost:1701/sunit/ZnUtilsTests/testBase64'.

Web Service API:

	GET /sunit/MyTestCase
	GET /sunit/MyTestCase/testOne

Part of Zinc HTTP Components!

!ZnTestRunnerDelegate class methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/27/2013 17:16'!
startInServerOn: port
	"Start a new server bound to port on the local network running a test runner web service"

	"self startInServerOn: 1701"
	
	 ^ (ZnServer on: port)
			bindingAddress: NetNameResolver loopBackAddress;
			delegate: self new;
			start;
			yourself! !

!ZnTestRunnerDelegate methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 5/28/2013 10:29'!
handleGetRequest: request
	| testCaseClass selector result |
	testCaseClass := Smalltalk globals
		at: request uri pathSegments second asSymbol
		ifAbsent: [ ^ self notFoundResponse: 'No such class, ', request uri pathSegments second ].
	(testCaseClass inheritsFrom: TestCase)
		ifFalse: [ ^ self notFoundResponse: testCaseClass printString, ' is not a TestCase subclass' ]. 
	selector := request uri pathSegments size = 3
		ifTrue: [ request uri pathSegments third asSymbol ]
		ifFalse: [ nil ].
	(selector isNil or: [ testCaseClass canUnderstand: selector ])
		ifFalse: [ ^ self notFoundResponse: selector, ' is not a method of ', testCaseClass printString ].
	result := selector 
		ifNil: [ testCaseClass suite run ] 
		ifNotNil: [ testCaseClass run: selector ].
	^ result runCount = result passedCount
		ifTrue: [ ZnResponse ok: (ZnEntity textCRLF: result printString) ]
		ifFalse: [ ZnResponse serverError: result printString ]! !

!ZnTestRunnerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/27/2013 17:20'!
handleRequest: request
	(request method = #GET 
		and: [ 
			request uri firstPathSegment = #sunit 
				and: [ 
					request uri pathSegments size > 1 ] ])
		ifTrue: [ 
			^ self handleGetRequest: request ].
	^ ZnResponse notFound: request uri! !

!ZnTestRunnerDelegate methodsFor: 'request handling' stamp: 'SvenVanCaekenberghe 5/28/2013 10:28'!
notFoundResponse: message
	^ ZnResponse new
		statusLine: ZnStatusLine notFound;
		headers: ZnHeaders defaultResponseHeaders;
		entity: (ZnEntity textCRLF: 'Not Found: ', message);
		yourself.! !

!ZnTestRunnerDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 5/27/2013 17:16'!
value: request
	"I implement the generic #value: message as equivalent to #handleRequest:"
	
	^ self handleRequest: request! !

Object subclass: #ZnUserAgentSession
	instanceVariableNames: 'cookiejar credentials currentRealm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUserAgentSession commentStamp: 'SvenVanCaekenberghe 12/6/2010 13:04' prior: 0!
I represent a session store for an HTTP client instance of ZnUserAgent. 

I hold the cookie jar for the session and any authentication credentials used by the client.

Part of Zinc HTTP Components.!

!ZnUserAgentSession class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
withCookieJar: aCookieJar
	^ self new cookiejar: aCookieJar; yourself! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 14:31'!
addCredential: aCredential
	self credentials at: aCredential domain, '@', aCredential realm put: aCredential ! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/30/2011 22:00'!
cookieJar
	^ cookiejar ifNil: [ cookiejar := ZnCookieJar new ]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:50'!
cookiejar: anObject
	cookiejar := anObject! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:24'!
credentialAt: aRealm forUrl: urlObject
	^ self credentials at: urlObject authority, '@', aRealm ifAbsent: [ nil ]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:14'!
credentials
	^ credentials ifNil: [ credentials := Dictionary new ]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 10:50'!
credentials: anObject
	credentials := anObject! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 8/23/2012 14:34'!
credentialsForUrl: urlObject
	^ self credentials values select: [ :cred | urlObject authority includesSubstring: (cred domain) ]! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:24'!
currentRealm
	^ currentRealm! !

!ZnUserAgentSession methodsFor: 'accessing' stamp: 'MattKennedy 9/28/2010 17:24'!
currentRealm: anObject
	currentRealm := anObject! !

!ZnUserAgentSession methodsFor: 'operations' stamp: 'SvenVanCaekenberghe 2/29/2012 17:40'!
resetCookies
	cookiejar ifNotNil: [ cookiejar removeAll ]! !

Object subclass: #ZnUtils
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Support'!

!ZnUtils commentStamp: 'SvenVanCaekenberghe 1/18/2011 11:03' prior: 0!
I am ZnUtils holding various utilities related to HTTP handling.

Part of Zinc HTTP Components.!

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 5/18/2013 18:58'!
bufferedWriteStreamOn: stream
	^ (ZnBufferedWriteStream on: stream)
		sizeBuffer: self streamingBufferSize;
		yourself! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/10/2010 20:14'!
capitalizeString: string
	| capitalExpected |
	capitalExpected := true.
	^ String new: string size streamContents: [ :stream |
		string do: [ :each |
			each isAlphaNumeric
				ifTrue: [
					capitalExpected 
						ifTrue: [ 
							stream nextPut: each asUppercase.
							capitalExpected := false. ]
						ifFalse: [ 
							stream nextPut: each asLowercase ] ] 
		 		ifFalse: [ 
					stream nextPut: each.
					capitalExpected := true ] ] ]! !

!ZnUtils class methodsFor: 'accessing' stamp: 'SvenVanCaekenberghe 12/14/2010 13:10'!
currentProcessID
	"Return a integer of at most 6 digits that can be used as a description for the current active process.
	This is a reduced hash that is not necessarily unique (but uniqueness is very probably)"
	
	^ Processor activeProcess hash \\ 999999! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 2/24/2014 17:08'!
decodeBase64: string
	| bytes |
	bytes := ZnBase64Encoder new decode: string.
	^ ZnNullEncoder new decodeBytes: bytes! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 2/24/2014 17:07'!
encodeBase64: string
	| bytes |
	bytes := ZnNullEncoder new encodeString: string.
	^ ZnBase64Encoder new encode: bytes! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 9/16/2010 15:52'!
httpDate
	^ self httpDate: DateAndTime now! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 3/2/2014 19:02'!
httpDate: object
	"Answers a string formatting object as time stamp for HTTP1.1 (RFC 1123). 
	Eg. Sun, 06 Nov 1994 08:49:37 GMT"

	| utc |
	utc := object asDateAndTime asUTC.
	^ String streamContents: [ :stream |
		stream nextPutAll: utc dayOfWeekAbbreviation; nextPut: $,; space.
		stream nextPutAll: (utc dayOfMonth printStringLength: 2 padded: true).
		stream space; nextPutAll: utc monthAbbreviation; space.
		utc year printOn: stream.
		stream space.
		utc printHMSOn: stream.
		stream nextPutAll: ' GMT' ]! !

!ZnUtils class methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 8/18/2011 14:15'!
isCapitalizedString: string
	| capitalExpected |
	capitalExpected := true.
	string do: [ :each |
		each isAlphaNumeric
			ifTrue: [
				((capitalExpected and: [ each isUppercase ])
					or: [ capitalExpected not and: [ each isLowercase ] ])
					ifTrue: [ capitalExpected := false ]
					ifFalse: [ ^ false ] ] 
		 	ifFalse: [ capitalExpected := true ] ].
	^ true



! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 5/28/2013 13:04'!
nextPutAll: collection on: stream
	| totalWritten toWrite total |
	total := collection size.
	"Do not enter the loop for small sizes"
	total < self streamingBufferSize
		ifTrue: [ ^ stream nextPutAll: collection ].
	totalWritten := 0.
	[ totalWritten < total ] whileTrue: [ 
		toWrite := self streamingBufferSize min: (total - totalWritten).
		stream next: toWrite putAll: collection startingAt: totalWritten + 1.
		totalWritten := totalWritten + toWrite.
		self signalProgress: totalWritten total: total ]! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 11/10/2013 11:28'!
parseHttpDate: string
	"self parseHttpDate: 'Tue, 13 Sep 2011 08:04:49 GMT'."
	"self parseHttpDate: 'Tue, 13-Sep-2011 08:04:51 GMT'."
	"self parseHttpDate: 'Tue Jan 01 00:00:01 2036 GMT'."
	
	| tokens day month year hour minute second months map yearToken |
	tokens := (string findTokens: #( $ $- $: $, )) allButFirst.
	tokens last = 'GMT' ifTrue: [ tokens := tokens allButLast ].
	months := #(jan feb mar apr may jun jul aug sep oct nov dec).
	(Integer readFrom: tokens first ifFail: [ 0 ]) isZero
		ifTrue: [ map := #(2 1 6 3 4 5) ]
		ifFalse: [ map := #(1 2 3 4 5 6) ].
	day := Integer readFrom: (tokens at: map first) ifFail: [ 0 ].
	month := months indexOf: (((tokens at: map second) asLowercase) copyFrom: 1 to: 3) ifAbsent: [ 0 ].
	yearToken := tokens at: map third.
	year := yearToken isAllDigits
		ifTrue: [
			year := yearToken asNumber.
			yearToken size = 2
				ifTrue: [
					year < 50
						ifTrue: [ year + 2000 ]
						ifFalse: [ year + 1900 ] ]
				ifFalse: [ year ] ]
		ifFalse: [ 0 ].
	hour := Integer readFrom: (tokens at: map fourth) ifFail: [ 0 ].
	minute := Integer readFrom: (tokens at: map fifth) ifFail: [ 0 ].
	second := Integer readFrom: (tokens at: map sixth) ifFail: [ 0 ].
	^ DateAndTime year: year month: month day: day hour: hour minute: minute second: second offset: Duration zero! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 6/11/2013 14:04'!
readUpToEnd: inputStream limit: limit
	| species bufferSize buffer totalRead outputStream |
	bufferSize := limit ifNil: [ self streamingBufferSize ] ifNotNil: [ self streamingBufferSize min: limit ].
	species := inputStream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ].
	buffer := species new: bufferSize.
	totalRead := 0.
	outputStream := nil.
	[ inputStream atEnd ] whileFalse: [ | readCount |
		readCount := inputStream readInto: buffer startingAt: 1 count: bufferSize.
		totalRead := totalRead + readCount.
		(limit notNil and: [ totalRead > limit ])
			ifTrue: [ ZnEntityTooLarge signal ].
		outputStream ifNil: [ 
			inputStream atEnd
				ifTrue: [ ^ buffer copyFrom: 1 to: readCount ] 
				ifFalse: [ outputStream := (species new: bufferSize) writeStream ] ].
		outputStream next: readCount putAll: buffer startingAt: 1.
		self signalProgress: totalRead total: limit ].
	^ outputStream ifNil: [ species new ] ifNotNil: [ outputStream contents ]! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/28/2013 18:00'!
signalProgress: amount total: total
	ZnSignalProgress enabled
		ifTrue: [ 
			total 
				ifNil: [ 
					HTTPProgress new 
						signal: ('Transferred {1} bytes ...' format: { amount asBytesDescription }) ]
				ifNotNil: [ 
					HTTPProgress new 
						total: total; 
						amount: amount;
						signal: 'Transferring...' ] ]! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/17/2012 14:47'!
streamFrom: inputStream to: outputStream
	| buffer totalRead |
	buffer := (inputStream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: self streamingBufferSize.
	totalRead := 0.
	[ inputStream atEnd ]
		whileFalse: [ | readCount |
			readCount := inputStream readInto: buffer startingAt: 1 count: buffer size.
			readCount > 0
				ifTrue: [
					totalRead := totalRead + readCount.
					outputStream next: readCount putAll: buffer startingAt: 1 ].
			inputStream atEnd
				ifFalse: [
					self signalProgress: totalRead total: nil. 
					outputStream flush ] ]! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/17/2012 14:47'!
streamFrom: inputStream to: outputStream size: totalSize
	| bufferSize buffer leftToRead |
	bufferSize := self streamingBufferSize min: totalSize.
	buffer := (inputStream isBinary ifTrue: [ ByteArray ] ifFalse: [ String ]) new: bufferSize.
	leftToRead := totalSize.
	[ leftToRead > 0 ]
		whileTrue: [ | readCount |
			readCount := bufferSize min: leftToRead.
			inputStream next: readCount into: buffer.
			leftToRead := leftToRead - readCount.
			outputStream next: readCount putAll: buffer startingAt: 1.
			leftToRead > 0 
				ifTrue: [
					self signalProgress: (totalSize - leftToRead) total: totalSize.
					outputStream flush ] ]! !

!ZnUtils class methodsFor: 'streaming' stamp: 'SvenVanCaekenberghe 9/17/2012 14:27'!
streamingBufferSize
	"Return the byte count used as buffer size when streaming"
	
	^ 16384! !

!ZnUtils class methodsFor: 'conversions' stamp: 'SvenVanCaekenberghe 10/5/2010 21:24'!
trimString: string
	"Until String>>#trimBoth is available everywhere"
	
	| left right |
	left := 1.
	right := string size.
	[ left <= right and: [ (string at: left) isSeparator ] ]
		whileTrue: [ left := left + 1 ].
	[ left <= right and: [ (string at: right) isSeparator ] ]
		whileTrue: [ right := right - 1 ].
	^ string copyFrom: left to: right! !

Object subclass: #ZnValueDelegate
	instanceVariableNames: 'object'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zinc-HTTP-Client-Server'!

!ZnValueDelegate commentStamp: 'SvenVanCaekenberghe 12/7/2010 16:14' prior: 0!
I am ZnValueDelegate.
I forward #handleRequest: messages to the object that I wrap using #value:

ZnServer default delegate: 
	(ZnValueDelegate with: [ :request | 
		ZnResponse ok: (ZnEntity with: 'You asked for ', request uri printString) ] ).

Part of Zinc HTTP Components.!

!ZnValueDelegate class methodsFor: 'instance creation' stamp: 'SvenVanCaekenberghe 3/2/2014 22:36'!
with: object
	^ self new
		object: object;
		yourself! !

!ZnValueDelegate methodsFor: 'public' stamp: 'SvenVanCaekenberghe 12/7/2010 16:16'!
handleRequest: request
	^ object value: request! !

!ZnValueDelegate methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 12/7/2010 16:15'!
object: anObject
	object := anObject! !
ZnConstants initialize!
ZnHeaders initialize!
ZnLogEvent initialize!
ZnNetworkingUtils initialize!
ZnServer initialize!
ZnSingleThreadedServer initialize!
